package utils

import (
	"encoding/csv"
	"golang.org/x/text/encoding/unicode"
	"golang.org/x/text/transform"
	"log"
	"os"
)

func ExportCsv(filePath string, csvSeperator string, data [][]string) {
	fp, err := os.Create(filePath) // 创建文件句柄

	if err != nil {
		log.Fatalf("创建文件["+filePath+"]句柄失败,%v", err)
		return
	}
	defer fp.Close()

	var sep rune
	if csvSeperator == "" {
		sep = rune('|')
	}else{
		runes := []rune(csvSeperator)
		sep = runes[0]
	}

	fp.WriteString("\xEF\xBB\xBF") // 写入UTF-8 BOM
	w := csv.NewWriter(fp) //创建一个新的写入文件流
	w.Comma = sep
	w.WriteAll(data)
	w.Flush()
}

func ExportUnicodeCsv(filePath string, csvSeperator string, data [][]string) {
	fp, err := os.Create(filePath) // 创建文件句柄
	if err != nil {
		log.Fatalf("创建文件["+filePath+"]句柄失败,%v", err)
		return
	}
	defer fp.Close()

	var sep string
	if csvSeperator == "" {
		sep = "|"
	}else{
		sep = csvSeperator
	}

	fp.WriteString("\xFF\xFE")  // 写入UTF-16LE Unicode
	// 获取行字符串
	for _, row := range data {
		var rowbytes  []byte
		for i, cell := range row {
			var cellstring string
			if (i+1) != len(row){
				cellstring = cell + sep
			}else{
				cellstring = cell + "\r\n"
			}

			cellbytes := conv2utf16le(cellstring)
			for _, singlebyte := range cellbytes {
				rowbytes = append(rowbytes, singlebyte)
			}
		}

		fp.Write(rowbytes)
	}

}

// 转换成Unicode 字节码输出编码 utf-16le
func conv2utf16le(data string) []byte {
	bsUtf16le, _, _ := transform.Bytes(unicode.UTF16(unicode.LittleEndian, unicode.IgnoreBOM).NewEncoder(), []byte(data))
	return bsUtf16le
}


// 转换成Unicode 字节码输出编码 utf-16be
func conv2utf16be(data string) []byte {
	bsUtf16be, _, _ := transform.Bytes(unicode.UTF16(unicode.BigEndian, unicode.IgnoreBOM).NewEncoder(), []byte(data))
	return bsUtf16be
}
