package controllers

import (
	"LAPP_SJA_ME/utils"
	"LAPP_SJA_ME/web/middleware/jwts"
	"LAPP_SJA_ME/web/models"
	"LAPP_SJA_ME/web/supports"
	"fmt"
	"github.com/kataras/iris"
	"time"
)

func GetMeAttributeList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_attribute_controller.go"
	log.Level = "info"
	log.Function = "GetMeAttributeList"
	log.Message = "查询属性主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	var data models.MeAttribute
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Finr = user.Pid
	data.Attrcode, _ = ctx.URLParamInt("attrcode")
	data.Attrname = ctx.URLParam("attrname")
	data.Descr = ctx.URLParam("descr")
	result, count, err := data.GetPage(pageSize, pageIndex)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetMeAttribute(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.MeAttribute
	attrcode, _ := ctx.URLParamInt("attrcode")
	me.Finr = user.Pid
	me.Attrcode = attrcode
	result, err := me.SelectOne()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func InsertMeAttribute(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_attribute_controller.go"
	log.Level = "info"
	log.Function = "InsertMeAttribute"
	log.Message = "添加属性主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	data := new(models.MeAttribute)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "json解析错误", nil)
		return
	}
	data.Finr = user.Pid
	data.Lastuser = user.Userid
	data.Credatuz = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	err := data.Add()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, data)
}

func UpdateMeAttribute(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_attribute_controller.go"
	log.Level = "info"
	log.Function = "UpdateMeAttribute"
	log.Message = "更新属性主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	data := new(models.MeAttribute)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "json解析失败", nil)
		return
	}
	data.Lastuser = user.Userid
	data.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	err := data.Update()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}

	supports.Ok(ctx, "修改成功", "")
}

func DeleteMeAttribute(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_attribute_controller.go"
	log.Level = "info"
	log.Function = "DeleteMeAttribute"
	log.Message = "删除属性主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	me := new(models.MeAttribute)
	finr := ctx.URLParam("finr")
	attrcode, _ := ctx.URLParamInt("attrcode")
	me.Finr = utils.ValueToInt(finr, 0)
	me.Attrcode = attrcode
	res := me.Del()
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "删除失败", nil)
		return
	}
	supports.Ok(ctx, "删除成功", "")
}

func SelectArrMeAttribute(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_attribute_controller.go"
	log.Level = "info"
	log.Function = "SelectArrMeAttribute"
	log.Message = "查询属性主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	me := new(models.MeAttribute)
	me.Finr = user.Pid
	res, err := me.SelectAll()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "查询失败", nil)
		return
	}
	supports.Ok(ctx, "成功", res)
}

func SelectArrMeAttributeName(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	me := new(models.MeAttribute)
	me.Finr = user.Pid
	res, err := me.SelectArr()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "查询失败", nil)
		return
	}
	supports.Ok(ctx, "成功", res)
}

func SelectDataMeAttribute(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	me := new(models.MeAttribute)
	me.Finr = user.Pid
	res, err := me.SelectData()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "查询失败", nil)
		return
	}
	supports.Ok(ctx, "成功", res)
}

func DownloadMeAttribute(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	data := new(models.MeAttribute)
	data.Finr = user.Pid
	data.Attrcode, _ = ctx.URLParamInt("attrcode")
	data.Attrname = ctx.URLParam("attrname")
	data.Descr = ctx.URLParam("descr")
	_, filename, err := data.ReadData()
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}
