package controllers

import (
	"LAPP_SJA_ME/utils"
	"LAPP_SJA_ME/web/middleware/jwts"
	"LAPP_SJA_ME/web/models"
	"LAPP_SJA_ME/web/supports"
	"github.com/kataras/iris"
	"time"
)

func GetMeAttributeValstList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var data models.MeAttributeValst
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Finr = user.Pid
	data.Attrcode,_ = ctx.URLParamInt("attrcode")
	result, count, err := data.GetPage(pageSize, pageIndex)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetMeAttributeValst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.MeAttributeValst
	attrcode,_:= ctx.URLParamInt("attrcode")
	pos,_:= ctx.URLParamInt("pos")
	me.Finr = user.Pid
	me.Attrcode = attrcode
	me.Pos = pos
	result, err := me.SelectOne()
	if err !=nil{
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	result.Clipped()
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func InsertMeAttributeValst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	data := new(models.MeAttributeValst)
	data.Lastuser = user.Userid
	data.Credatuz = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	data.Lastmodif = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "json解析错误", nil)
		return
	}
	data.Finr = user.Pid
	err := data.Add()
	if err != nil{
		supports.Error(ctx, iris.StatusBadRequest,"数据已经存在！", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, data)
}

func UpdateMeAttributeValst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	data := new(models.MeAttributeValst)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	data.Lastuser = user.Userid
	data.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	res:= data.Update()
	if !res{
		supports.Error(ctx, iris.StatusBadRequest, "更新失败", nil)
		return
	}

	supports.Ok(ctx, "修改成功", "")
}

func DeleteMeAttributeValst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	me:=new(models.MeAttributeValst)
	finr := ctx.URLParam("finr")
	attrcode,_:= ctx.URLParamInt("attrcode")
	pos,_:= ctx.URLParamInt("pos")
	me.Finr = utils.ValueToInt(finr,0)
	me.Attrcode = attrcode
	me.Pos = pos
	res := me.Del()
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "删除失败", nil)
		return
	}
	supports.Ok(ctx, "删除成功", "")
}

