package controllers

import (
	"LAPP_SJA_ME/utils"
	"LAPP_SJA_ME/web/middleware/jwts"
	"LAPP_SJA_ME/web/models"
	"LAPP_SJA_ME/web/supports"
	"github.com/kataras/iris"
	"time"
)

func GetMeBomlstList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var MeBomlst models.MeBomlst
	MeBomlst.Finr = user.Pid
	MeBomlst.Bomid = ctx.URLParam("bomid")
	result, err := MeBomlst.SetMenu()
	if err != nil{
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}
func GetMeBomlst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.MeBomlst
	me.Finr = user.Pid
	me.Bomid = ctx.URLParam("bomid")
	me.Fmatnr = ctx.URLParam("fmatnr")
	me.Cmatnr = ctx.URLParam("cmatnr")
	result, err := me.SelectOne()
	if err !=nil{
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	result.Clipped()
	supports.Ok(ctx, supports.OptionSuccess, result)
}
func InsertMeBomlst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/workline_controller.go"
	log.Level = "info"
	log.Function = "InsertMeBomlst"
	log.Message = "添加单位"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	log.InsertRecord()
	data := new(models.MeBomlst)
	data.Lastuser = user.Userid
	data.Credatuz = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	data.Lastmodif = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	if err := ctx.ReadJSON(data); err != nil {
		log := new(models.LeitServerLog)
		log.File = "/controllers/MeBomlst_controller.go"
		log.Level = "debug"
		log.Function = "InsertMeBomlst"
		log.Message = error.Error(err)
		log.Operator = user.Userid
		log.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
		log.InsertRecord()
		supports.Error(ctx, iris.StatusBadRequest, "json解析错误", nil)
		return
	}
	data.Finr = user.Pid
	if utils.ValueIsEmpty(data.Bomid) || utils.ValueIsEmpty(data.Finr) || utils.ValueIsEmpty(data.Fmatnr) || utils.ValueIsEmpty(data.Cmatnr){
		supports.Error(ctx, iris.StatusBadRequest, "参数缺失!", nil)
		return
	}
	err := data.Add()
	if err != nil{
		supports.Error(ctx, iris.StatusBadRequest,err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, data)
}

func UpdateMeBomlst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/MeBomlst_controller.go"
	log.Level = "info"
	log.Function = "UpdateMeBomlst"
	log.Message = "更新单位"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	log.InsertRecord()
	data := new(models.MeBomlst)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	data.Lastuser = user.Userid
	data.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Finr = user.Pid
	res:= data.Update()
	if !res{
		supports.Error(ctx, iris.StatusBadRequest, "更新失败", nil)
		return
	}

	supports.Ok(ctx, "修改成功", "")
}

func DeleteMeBomlst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/MeBomlst_controller.go"
	log.Level = "info"
	log.Function = "DeleteMeBomlst"
	log.Message = "删除"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	log.InsertRecord()
	me:=new(models.MeBomlst)
	me.Finr = user.Pid
	me.Bomid = ctx.URLParam("bomid")
	me.Fmatnr = ctx.URLParam("fmatnr")
	me.Cmatnr = ctx.URLParam("cmatnr")
	res := me.Del()
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "删除失败", nil)
		return
	}
	supports.Ok(ctx, "删除成功", "")
}

