package controllers

import (
	"LAPP_SJA_ME/utils"
	"LAPP_SJA_ME/web/middleware/jwts"
	"LAPP_SJA_ME/web/models"
	"LAPP_SJA_ME/web/supports"
	"github.com/kataras/iris"
	"sync"
	"time"
)

//锁
var l sync.Mutex

//显示界面
func GetMePartAtcodlstList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_part_atcodlst_controller.go"
	log.Level = "info"
	log.Function = "GetMePartAtcodlstList"
	log.Message = "查询零件属性数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	var data models.MePartAtcodlst
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}

	//var modelParts []string
	var modelAttrs []int
	var parts []string
	partgroupid := ctx.URLParam("partgroupid")
	if partgroupid != "" {
		//根据模块查询属性组
		modelAttr := new(models.MePartgroupAttrgrplst)
		modelAttr.Finr = user.Pid
		modelAttr.Partgroupid = partgroupid
		modelAttrs, _ = modelAttr.SelectAttrgroups()
	}

	if partid := ctx.URLParam("partid"); partid != "" {
		data.Partid = partid
	}
	attrcode := ctx.URLParam("attrcode")
	projectid := ctx.URLParam("projectid")
	attrval := ctx.URLParam("attrvalue")
	ishide, _ := ctx.URLParamBool("isHide")

	version, err := ctx.URLParamInt("version")
	if version <= 0 || err != nil {
		ver := new(models.MePartAtcodver)
		ver.Finr = user.Pid
		version, err = ver.SelectLastVersion()

		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "版本号不能为空", nil)
			return
		}
	}
	if attrcode != "" || attrval != "" {
		//根据属性名和属性组查询
		attrPart := new(models.MePartAtcodlst)
		attrPart.Version = version
		attrPart.Attrcode = utils.ValueToInt(attrcode, 0)
		attrPart.Attrvalue = attrval
		parts, _ = attrPart.SelectParts()
		if utils.ValueIsEmpty(parts) {
			parts = append(parts, "NoResult")
		}
	}
	data.Finr = user.Pid
	data.Version = version
	result, count, err := data.GetPage(pageSize, pageIndex, partgroupid, parts, modelAttrs, ishide, projectid)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	//查询当前版本信息
	me := new(models.MePartAtcodver)
	me.Finr = user.Pid
	me.Version = version
	verdata, _ := me.SelectOne()
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["version"] = verdata
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

//保存
func SaveMePartAtcodlst(ctx iris.Context) {
	l.Lock()
	defer l.Unlock()
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_part_atcodlst_controller.go"
	log.Level = "info"
	log.Function = "SaveMePartAtcodlst"
	log.Message = "保存零件属性数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	//解析json
	var datalist map[string]interface{}
	if err := ctx.ReadJSON(&datalist); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "json解析失败", nil)
		return
	}
	verval, ok := datalist["version"]
	version := utils.ValueToInt(verval, 0)

	if version <= 0 || !ok {
		supports.Error(ctx, iris.StatusBadRequest, "版本号不能为空", nil)
		return

	}
	data, ok := datalist["data"]
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, "值不能为空", nil)
		return

	}
	//查询出所有的属性
	var attr models.MeAttribute
	attr.Finr = user.Pid
	arr, err := attr.SelectData()

	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "属性列表为空！", nil)
		return
	}
	//转义为k=>V形式
	tem := map[string]int{}
	for _, v := range arr {
		key := v.Attrname
		tem[key] = v.Attrcode
	}
	Lastuser := user.Userid
	Credatuz := utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	Lastmodif := utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	//获取更新前的信息
	Adata := make([]models.MePartAtcodlst, 0)
	for _, v := range data.([]interface{}) {
		valv := v.(map[string]interface{})
		//初始化对象
		selectData := new(models.MePartAtcodlst)
		selectData.Finr = user.Pid
		selectData.Version = version
		partid, ok := valv["JCI part NO."]
		if ok {
			selectData.Partid = utils.ValueToString(partid, "")
		}
		projnr, ok := valv["Project No"]
		if ok {
			selectData.Projnr = utils.ValueToString(projnr, "")
		}
		arr, _ := selectData.SelectDataVal()
		for _, vv := range arr {
			Adata = append(Adata, vv)
		}
	}
	//记录更新后的信息
	Bdata := make([]models.MePartAtcodlst, 0)
	for _, v := range data.([]interface{}) {
		valv := v.(map[string]interface{})
		//初始化对象
		insertData := new(models.MePartAtcodlst)
		insertData.Finr = user.Pid
		insertData.Version = version
		partid, ok := valv["JCI part NO."]
		if ok {
			insertData.Partid = utils.ValueToString(partid, "")
		}
		projnr, ok := valv["Project No"]
		if ok {
			insertData.Projnr = utils.ValueToString(projnr, "")
		}
		//删除数据
		delMePartAtcodlst := new(models.MePartAtcodlst)
		delMePartAtcodlst.Finr = user.Pid
		delMePartAtcodlst.Version = version
		delMePartAtcodlst.Partid = utils.ValueToString(partid, "")
		delMePartAtcodlst.Projnr = utils.ValueToString(projnr, "")
		delMePartAtcodlst.Del()
		//保存数据
		val, ok := valv["part"]
		//判断值是否存在
		if ok {
			//强制类型转换，否则报错
			for kk, vv := range val.(map[string]interface{}) {
				_, ok := tem[kk]
				if ok {
					if utils.ValueIsEmpty(vv) {
						continue
					}
					insertData.Attrcode = tem[kk]
					insertData.Attrvalue = utils.ValueToString(vv, "")
					insertData.Lastuser = Lastuser
					insertData.Lastmodif = Lastmodif
					insertData.Credatuz = Credatuz
					Bdata = append(Bdata, *insertData)
					err = insertData.Save()
					if err != nil {
						continue
					}
				}

			}

		}
	}
	//调用零件分配记录方法
	m := new(models.MePartAtcodverTrace)
	m.Lastuser = Lastuser
	m.Credatuz = Credatuz
	m.Lastmodif = Lastmodif
	m.Version = version
	m.SelectAdd(Adata, Bdata)
	supports.Ok(ctx, supports.OptionSuccess, data)
}

func DeleteMePartAtcodlst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_part_atcodlst_controller.go"
	log.Level = "info"
	log.Function = "DeleteMePartAtcodlst"
	log.Message = "删除零件属性数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()

	me := new(models.MePartAtcodlst)
	finr := user.Pid
	partid := ctx.URLParam("partid")
	projnr := ctx.URLParam("projnr")
	version := ctx.URLParam("version")
	me.Finr = utils.ValueToInt(finr, 0)
	me.Version = utils.ValueToInt(version, 0)
	me.Partid = partid
	me.Projnr = utils.ValueToString(projnr, "")
	res := me.Del()
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "删除失败", nil)
		return
	}
	supports.Ok(ctx, "删除成功", "")
}

//上传excel零件属性表
//上传excel
func UpExcelMePartAtcodlst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_part_atcodlst_controller.go"
	log.Level = "info"
	log.Function = "UpExcelMePartAtcodlst"
	log.Message = "上传零件属性excel表格"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()

	//获取通过iris.WithPostMaxMemory获取的最大上传值大小。
	maxSize := ctx.Application().ConfigurationReadOnly().GetPostMaxMemory()
	err := ctx.Request().ParseMultipartForm(maxSize)
	if err != nil {
		ctx.StatusCode(iris.StatusInternalServerError)
		ctx.WriteString(err.Error())
		return
	}
	form := ctx.Request().MultipartForm
	files := form.File["upload[]"]

	guid := utils.MakeOrderSn("MePart")
	savePath := "web/public/uploadxlsx/" + guid + ".xlsx"

	for _, file := range files {
		// 上传文件至指定目录
		err := utils.SaveUploadedFile(file, savePath)
		if err != nil {
			return
		}
	}
	//生成新的版本
	data := new(models.MePartAtcodver)
	data.Finr = user.Pid
	data.Lastuser = user.Userid
	data.Released = 0
	data.Descr = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Credatuz = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	version, err := data.AddNew()
	tab := new(models.MePartAtcodlst)
	tab.Finr = user.Pid
	tab.Lastuser = user.Userid
	tab.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	tab.Credatuz = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	tab.Version = int(version)
	res := tab.ReadXlsx(savePath)
	if res {
		supports.Ok(ctx, supports.OptionSuccess, "")
		return
	}
	supports.Error(ctx, iris.StatusBadRequest, "上传失败", nil)
	return
}

//查询所有的模块名
func SelectModelsArr(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	mod := new(models.MePartgroup)
	mod.Finr = user.Pid
	mod.Projnr = ctx.URLParam("projnr")
	data, _ := mod.SelectData()
	supports.Ok(ctx, supports.OptionSuccess, data)
	return
}

//查询所有的属性名
func SelectAttrsArr(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	mod := new(models.MeAttribute)
	mod.Finr = user.Pid
	data, _ := mod.SelectData()
	supports.Ok(ctx, supports.OptionSuccess, data)
	return
}

//根据属性名，查询所有的属性值
func SelectAttrVals(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	attrcode := ctx.URLParam("attrcode")
	mod := new(models.MeAttributeValst)
	mod.Finr = user.Pid
	mod.Attrcode = utils.ValueToInt(attrcode, 0)
	data, _ := mod.SelectVal()
	supports.Ok(ctx, supports.OptionSuccess, data)
	return
}
