package controllers

import (
	"LAPP_SJA_ME/utils"
	"LAPP_SJA_ME/web/middleware/jwts"
	"LAPP_SJA_ME/web/models"
	"LAPP_SJA_ME/web/supports"
	"github.com/kataras/iris"
	"time"
)

func GetMePartgroupList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_partgroup_controller.go"
	log.Level = "info"
	log.Function = "GetMePartgroupList"
	log.Message = "查询模块主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	var data models.MePartgroup
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Finr = user.Pid
	data.Partgroupid = ctx.URLParam("partgroupid")
	data.Projnr = ctx.URLParam("projnr")
	data.Descr = ctx.URLParam("descr")
	result, count, err := data.GetPage(pageSize, pageIndex)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetMePartgroup(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.MePartgroup
	partgroupid := ctx.URLParam("partgroupid")
	projnr := ctx.URLParam("projnr")
	me.Finr = user.Pid
	me.Partgroupid = partgroupid
	me.Projnr = projnr
	result, err := me.SelectOne()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	result.Clipped()
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func InsertMePartgroup(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_partgroup_controller.go"
	log.Level = "info"
	log.Function = "InsertMePartgroup"
	log.Message = "添加模块主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	data := new(models.MePartgroup)
	data.Finr = user.Pid
	data.Lastuser = user.Userid
	data.Credatuz = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "json解析错误", nil)
		return
	}
	err := data.Add()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "数据已经存在！", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, data)
}

func UpdateMePartgroup(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_partgroup_controller.go"
	log.Level = "info"
	log.Function = "UpdateMePartgroup"
	log.Message = "更新模块主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	data := new(models.MePartgroup)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	data.Lastuser = user.Userid
	data.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	res := data.Update()
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "更新失败", nil)
		return
	}

	supports.Ok(ctx, "修改成功", "")
}

func DeleteMePartgroup(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_partgroup_controller.go"
	log.Level = "info"
	log.Function = "DeleteMePartgroup"
	log.Message = "删除模块主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	me := new(models.MePartgroup)
	partgroupid := ctx.URLParam("partgroupid")
	projnr := ctx.URLParam("projnr")
	me.Finr = user.Pid
	me.Projnr = projnr
	me.Partgroupid = partgroupid
	res := me.Del()
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "删除失败", nil)
		return
	}
	supports.Ok(ctx, "删除成功", "")
}

func SelectInfoPartGroup(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_partgroup_controller.go"
	log.Level = "info"
	log.Function = "SelectInfoMeGroup"
	log.Message = "查询属性组主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	me := new(models.MePartgroupPartlst)
	partgroupid := ctx.URLParam("partgroupid")
	projnr := ctx.URLParam("projnr")
	me.Finr = user.Pid
	me.Partgroupid = partgroupid
	me.Projnr = projnr
	res, err := me.SelectPartGroup()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "查询失败", nil)
		return
	}
	supports.Ok(ctx, "成功", res)
}

func SelectInfoAttrGroup(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_partgroup_controller.go"
	log.Level = "info"
	log.Function = "SelectInfoAttrGroup"
	log.Message = "查询属性组主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	me := new(models.MePartgroupAttrgrplst)
	Partgroupid := ctx.URLParam("partgroupid")
	projnr := ctx.URLParam("projnr")
	me.Finr = user.Pid
	me.Partgroupid = Partgroupid
	res, err := me.SelectAttrGroup(projnr)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "查询失败", nil)
		return
	}
	supports.Ok(ctx, "成功", res)
}

//保存模块零件
func SavePartGroup(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_partgroup_controller.go"
	log.Level = "info"
	log.Function = "SavePartGroup"
	log.Message = "保存零件模块"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	data := new(models.MePartgroup)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	projnr := ctx.URLParam("projnr")
	me := new(models.MePartgroupPartlst)
	me.Lastuser = user.Userid
	me.Credatuz = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	me.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	me.Finr = user.Pid
	me.Projnr = projnr
	res := me.SavePartGroup(data)
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "保存失败", nil)
		return
	}

	supports.Ok(ctx, "保存成功", "")
}

//保存模块零件
func SaveAttrGroup(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_partgroup_controller.go"
	log.Level = "info"
	log.Function = "SavePartGroup"
	log.Message = "保存零件模块"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	data := new(models.MePartgroup)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	me := new(models.MePartgroupAttrgrplst)
	me.Lastuser = user.Userid
	me.Credatuz = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	me.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	me.Finr = user.Pid
	res := me.SaveAttrGroup(data)
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "保存失败", nil)
		return
	}

	supports.Ok(ctx, "保存成功", "")
}
