package controllers

import (
	"LAPP_SJA_ME/utils"
	"LAPP_SJA_ME/web/middleware/glog"
	"LAPP_SJA_ME/web/middleware/jwts"
	"LAPP_SJA_ME/web/models"
	"LAPP_SJA_ME/web/supports"
	"fmt"
	"github.com/kataras/iris"
	"time"
)

func GetMePartruleList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_partrule_controller.go"
	log.Level = "info"
	log.Function = "GetMePartRuleList"
	log.Message = "查询零件数量分配"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	var data models.MePartRule
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Finr = user.Pid
	data.Partid = ctx.URLParam("partid")
	data.Projnr = ctx.URLParam("projnr")
	data.Ruleid = ctx.URLParam("ruleid")
	data.Descr = ctx.URLParam("descr")
	result, count, err := data.GetPage(pageSize, pageIndex)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetMePartrule(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.MePartRule
	me.Finr = user.Pid
	me.Partid = ctx.URLParam("partid")
	me.Projnr = ctx.URLParam("projnr")
	me.Ruleid = ctx.URLParam("ruleid")
	result, err := me.SelectOne()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func InsertMePartrule(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/me_partrule_controller.go"
	logs.Level = "info"
	logs.Function = "InsertMePartRule"
	logs.Message = "添加零件分配数据"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	data := new(models.MePartRule)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "json解析错误", nil)
		return
	}
	data.Lastuser = user.Userid
	data.Credatuz = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Finr = user.Pid
	err := data.Add()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "数据已经存在！", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, data)
}

func UpdateMePartrule(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/me_partrule_controller.go"
	logs.Level = "info"
	logs.Function = "UpdateMePartRule"
	logs.Message = "更新零件分配数据"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	data := new(models.MePartRule)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "json解析失败", nil)
		return
	}
	data.Lastuser = user.Userid
	data.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	res := data.Update()
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "更新失败", nil)
		return
	}

	supports.Ok(ctx, "修改成功", "")
}

func DeleteMePartrule(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/me_partrule_controller.go"
	logs.Level = "info"
	logs.Function = "DeleteMePartRule"
	logs.Message = "删除零件分配数据"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	me := new(models.MePartRule)
	finr := ctx.URLParam("finr")
	me.Finr = utils.ValueToInt(finr, 0)
	me.Partid = ctx.URLParam("partid")
	me.Projnr = ctx.URLParam("projnr")
	me.Ruleid = ctx.URLParam("ruleid")
	res := me.Del()
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "删除失败", nil)
		return
	}
	supports.Ok(ctx, "删除成功", "")
}

func SelectArrMePartrule(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/me_partrule_controller.go"
	logs.Level = "info"
	logs.Function = "SelectArrMePartRule"
	logs.Message = "查询未分配供应组"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	me := new(models.MePartRule)
	me.Finr = user.Pid
	me.Partid = ctx.URLParam("partid")
	me.Projnr = ctx.URLParam("projnr")
	me.Ruleid = ctx.URLParam("ruleid")
	me.PartruleCprecond = ctx.URLParam("partrule_cprecond")
	res, err := me.SelectVal()
	if err != nil {
		glog.InfoExt("错误日志", "查询未分配供应组:", err)
		supports.Error(ctx, iris.StatusBadRequest, "查询失败", nil)
		return
	}
	supports.Ok(ctx, "成功", res)
}

func DownloadMePartrule(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	data := new(models.MePartRule)
	data.Finr = user.Pid
	data.Partid = ctx.URLParam("partid")
	data.Projnr = ctx.URLParam("projnr")
	data.Ruleid = ctx.URLParam("ruleid")
	data.Descr = ctx.URLParam("descr")
	_, filename, err := data.ReadData()
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}
