package controllers

import (
	"LAPP_SJA_ME/utils"
	"LAPP_SJA_ME/web/middleware/glog"
	"LAPP_SJA_ME/web/middleware/jwts"
	"LAPP_SJA_ME/web/models"
	"LAPP_SJA_ME/web/supports"
	"github.com/kataras/iris"
	"time"
)

func GetMeSupplyGroupList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_supplygroup_controller.go"
	log.Level = "info"
	log.Function = "GetMeSupplyGroupList"
	log.Message = "查询供应主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	log.InsertRecord()
	var data models.MeSupplyGroup
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Finr = user.Pid
	data.Supplygroupid = ctx.URLParam("supplygroupid")
	result, count, err := data.GetPage(pageSize, pageIndex)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetMeSupplyGroup(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.MeSupplyGroup
	supplygroupid := ctx.URLParam("supplygroupid")
	me.Finr =user.Pid
	me.Supplygroupid = supplygroupid
	result, err := me.SelectOne()
	if err !=nil{
		glog.InfoExt("错误日志","GetMeSupplyGroup:",err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func InsertMeSupplyGroup(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_supplygroup_controller.go"
	log.Level = "info"
	log.Function = "InsertMeSupplyGroup"
	log.Message = "添加供应主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	log.InsertRecord()
	data := new(models.MeSupplyGroup)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "json解析错误", nil)
		return
	}
	data.Finr = user.Pid
	data.Lastuser = user.Userid
	data.Credatuz = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	data.Lastmodif = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	err := data.Add()
	if err != nil{
		supports.Error(ctx, iris.StatusBadRequest,"添加失败！", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, data)
}

func UpdateMeSupplyGroup(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_supplygroup_controller.go"
	log.Level = "info"
	log.Function = "UpdateMeSupplyGroup"
	log.Message = "更新供应主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	log.InsertRecord()
	data := new(models.MeSupplyGroup)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "json解析失败", nil)
		return
	}
	data.Lastuser = user.Userid
	data.Finr = user.Pid
	data.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	res:= data.Update()
	if !res{
		supports.Error(ctx, iris.StatusBadRequest, "更新失败", nil)
		return
	}

	supports.Ok(ctx, "修改成功", "")
}

func DeleteMeSupplyGroup(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/me_supplygroup_controller.go"
	log.Level = "info"
	log.Function = "DeleteMeSupplyGroup"
	log.Message = "删除供应主数据"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	log.InsertRecord()
	me:=new(models.MeSupplyGroup)
	supplygroupid := ctx.URLParam("supplygroupid")
	me.Supplygroupid = supplygroupid
	me.Finr = user.Pid
	res := me.Del()
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "删除失败", nil)
		return
	}
	supports.Ok(ctx, "删除成功", "")
}

func SelectArrMeSupplyGroup(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	me:=new(models.MeSupplyGroup)
	me.Finr = user.Pid
	result,err := me.SelectUnArr()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "数据为空", nil)
		return
	}
	supports.Ok(ctx, "成功", result)
}