package controllers

import (
	"LAPP_SJA_ME/utils"
	"LAPP_SJA_ME/web/middleware/glog"
	"LAPP_SJA_ME/web/middleware/jwts"
	"LAPP_SJA_ME/web/models"
	"LAPP_SJA_ME/web/supports"
	"github.com/kataras/iris"
	"time"
)

func GetPlnForecastCalendarList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastCalendar_controller.go"
	log.Level = "info"
	log.Function = "GetPlnForecastCalendarList"
	log.Message = "工作日历查看列表"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	log.InsertRecord()
	var data models.PlnForecastCalendar
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Finr = user.Pid
	data.Calendarid = ctx.URLParam("calendarid")
	result, count, err := data.GetPage(pageSize, pageIndex)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetPlnForecastCalendar(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	me := new(models.PlnForecastCalendar)
	calendarid:= ctx.URLParam("calendarid")
	me.Finr = user.Pid
	me.Calendarid = calendarid
	result, err := me.SelectOne()
	if err !=nil{

		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	result.Clipped()
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func InsertPlnForecastCalendar(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastCalendar_controller.go"
	log.Level = "info"
	log.Function = "InsertPlnForecastCalendar"
	log.Message = "添加工作日历"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	log.InsertRecord()
	data := new(models.PlnForecastCalendar)
	data.Lastuser = user.Userid
	data.Credatuz = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	data.Lastmodif = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	if err := ctx.ReadJSON(data); err != nil {
		log := new(models.LeitServerLog)
		log.File = "/controllers/PlnForecastCalendar_controller.go"
		log.Level = "debug"
		log.Function = "InsertPlnForecastCalendar"
		log.Message = error.Error(err)
		log.Operator = user.Userid
		log.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
		log.InsertRecord()
		supports.Error(ctx, iris.StatusBadRequest, "json解析错误", nil)
		return
	}
	data.Finr = user.Pid
	err := data.Add()
	if err != nil{
		supports.Error(ctx, iris.StatusBadRequest,"添加失败！", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, data)
}

func UpdatePlnForecastCalendar(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastCalendar_controller.go"
	log.Level = "info"
	log.Function = "UpdatePlnForecastCalendar"
	log.Message = "更新工作日历"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	log.InsertRecord()
	data := new(models.PlnForecastCalendar)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	data.Lastuser = user.Userid
	data.Finr = user.Pid
	data.Lastmodif = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	err := data.Update()
	if err != nil{
		glog.InfoExtln("工作日历更新失败","失败原因:",err)
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}

	supports.Ok(ctx, "修改成功", "")
}

func DeletePlnForecastCalendar(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastCalendar_controller.go"
	log.Level = "info"
	log.Function = "DeletePlnForecastCalendar"
	log.Message = "删除工作日历"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	log.InsertRecord()
	me:=new(models.PlnForecastCalendar)
	calendarid:= ctx.URLParam("calendarid")
	me.Finr = user.Pid
	me.Calendarid = calendarid
	res := me.Del()
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "删除失败", nil)
		return
	}
	supports.Ok(ctx, "删除成功", "")
}