package controllers

import (
	"LAPP_SJA_ME/utils"
	"LAPP_SJA_ME/web/middleware/jwts"
	"LAPP_SJA_ME/web/models"
	"LAPP_SJA_ME/web/supports"
	"fmt"
	"github.com/kataras/iris"
	"strings"
	"time"
)

func GetPlnForecastItemDailydemandList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastItemDailydemand_controller.go"
	log.Level = "info"
	log.Function = "GetPlnForecastItemDailydemandList"
	log.Message = "产品计划查看列表"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	var data models.PlnForecastItemDailydemand
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}

	data.Finr = user.Pid
	data.Artnr = ctx.URLParam("artnr")
	data.Forecastid = ctx.URLParam("forecastid")
	data.Demandyear, _ = ctx.URLParamInt("demandyear")
	data.Demandmonth, _ = ctx.URLParamInt("demandmonth")
	timedate := ctx.URLParam("monthRange")
	articles := ctx.URLParam("article")
	result, count, err := data.GetPage(pageSize, pageIndex, timedate,articles)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetPlnForecastItemDailydemandByDayList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastItemDailydemand_controller.go"
	log.Level = "info"
	log.Function = "GetPlnForecastItemDailydemandByDayList"
	log.Message = "产品计划日报查看列表"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	var data models.PlnForecastItemDailydemand
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}

	data.Finr = user.Pid
	data.Artnr = ctx.URLParam("artnr")
	data.Forecastid = ctx.URLParam("forecastid")
	data.Demandyear, _ = ctx.URLParamInt("demandyear")
	data.Demandmonth, _ = ctx.URLParamInt("demandmonth")
	timedate := ctx.URLParam("dateRange")
	articles := ctx.URLParam("article")
	result, count, err := data.GetDayPage(pageSize, pageIndex, timedate,articles)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetDendrogram(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastItemDailydemand_controller.go"
	log.Level = "info"
	log.Function = "GetDendrogram"
	log.Message = "产品树状图对比"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()

	//查询产品详情
	info := new(models.PlnForecastArticle)
	info.Finr = user.Pid
	info.Artnr = ctx.URLParam("artnr")
	artnrinfo, err := info.SelectOne()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "商品信息不存在！", nil)
		return
	}
	var data models.PlnForecastItemDailydemand

	data.Finr = user.Pid
	data.Artnr = ctx.URLParam("artnr")
	data.Demandyear, _ = ctx.URLParamInt("demandyear")
	forecastidstr := ctx.URLParam("forecastids")
	forecastids := strings.Split(forecastidstr, ",")
	fmt.Println(forecastids)
	timedate := ctx.URLParam("monthRange")
	result, err := data.Dendrogram(forecastids, timedate)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["info"] = artnrinfo
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetPlnForecastItemDailydemandTotal(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastItemDailydemand_controller.go"
	log.Level = "info"
	log.Function = "GetPlnForecastItemDailydemandTotal"
	log.Message = "报表"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()

	var data models.PlnForecastItemDailydemand

	data.Finr = user.Pid
	data.Demandyear, _ = ctx.URLParamInt("demandyear")
	data.Forecastid = ctx.URLParam("forecastid")
	articles := ctx.URLParam("article")
	result, err := data.TotalToDealWith(articles)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func TotalToCustplantnr(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastItemDailydemand_controller.go"
	log.Level = "info"
	log.Function = "TotalToCustplantnr"
	log.Message = "报表"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()

	var data models.PlnForecastItemDailydemand

	data.Finr = user.Pid
	data.Demandyear, _ = ctx.URLParamInt("demandyear")
	data.Forecastid = ctx.URLParam("forecastid")
	timedate := ctx.URLParam("monthRange")
	articles := ctx.URLParam("article")
	result, err := data.TotalToCustplantnr(timedate,articles)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	supports.Ok(ctx, supports.OptionSuccess, res)
}

//导出月报
func ReadExcelMonthdemand(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastItemDailydemand_controller.go"
	log.Level = "info"
	log.Function = "ReadExcelDailydemand"
	log.Message = "报表"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()

	var data models.PlnForecastItemDailydemand

	data.Finr = user.Pid
	data.Demandyear, _ = ctx.URLParamInt("demandyear")
	data.Forecastid = ctx.URLParam("forecastid")
	timedate := ctx.URLParam("monthRange")
	articles := ctx.URLParam("article")
	_, filename, err := data.ReadExcelMonthdemand(timedate,articles)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}

//导出日报
func ReadExcelDaydemand(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastItemDailydemand_controller.go"
	log.Level = "info"
	log.Function = "ReadExcelDaydemand"
	log.Message = "日报表"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()

	var data models.PlnForecastItemDailydemand

	data.Finr = user.Pid
	data.Demandyear, _ = ctx.URLParamInt("demandyear")
	data.Forecastid = ctx.URLParam("forecastid")
	timedate := ctx.URLParam("dateRange")
	articles := ctx.URLParam("article")
	_, filename, err := data.ReadExcelDaydemand(timedate,articles)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}

//导出预测报表
func ReadExcelDemand(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastItemDailydemand_controller.go"
	log.Level = "info"
	log.Function = "ReadExcelDemand"
	log.Message = "预测报表"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()

	var data models.PlnForecastItemDailydemand

	data.Finr = user.Pid
	data.Demandyear, _ = ctx.URLParamInt("demandyear")
	data.Forecastid = ctx.URLParam("forecastid")
	articles := ctx.URLParam("article")
	_, filename, err := data.ReadExcelDemand(articles)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}

//导出年报
func ReadYearExcelDemand(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastItemDailydemand_controller.go"
	log.Level = "info"
	log.Function = "ReadYearExcelDemand"
	log.Message = "导出报表"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()

	var data models.PlnForecastItemDailydemand

	data.Finr = user.Pid
	data.Demandyear, _ = ctx.URLParamInt("demandyear")
	data.Forecastid = ctx.URLParam("forecastid")
	timedate := ctx.URLParam("monthRange")
	articles := ctx.URLParam("article")
	_, filename, err := data.ReadYearExcelDemand(timedate,articles)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}
