package controllers

import (
	"LAPP_SJA_ME/utils"
	"LAPP_SJA_ME/web/middleware/glog"
	"LAPP_SJA_ME/web/middleware/jwts"
	"LAPP_SJA_ME/web/models"
	"LAPP_SJA_ME/web/supports"
	"fmt"
	"github.com/kataras/iris"
	"log"
	"time"
)

func GetPlnForecastLandingList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/PlnForecastLanding_controller.go"
	log.Level = "info"
	log.Function = "GetPlnForecastLandingList"
	log.Message = "导入文件查看列表"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	var data models.PlnForecastLanding
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Finr = user.Pid
	data.Forecastid = ctx.URLParam("forecastid")
	result, count, err := data.GetPage(pageSize, pageIndex)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func ParseInfoPlnForecast(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/PlnForecastLanding_controller.go"
	logs.Level = "info"
	logs.Function = "ParseInfoPlnForecast"
	logs.Message = "生产计划解析"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()

	data := new(models.PlnForecastLanding)
	data.Finr = user.Pid
	data.Forecastid = ctx.URLParam("forecastid")
	data.Serialnumber = ctx.URLParam("serialnumber")
	data.Custplantnr = ctx.URLParam("custplantnr")
	data.Lastuser = user.Userid
	data.Credatuz = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	if utils.ValueIsEmpty(data.Forecastid) || utils.ValueIsEmpty(data.Custplantnr) {
		supports.Error(ctx, iris.StatusBadRequest, "参数缺失！", nil)
		return
	}
	err := data.ParseInfo()
	if err != nil {
		log.Printf("解析失败原因:%v", err)
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, "")
}

func ResetParseInfoPlnForecast(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/PlnForecastLanding_controller.go"
	logs.Level = "info"
	logs.Function = "ResetParseInfoPlnForecast"
	logs.Message = "生产计划重新解析"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	var data models.PlnForecastLanding
	var err error
	data.Finr = user.Pid
	data.Forecastid = ctx.URLParam("forecastid")
	data.Serialnumber = ctx.URLParam("serialnumber")
	data.Custplantnr = ctx.URLParam("custplantnr")
	data.Lastuser = user.Userid
	data.Credatuz = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	if utils.ValueIsEmpty(data.Forecastid) || utils.ValueIsEmpty(data.Custplantnr) {
		supports.Error(ctx, iris.StatusBadRequest, "参数缺失！", nil)
		return
	}
	err = data.ResetParseInfo()
	if err != nil {
		glog.InfoExt("重新解析", "解析失败原因 err is :", err)
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, "")
}

func SelectPlnForecastAll(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	me := new(models.PlnForecastLanding)
	me.Finr = user.Pid
	data, err := me.SelectAll()
	if err !=nil{
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, "成功", data)
}

func DelPlnForecast(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	me := new(models.PlnForecastLanding)
	me.Finr = user.Pid
	me.Forecastid = ctx.URLParam("forecastid")
	me.Serialnumber = ctx.URLParam("serialnumber")
	me.Custplantnr = ctx.URLParam("custplantnr")
	err := me.Del()
	if err != nil {
		fmt.Printf("错误信息err:%v", err)
		supports.Error(ctx, iris.StatusBadRequest, "删除失败！", nil)
		return
	}
	supports.Ok(ctx, "成功", "")
}
