package controllers

import (
	"LAPP_SJA_ME/utils"
	"LAPP_SJA_ME/web/middleware/jwts"
	"LAPP_SJA_ME/web/models"
	"LAPP_SJA_ME/web/supports"
	"github.com/kataras/iris"
	"time"
)

func GetPrintheadtabList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/printheadtab_controller.go"
	logs.Level = "info"
	logs.Function = "GetPrintheadtabList"
	logs.Message = "打印任务列表"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	logs.InsertRecord()
	var data models.Printheadtab
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Finr = user.Pid
	data.Printheadid = ctx.URLParam("printheadid")
	result, count, err := data.GetPage(pageSize, pageIndex)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetPrintheadtab(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Printheadtab
	me.Finr = user.Pid
	me.Printheadid = ctx.URLParam("printheadid")
	result, err := me.SelectOne()
	if err !=nil{
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	result.Clipped()
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func InsertPrintheadtab(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/printheadtab_controller.go"
	logs.Level = "info"
	logs.Function = "InsertPrintheadtab"
	logs.Message = "添加任务"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	logs.InsertRecord()
	data := new(models.Printheadtab)
	data.Lastuser = user.Userid
	data.Credatuz = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	data.Lastmodif = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	if err := ctx.ReadJSON(data); err != nil {
		logs := new(models.LeitServerLog)
		logs.File = "/controllers/printheadtab_controller.go"
		logs.Level = "debug"
		logs.Function = "InsertPrintheadtab"
		logs.Message = error.Error(err)
		logs.Operator = user.Userid
		logs.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
		logs.InsertRecord()
		supports.Error(ctx, iris.StatusBadRequest, "json解析错误", nil)
		return
	}
	data.Finr = user.Pid
	err := data.Add()
	if err != nil{
		supports.Error(ctx, iris.StatusBadRequest,"添加失败！", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, data)
}

func UpdatePrintheadtab(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/printheadtab_controller.go"
	logs.Level = "info"
	logs.Function = "UpdatePrintheadtab"
	logs.Message = "更新任务"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	logs.InsertRecord()
	data := new(models.Printheadtab)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	data.Lastuser = user.Userid
	data.Lastmodif = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	res:= data.Update()
	if !res{
		supports.Error(ctx, iris.StatusBadRequest, "更新失败", nil)
		return
	}

	supports.Ok(ctx, "修改成功", "")
}

func DeletePrintheadtab(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user,*user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/printheadtab_controller.go"
	logs.Level = "info"
	logs.Function = "DeletePrintheadtab"
	logs.Message = "删除任务"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(),"yyyyMMddHHmmss")
	logs.InsertRecord()
	me:=new(models.Printheadtab)
	finr := ctx.URLParam("finr")
	me.Finr = utils.ValueToInt(finr,0)
	me.Printheadid = ctx.URLParam("printheadid")
	res := me.Del()
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "删除失败", nil)
		return
	}
	supports.Ok(ctx, "删除成功", "")
}

