package casbins

import (
	"LAPP_SJA_ME/conf"
	"LAPP_SJA_ME/utils"
	"LAPP_SJA_ME/web/middleware/jwts"
	"LAPP_SJA_ME/web/models"
	"LAPP_SJA_ME/web/supports"
	"fmt"
	"log"
	"net/http"
	"strings"
	"time"
	"github.com/casbin/casbin/v2"
	xormadapter "github.com/casbin/xorm-adapter/v2"
	_ "github.com/denisenkom/go-mssqldb"
	"github.com/kataras/iris/context"
)

// 获取Enforcer
func NewCasbin() (*casbin.Enforcer, error) {
	driveSource := fmt.Sprintf("server=%s;database=%s;user id=%s;password=%s;port=%d;encrypt=disable",
		conf.DbConfig.Server, conf.DbConfig.DbName, conf.DbConfig.User, conf.DbConfig.Password, conf.DbConfig.Port)

	a, err := xormadapter.NewAdapter(conf.DriverName, driveSource, true)
	if err != nil {
		log.Printf("连接数据库错误: %v", err)
		return nil, err
	}
	savePath, err := utils.GetCurrentPath("conf/rbac_model.conf")
	if err != nil {
		log.Printf("savePath路径错误: %v", err)
		return nil, err
	}
	e, err := casbin.NewEnforcer(savePath, a)
	if err != nil {
		//日志
		logs := new(models.LeitServerLog)
		logs.File = "/casbins.go"
		logs.Level = "debug"
		logs.Function = "CheckPermissions:NewCasbin()"
		logs.Message = "conf/rbac_model.conf文件加载错误"
		logs.Operator = "debug"
		logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
		logs.InsertRecord()

		return nil, err
	}
	if err := e.LoadPolicy(); err == nil {
		return e, err
	} else {
		fmt.Printf("casbin rbac_model or policy init error, message: %v", err)
		return nil, err
	}
}

// ServeHTTP is the iris compatible casbins handler which should be passed to specific routes or parties.
// Usage:
// [...]
// app.Get("/dataset1/resource1", casbinMiddleware.ServeHTTP, myHandler)
// [...]
func CheckPermissions(ctx context.Context) bool {
	user, ok := jwts.ParseToken(ctx)
	if !ok {
		return false
	}

	// 去除空格
	//uid := strings.Replace(user.Userid, " ", "", -1)
	uid := strings.TrimSpace(user.Role)
	e, err := NewCasbin()
	if err != nil {
		supports.Unauthorized(ctx, supports.PermissionsLess, nil)
		ctx.StopExecution()
		return false
	}
	ok, err = e.Enforce(uid, ctx.Path(), ctx.Method())
	log.Println("----------------", uid, ctx.Path(), ctx.Method())
	log.Println(ok)
	if err != nil {
		log.Printf("casbin权限控制错误: %v", err)
	}
	if !ok {
		supports.Unauthorized(ctx, supports.PermissionsLess, nil)
		ctx.StopExecution()
		return false
	}

	return true
	//ctx.Next()
}

// Wrapper is the router wrapper, prefer this method if you want to use casbins to your entire iris application.
// Usage:
// [...]
// app.WrapRouter(casbinMiddleware.Wrapper())
// app.Get("/dataset1/resource1", myHandler)
// [...]
func Wrapper() func(w http.ResponseWriter, r *http.Request, router http.HandlerFunc) {
	return func(w http.ResponseWriter, r *http.Request, router http.HandlerFunc) {
		//if !c.Check(r) {
		//	w.WriteHeader(http.StatusForbidden)
		//	w.Write([]byte("403 Forbidden"))
		//	return
		//}
		router(w, r)
	}
}
