package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"github.com/tealeg/xlsx"
	"xorm.io/core"
	"errors"
)

//属性主数据
type MeAttribute struct {
	Finr       int                `xorm:"pk comment('工厂号') int(0)" json:"me_attribute-finr"`
	Attrcode   int                `xorm:"pk comment('属性ID') int(0)" json:"me_attribute-attrcode"`
	Attrname   string             `xorm:"comment('属性名') VARCHAR(40)" json:"me_attribute-attrname"`
	Descr      string             `xorm:"comment('描述1') VARCHAR(40)" json:"me_attribute-descr"`
	Shortcode  string             `xorm:"comment('值短码') VARCHAR(40)" json:"me_attribute-shortcode"`
	Attrtype   string             `xorm:"comment('属性类型') VARCHAR(40)" json:"me_attribute-attrtype"`
	Valformat  string             `xorm:"comment('值格式') VARCHAR(40)" json:"me_attribute-valformat"`
	Lastmodif  string             `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"me_attribute-lastmodif"`
	Lastuser   string             `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"me_attribute-lastuser"`
	Credatuz   string             `xorm:"comment('创建时间') VARCHAR(14)" json:"me_attribute-credatuz"`
	Valst      []MeAttributeValst `json:"valst" xorm:"-"`
	Attrvalue1 string             `json:"attrvalue1" xorm:"-"`
	Attrvalue2 string             `json:"attrvalue2" xorm:"-"`
	Mandatory  int                `json:"mandatory" xorm:"-"`
	Verifyrule string             `json:"verifyrule" xorm:"-"`
}

func (t *MeAttribute) TableName() string {
	return "me_attribute"
}

// 清除string字段的右侧空格
func (t *MeAttribute) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *MeAttribute) Add() error {
	engine := db.Eloquent.Master()
	session := engine.NewSession()
	defer session.Close()
	// add Begin() before any action
	err := session.Begin()
	if err != nil{
		return err
	}
	Attr := new(Snrtab)
	Attr.Finr = t.Finr
	id, err := Attr.GetNextSnr("Attr")
	if err != nil {
		return errors.New("流水号生成失败！")
	}
	t.Attrcode = utils.ValueToInt(id, 0)
	_, err = session.Table("me_attribute").Insert(t)
	if err != nil {
		return err
	}

	if err != nil {
		session.Rollback()
		return err
	}
	//批量插入
	for k, v := range t.Valst {
		one := MeAttributeValst{}
		one.Finr = t.Finr
		one.Attrcode = t.Attrcode
		one.Shortcode = v.Shortcode
		one.Attrvalue = v.Attrvalue
		one.Descr = v.Descr
		one.Pos = k+1
		one.Valformat = v.Valformat
		one.Credatuz = t.Credatuz
		one.Lastuser = t.Lastuser
		one.Lastmodif = t.Lastmodif
		_, err = session.Table("me_attribute_valst").Insert(&one)
		if err != nil{
			session.Rollback()
			return err
		}
	}
	err = session.Commit()
	if err != nil {
		session.Rollback()
		return err
	}
	return nil
}

//删
func (t *MeAttribute) Del() bool {
	e := db.Eloquent.Master()
	_, err := e.Where("finr = ? and attrcode = ?", t.Finr, t.Attrcode).Delete(&MeAttribute{})
	if err != nil {
		return false
	}
	countrole := new(MeAttributeValst)
	affw, _ := e.Table("me_attribute_valst").Where("finr = ? and attrcode = ?", t.Finr, t.Attrcode).Count(countrole)
	if affw > 0 {
		_, err = e.Where("finr = ? and attrcode = ?", t.Finr, t.Attrcode).Delete(&MeAttributeValst{})
		if err != nil {
			return false
		}
	}

	return true
}

//改
func (t *MeAttribute) Update() error {
	engine := db.Eloquent.Master()
	session := engine.NewSession()
	defer session.Close()
	// add Begin() before any action
	err := session.Begin()
	if err != nil{
		return err
	}
	_, err = session.ID(core.PK{t.Finr, t.Attrcode}).Update(t)
	if err != nil {
		session.Rollback()
		return err
	}
	//批量删除
	_, err = session.Table("me_attribute_valst").Where("finr = ? and attrcode = ?", t.Finr, t.Attrcode).Delete(&MeAttributeValst{})
	if err != nil {
		session.Rollback()
		return err
	}
	//批量插入
	for k, v := range t.Valst {
		one := MeAttributeValst{}
		one.Finr = t.Finr
		one.Attrcode = t.Attrcode
		one.Shortcode = v.Shortcode
		one.Attrvalue = v.Attrvalue
		one.Descr = v.Descr
		one.Pos = k+1
		one.Valformat = v.Valformat
		one.Credatuz = t.Credatuz
		one.Lastuser = t.Lastuser
		one.Lastmodif = t.Lastmodif
		_, err = session.Table("me_attribute_valst").Insert(&one)
		if err != nil{
			session.Rollback()
			return err
		}
	}
	err = session.Commit()
	if err != nil {
		session.Rollback()
		return err
	}
	return nil
}

//查
func (t *MeAttribute) SelectOne() (MeAttribute, error) {
	e := db.Eloquent.Master()
	var data MeAttribute
	_, err := e.ID(core.PK{t.Finr, t.Attrcode}).Get(&data)
	if err != nil {
		return data, err
	}
	data.Clipped()
	var datalist []MeAttributeValst
	err = e.Where("finr = ? and attrcode = ?", t.Finr, t.Attrcode).Find(&datalist)
	if err != nil {
		return data, err
	}
	for k, _ := range datalist {
		datalist[k].Clipped()
	}
	data.Valst = datalist
	return data, nil
}

//分页
func (t *MeAttribute) GetPage(pageSize int, pageIndex int) ([]MeAttribute, int, error) {
	data := make([]MeAttribute, 0)
	e := db.Eloquent.Master()
	table := e.Table("me_attribute").Where("finr = ? ", t.Finr)
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if t.Attrcode > 0 {
		table = table.And("attrcode = ?", t.Attrcode)
		where += " and attrcode = " + "'" + utils.ValueToString(t.Attrcode, "") + "'"
	}
	if !utils.ValueIsEmpty(t.Attrname) {
		table = table.And("attrname = ?", t.Attrname)
		where += " and attrname = " + "'" + utils.ValueToString(t.Attrname, "") + "'"
	}
	if !utils.ValueIsEmpty(t.Descr) {
		descr := "%" + t.Descr + "%"
		table = table.And("descr like  ?", descr)
		where += " and descr like " + "'" + descr + "'"
	}
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " me_attribute.* FROM me_attribute " + where + " AND (convert(varchar(10),finr)+convert(varchar(40),attrcode) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),attrcode) FROM me_attribute " + where + " ORDER BY attrcode DESC)) ORDER BY attrcode DESC").Find(&data)
	pcount := new(MeAttribute)
	count, err := table.Count(pcount)
	if err != nil {
		return nil, 0, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, int(count), nil
}

//查询所有的属性及对应的值
func (t *MeAttribute) SelectAll() (map[string]interface{}, error) {
	//定义返回的数组
	result := make(map[string]interface{})

	e := db.Eloquent.Master()
	data := make([]MeAttribute, 0)

	//查询所有属性
	err := e.Table("me_attribute").Where("finr = ?", t.Finr).Find(&data)
	if err != nil {
		return result, err
	}

	//查出所有的属性对应的值，形成结构体
	for _, v := range data {
		//去空格
		v.Clipped()
		//key
		key := v.Attrname
		//查询值列表
		datalist := make([]MeAttributeValst, 0)
		err = e.Table("me_attribute_valst").Where("finr = ? and attrcode = ?", v.Finr, v.Attrcode).Find(&datalist)
		if err != nil {
			continue
		}
		tem := make([]map[string]string, 0)
		for _, vv := range datalist {
			one := make(map[string]string)
			vv.Clipped()
			one["attrcode"] = vv.Attrvalue
			one["shortcode"] = vv.Shortcode
			tem = append(tem, one)
		}
		result[key] = tem
	}

	return result, nil
}

//查询所有的属性名
func (t *MeAttribute) SelectData() ([]MeAttribute, error) {
	e := db.Eloquent.Master()
	data := make([]MeAttribute, 0)

	//查询所有属性
	err := e.Table("me_attribute").Where("finr = ?", t.Finr).Find(&data)
	if err != nil {
		return data, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, nil
}

//查询所有的属性名
func (t *MeAttribute) SelectArr() (map[string]string, error) {
	e := db.Eloquent.Master()
	data := make([]MeAttribute, 0)
	arr := map[string]string{}
	//查询所有属性
	err := e.Table("me_attribute").Where("finr = ?", t.Finr).Find(&data)
	if err != nil {
		return nil, err
	}
	for _, v := range data {
		v.Clipped()
		key := v.Attrname
		arr[key] = v.Descr
	}
	return arr, nil
}

//查询模块下的所有的属性名
func (t *MeAttribute) SelectInData(attrs []int) ([]MeAttribute, error) {
	e := db.Eloquent.Master()
	data := make([]MeAttribute, 0)

	//查询所有属性
	err := e.Table("me_attribute").Join("LEFT", "me_attributegroup_valst", "me_attributegroup_valst.attrcode = me_attribute.attrcode").In("me_attributegroup_valst.attrgroup", attrs).Where("me_attribute.finr = ?", t.Finr).Find(&data)

	if err != nil {
		return data, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, nil
}

//查
func (t *MeAttribute) SelectUnArr(attrcodes []int) ([]MeAttribute, error) {
	e := db.Eloquent.Master()

	var data []MeAttribute
	err := e.Where("finr = ? ", t.Finr).NotIn("attrcode", attrcodes).Find(&data)
	if err != nil {
		return data, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, nil
}

//导出报表
func (t *MeAttribute) ReadData() (string, string, error) {
	datalist := make([]MeAttribute, 0)
	e := db.Eloquent.Master()
	table := e.Table("me_attribute").Where("finr = ? ", t.Finr)
	if t.Attrcode > 0 {
		table = table.And("attrcode = ?", t.Attrcode)
	}
	if !utils.ValueIsEmpty(t.Attrname) {
		table = table.And("attrname = ?", t.Attrname)
	}
	if !utils.ValueIsEmpty(t.Descr) {
		descr := "%" + t.Descr + "%"
		table = table.And("descr like  ?", descr)
	}
	err := table.Find(&datalist)
	if err != nil {
		return "", "", err
	}
	for k, _ := range datalist {
		datalist[k].Clipped()
	}

	var file *xlsx.File
	var sheet *xlsx.Sheet
	var row *xlsx.Row
	var cell *xlsx.Cell

	file = xlsx.NewFile()
	sheet, err = file.AddSheet("Sheet1")
	if err != nil {
		return "", "", err
	}

	tableHead := true

	for _, val := range datalist {
		if tableHead {
			//添加表头
			row = sheet.AddRow()
			//描述
			cell = row.AddCell()
			cell.Value = "描述"
			//属性名
			cell = row.AddCell()
			cell.Value = "属性名"
			//属性值
			cell = row.AddCell()
			cell.Value = "属性值"
			//值短码
			cell = row.AddCell()
			cell.Value = "值短码"
			//属性类型
			cell = row.AddCell()
			cell.Value = "属性类型"
			//值格式
			cell = row.AddCell()
			cell.Value = "值格式"
			//修改人
			cell = row.AddCell()
			cell.Value = "修改人"
			//更改时间
			cell = row.AddCell()
			cell.Value = "更改时间"
			//创建时间
			cell = row.AddCell()
			cell.Value = "创建时间"
			tableHead = false
		}
		row = sheet.AddRow()
		//描述
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Descr, "")
		//属性名
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Attrname, "")
		//属性值
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Attrcode, "")
		//值短码
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Shortcode, "")
		//属性类型
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Attrtype, "")
		//值格式
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Valformat, "")
		//修改人
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Lastuser, "")
		//更改时间
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Lastmodif, "")
		//创建时间
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Credatuz, "")
	}
	name := utils.MakeOrderSn("属性主数据")
	filename := name + ".xlsx" //文件名

	exceldir,_ := utils.GetCurrentPath("web/public/uploadxlsx") // 目录+文件名
	excelfile := exceldir + "/" + filename
	err = file.Save(excelfile)
	if err != nil {
		return "", "", err
	}
	return excelfile, filename, nil
}
