package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"xorm.io/core"
	"errors"
)

type MeAttributegroup struct {
	Finr      int           `xorm:"pk comment('工厂号') int(0)" json:"me_attributegroup-finr"`
	Attrgroup int           `xorm:"pk comment('属性ID') int(0)" json:"me_attributegroup-attrgroup"`
	Descr     string        `xorm:"comment('描述1') VARCHAR(40)" json:"me_attributegroup-descr"`
	Color     string        `xorm:"comment('颜色') VARCHAR(20)" json:"me_attributegroup-color"`
	Lastmodif string        `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"me_attributegroup-lastmodif"`
	Lastuser  string        `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"me_attributegroup-lastuser"`
	Credatuz  string        `xorm:"comment('创建时间') VARCHAR(14)" json:"me_attributegroup-credatuz"`
	Valst     []MeAttribute `json:"valst" xorm:"-"`
	UnValst   []MeAttribute `json:"unvalst" xorm:"-"`
}

func (t *MeAttributegroup) TableName() string {
	return "me_attributegroup"
}

// 清除string字段的右侧空格
func (t *MeAttributegroup) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *MeAttributegroup) Add() error {
	e := db.Eloquent.Master()
	Attrgroup := new(Snrtab)
	Attrgroup.Finr = t.Finr
	id, err := Attrgroup.GetNextSnr("Attribute")
	if err != nil {
		return errors.New("流水号生成失败！")
	}
	t.Attrgroup = utils.ValueToInt(id, 0)
	_, err = e.Table("me_attributegroup").Insert(t)
	if err != nil {
		return err
	}
	//批量插入
	alldata := make([]MeAttributegroupValst, 0)
	for _, v := range t.Valst {
		onedata := MeAttributegroupValst{}
		onedata.Finr = t.Finr
		onedata.Attrgroup = t.Attrgroup
		onedata.Attrcode = v.Attrcode
		onedata.Credatuz = t.Credatuz
		onedata.Lastuser = t.Lastuser
		onedata.Lastmodif = t.Lastmodif
		alldata = append(alldata, onedata)
	}
	_, err = e.Table("me_attributegroup_valst").Insert(&alldata)
	if err != nil {
		return err
	}

	return nil
}

//删
func (t *MeAttributegroup) Del() bool {
	e := db.Eloquent.Master()
	_, err := e.Where("finr = ? and attrgroup = ?", t.Finr, t.Attrgroup).Delete(&MeAttributegroup{})
	if err != nil {
		return false
	}
	//删除分配零件
	_, err = e.Table("me_attributegroup_valst").Where("finr = ? and attrgroup = ?", t.Finr, t.Attrgroup).Delete(&MeAttributegroupValst{})
	if err != nil {
		return false
	}
	return true
}

//更新
func (t *MeAttributegroup) Update() bool {
	e := db.Eloquent.Master()
	//更新属性组表
	columns := []string{"descr", "color", "lastmodif", "lastuser"}
	_, err := e.ID(core.PK{t.Finr, t.Attrgroup}).Cols(columns...).Update(t)
	if err != nil {
		return false
	}
	//批量删除
	_, err = e.Table("me_attributegroup_valst").Where("finr = ? and attrgroup = ?", t.Finr, t.Attrgroup).Delete(&MeAttributegroupValst{})
	if err != nil {
		return false
	}
	//批量插入
	alldata := make([]MeAttributegroupValst, 0)
	for _, v := range t.Valst {
		onedata := MeAttributegroupValst{}
		onedata.Finr = t.Finr
		onedata.Attrgroup = t.Attrgroup
		onedata.Attrcode = v.Attrcode
		onedata.Credatuz = t.Credatuz
		onedata.Lastuser = t.Lastuser
		onedata.Lastmodif = t.Lastmodif
		alldata = append(alldata, onedata)
	}
	_, err = e.Table("me_attributegroup_valst").Insert(&alldata)
	if err != nil {
		return false
	}
	return true
}

//查
func (t *MeAttributegroup) SelectOne() (MeAttributegroup, error) {
	e := db.Eloquent.Master()
	var data MeAttributegroup
	_, err := e.ID(core.PK{t.Finr, t.Attrgroup}).Get(&data)
	if err != nil {
		return data, err
	}
	data.Clipped()
	var datalist []MeAttribute
	e.Table("me_attribute").Join("LEFT", "me_attributegroup_valst", "me_attributegroup_valst.attrcode = me_attribute.attrcode").Where("me_attribute.finr = ? and me_attributegroup_valst.attrgroup = ?", t.Finr, t.Attrgroup).Find(&datalist)
	var attrcodes []int
	for _, v := range datalist {
		attrcodes = append(attrcodes, v.Attrcode)
	}
	data.Valst = datalist
	var me MeAttribute
	me.Finr = t.Finr
	UnValst, _ := me.SelectUnArr(attrcodes)
	for k, _ := range UnValst {
		UnValst[k].Clipped()
	}
	data.UnValst = UnValst
	return data, nil
}

//分页
func (t *MeAttributegroup) GetPage(pageSize int, pageIndex int) ([]MeAttributegroup, int, error) {
	data := make([]MeAttributegroup, 0)
	e := db.Eloquent.Master()
	table := e.Table("me_attributegroup").Where("finr = ? ", t.Finr)
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if t.Attrgroup > 0 {
		table = table.And("attrgroup = ?", t.Attrgroup)
		where += " and attrgroup = " + "'" + utils.ValueToString(t.Attrgroup, "") + "'"
	}
	if !utils.ValueIsEmpty(t.Descr) {
		descr := "%" + t.Descr + "%"
		table = table.And("descr like ?", descr)
		where += " and descr like " + "'" + descr + "'"
	}
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " me_attributegroup.* FROM me_attributegroup " + where + " AND (convert(varchar(10),finr)+convert(varchar(40),attrgroup) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),attrgroup) FROM me_attributegroup " + where + " ORDER BY credatuz DESC)) ORDER BY credatuz DESC").Find(&data)
	pcount := new(MeAttributegroup)
	count, err := table.Count(pcount)
	if err != nil {
		return nil, 0, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, int(count), nil
}

func (t *MeAttributegroup) SelectArr() ([]MeAttribute, error) {
	e := db.Eloquent.Master()
	var undatalist []MeAttribute
	e.Table("me_attribute").Where("me_attribute.finr = ?", t.Finr).Find(&undatalist)
	for k, _ := range undatalist {
		undatalist[k].Clipped()
	}
	return undatalist, nil
}

//查询属性组
func (t *MeAttributegroup) SelectKV() (map[string]string, error) {
	e := db.Eloquent.Master()
	data := make([]MeAttributegroup, 0)
	e.Table("me_attributegroup").Where("me_attributegroup.finr = ?", t.Finr).Find(&data)
	kv := map[string]string{}
	kv["attr"] = "未分组"
	for _, v := range data {
		v.Clipped()
		key := utils.ValueToString(v.Attrgroup, "")
		kv[key] = v.Descr
	}
	return kv, nil
}
