package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"xorm.io/core"
	"errors"
)

type MeBomhead struct {
	Finr         int        `xorm:"pk comment('车间编号') Int" json:"me_bomhead-finr"`
	Bomid        string     `xorm:"pk comment('BOM记录标识') VARCHAR(20)" json:"me_bomhead-bomid"`
	Matnr        string     `xorm:"comment('物料号') VARCHAR(20)" json:"me_bomhead-matnr"`
	Descr        string     `xorm:"comment('描述') VARCHAR(40)" json:"me_bomhead-descr"`
	BomheadCval1 string     `xorm:"comment('') VARCHAR(40)" json:"me_bomhead-bomhead_cval1"`
	BomheadCval2 string     `xorm:"comment('') VARCHAR(40)" json:"me_bomhead-bomhead_cval2"`
	BomheadIval1 int        `xorm:"comment('') INT" json:"me_bomhead-bomhead_ival1"`
	BomheadIval2 int        `xorm:"comment('') INT" json:"me_bomhead-bomhead_ival2"`
	BomheadDval1 float64    `xorm:"comment('') Double" json:"me_bomhead-bomhead_dval1"`
	BomheadDval2 float64    `xorm:"comment('') Double" json:"me_bomhead-bomhead_dval2"`
	Lastmodif    string     `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"me_bomhead-lastmodif"`
	Lastuser     string     `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"me_bomhead-lastuser"`
	Credatuz     string     `xorm:"comment('创建时间') VARCHAR(14)" json:"me_bomhead-credatuz"`
	Valst        []MeBomlst `xorm:"-" json:"valst"`
}

func (t *MeBomhead) TableName() string {
	return "me_bomhead"
}

// 清除string字段的右侧空格
func (t *MeBomhead) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *MeBomhead) Add() error {
	e := db.Eloquent.Master()
	count := new(MeBomhead)
	affw, err := e.Table("me_bomhead").ID(core.PK{t.Finr, t.Bomid}).Count(count)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("me_bomhead").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//删
func (t *MeBomhead) Del() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Bomid}).Delete(&MeBomhead{})
	if err != nil {
		return false
	}
	//删除对应的me_bomlst
	_, err = e.Table("me_bomlst").Where("finr = ? and bomid = ?",t.Finr, t.Bomid).Delete(&MeBomlst{})
	if err != nil {
		return false
	}
	return true
}

//改
func (t *MeBomhead) Update() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Bomid}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//查
func (t *MeBomhead) SelectOne() (MeBomhead, error) {
	e := db.Eloquent.Master()
	var data MeBomhead
	_, err := e.ID(core.PK{t.Finr, t.Bomid}).Get(&data)
	if err != nil {
		return data, err
	}
	var Me MeBomlst
	Me.Finr = t.Finr
	Me.Bomid = t.Bomid
	result, err := Me.SetMenu()
	if err != nil {
		return data, err
	}
	data.Valst = result
	return data, nil
}

//分页
func (t *MeBomhead) GetPage(pageSize int, pageIndex int) ([]MeBomhead, int, error) {
	data := make([]MeBomhead, 0)
	e := db.Eloquent.Master()
	table := e.Table("me_bomhead").Where("finr = ? ", t.Finr)
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if t.Bomid != "" {
		bomid := "%"+ t.Bomid +"%"
		table = table.And("bomid like ?", bomid)
		where += " and bomid like " + "'" + bomid + "'"
	}
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " me_bomhead.* FROM me_bomhead " + where + " AND (convert(varchar(10),finr)+convert(varchar(40),bomid) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),bomid) FROM me_bomhead " + where + " ORDER BY credatuz DESC)) ORDER BY credatuz DESC").Find(&data)
	pcount := new(MeBomhead)
	count, err := table.Count(pcount)
	if err != nil {
		return data, 0, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, int(count), nil
}
