package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"errors"
	"xorm.io/core"
)

type MeBomlst struct {
	Finr          int        `xorm:"pk comment('车间编号') Int" json:"me_bomlst-finr"`
	Bomid         string     `xorm:"pk comment('BOM记录标识') VARCHAR(20)" json:"me_bomlst-bomid"`
	Fmatnr        string     `xorm:"pk comment('父项物料编码') VARCHAR(20)" json:"me_bomlst-fmatnr"`
	Cmatnr        string     `xorm:"pk comment('子项物料编码') VARCHAR(40)" json:"me_bomlst-cmatnr"`
	Fmatname      string     `xorm:"comment('父项物料名称') VARCHAR(40)" json:"me_bomlst-fmatname"`
	Fmatspec      string     `xorm:"comment('父项物料规格') VARCHAR(40)" json:"me_bomlst-fmatspec"`
	Fmatqty       float64    `xorm:"comment('父项物料用量') Double" json:"me_bomlst-fmatqty"`
	Fmatuom       string     `xorm:"comment('父项单位') VARCHAR(40)" json:"me_bomlst-fmatuom"`
	Levelnr       int        `xorm:"comment('层级') INT" json:"me_bomlst-levelnr"`
	Cmatname      string     `xorm:"comment('子项物料名称') VARCHAR(40)" json:"me_bomlst-cmatname"`
	Cmatqty       float64    `xorm:"comment('子项物料用量') Double" json:"me_bomlst-cmatqty"`
	Cmatuom       string     `xorm:"comment('子项单位') VARCHAR(40)" json:"me_bomlst-cmatuom"`
	Cmattype      string     `xorm:"comment('子项类型') VARCHAR(40)" json:"me_bomlst-cmattype"`
	Position      int        `xorm:"comment('位置') INT" json:"me_bomlst-position"`
	Leadtime      float64    `xorm:"comment('生产提前期') Double" json:"me_bomlst-leadtime"`
	Timeuom       string     `xorm:"comment('时间单位') VARCHAR(40)" json:"me_bomlst-timeuom"`
	Lossrate      float64    `xorm:"comment('损耗率') Double" json:"me_bomlst-lossrate"`
	Backflushitem int        `xorm:"comment('倒冲件') INT" json:"me_bomlst-backflushitem"`
	Criticalitem  int        `xorm:"comment('关键件') INT" json:"me_bomlst-criticalitem"`
	Validfrom     string     `xorm:"comment('有效起始日期') VARCHAR(40)" json:"me_bomlst-validfrom"`
	Validto       string     `xorm:"comment('有效截止日期') VARCHAR(40)" json:"me_bomlst-validto"`
	BomlstCval1   string     `xorm:"comment('') VARCHAR(40)" json:"me_bomlst-bomlst_cval1"`
	BomlstCval2   string     `xorm:"comment('') VARCHAR(40)" json:"me_bomlst-bomlst_cval2"`
	BomlstIval1   int        `xorm:"comment('') INT" json:"me_bomlst-bomlst_ival1"`
	BomlstIval2   int        `xorm:"comment('') INT" json:"me_bomlst-bomlst_ival2"`
	Lastmodif     string     `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"me_bomlst-lastmodif"`
	Lastuser      string     `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"me_bomlst-lastuser"`
	Credatuz      string     `xorm:"comment('创建时间') VARCHAR(14)" json:"me_bomlst-credatuz"`
	Children      []MeBomlst `json:"children" xorm:"-"`
}

func (t *MeBomlst) TableName() string {
	return "me_bomlst"
}

// 清除string字段的右侧空格
func (t *MeBomlst) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *MeBomlst) Add() error {
	e := db.Eloquent.Master()
	count := new(MeBomlst)
	affw, err := e.Table("me_bomlst").ID(core.PK{t.Finr, t.Bomid, t.Fmatnr, t.Cmatnr}).Count(count)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("me_bomlst").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//删
func (t *MeBomlst) Del() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Bomid, t.Fmatnr, t.Cmatnr}).Delete(&MeBomlst{})
	if err != nil {
		return false
	}
	return true
}

//改
func (t *MeBomlst) Update() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Bomid, t.Fmatnr, t.Cmatnr}).Cols("fmatname,fmatspec,fmatqty,fmatuom,levelnr,cmatname,cmatqty,cmatuom,cmattype,position,leadtime,timeuom,lossrate,backflushitem,criticalitem,validfrom,validto,bomlst_cval1,bomlst_cval2,bomlst_ival1,bomlst_ival2,lastmodif,lastuser,credatuz").Update(t)
	if err != nil {
		return false
	}
	return true
}

//查
func (t *MeBomlst) SelectOne() (MeBomlst, error) {
	e := db.Eloquent.Master()
	var data MeBomlst
	_, err := e.ID(core.PK{t.Finr, t.Bomid, t.Fmatnr, t.Cmatnr}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

func (t *MeBomlst) GetPage() (Menus []MeBomlst, err error) {
	e := db.Eloquent.Master()
	table := e.Table(t.TableName()).Where("finr = ?", t.Finr)
	if t.Bomid != "" {
		table = table.And("bomid = ?", t.Bomid)
	}
	if err = table.Asc("fmatnr").Find(&Menus); err != nil {
		return
	}
	return
}

//递归查询侧边栏
func (t *MeBomlst) SetMenu() (m []MeBomlst, err error) {
	menulist, err := t.GetPage()
	m = make([]MeBomlst, 0)
	for i := 0; i < len(menulist); i++ {
		menulist[i].Clipped()
		if menulist[i].Fmatnr != t.Bomid {
			continue
		}
		menusInfo := DiguiMeBomlst(&menulist, menulist[i])
		m = append(m, menusInfo)
	}
	return
}

//递归查询
func DiguiMeBomlst(me_bomlst *[]MeBomlst, me MeBomlst) MeBomlst {
	list := *me_bomlst

	min := make([]MeBomlst, 0)
	for j := 0; j < len(list); j++ {
		list[j].Clipped()
		if me.Cmatnr != list[j].Fmatnr {
			continue
		}
		mi := MeBomlst{}
		mi.Finr = list[j].Finr
		mi.Bomid = list[j].Bomid
		mi.Fmatnr = list[j].Fmatnr
		mi.Cmatnr = list[j].Cmatnr
		mi.Fmatname = list[j].Fmatname
		mi.Fmatspec = list[j].Fmatspec
		mi.Fmatqty = list[j].Fmatqty
		mi.Fmatuom = list[j].Fmatuom
		mi.Levelnr = list[j].Levelnr
		mi.Cmatname = list[j].Cmatname
		mi.Cmatqty = list[j].Cmatqty
		mi.Cmatuom = list[j].Cmatuom
		mi.Cmattype = list[j].Cmattype
		mi.Position = list[j].Position
		mi.Leadtime = list[j].Leadtime
		mi.Timeuom = list[j].Timeuom
		mi.Lossrate = list[j].Lossrate
		mi.Backflushitem = list[j].Backflushitem
		mi.Criticalitem = list[j].Criticalitem
		mi.Validfrom = list[j].Validfrom
		mi.Validto = list[j].Validto
		mi.BomlstCval1 = list[j].BomlstCval1
		mi.BomlstCval2 = list[j].BomlstCval2
		mi.BomlstIval1 = list[j].BomlstIval1
		mi.BomlstIval2 = list[j].BomlstIval2
		mi.Children = []MeBomlst{}
		ms := DiguiMeBomlst(me_bomlst, mi)
		utils.TrimStruct(&ms, ms)
		min = append(min, ms)
	}
	me.Children = min
	return me
}
