package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"xorm.io/core"
)

//版本记录表
type MePartAtcodver struct {
	Finr        int    `xorm:"pk comment('工厂号') int(0)" json:"me_part_atcodver-finr"`
	Version     int    `xorm:"pk comment('版本号') int(0)" json:"me_part_atcodver-version"`
	Descr       string `xorm:"comment('描述') VARCHAR(60)" json:"me_part_atcodver-descr"`
	Fromversion int    `xorm:"comment('来源版本号') int(0)" json:"me_part_atcodver-fromversion"`
	Released    int    `xorm:"comment('发布') int(0)" json:"me_part_atcodver-released"`
	Lastmodif   string `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"me_part_atcodver-lastmodif"`
	Lastuser    string `xorm:"comment('最近一次更改人') VARCHAR(14)" json:"me_part_atcodver-lastuser"`
	Credatuz    string `xorm:"comment('创建时间') VARCHAR(20)" json:"me_part_atcodver-credatuz"`
}

func (t *MePartAtcodver) TableName() string {
	return "me_part_atcodver"
}

// 清除string字段的右侧空格
func (t *MePartAtcodver) Clipped() {
	utils.TrimStruct(t, *t)
}

//发布版本
func (t *MePartAtcodver) Add() error {
	e := db.Eloquent.Master()
	vdata := new(MePartAtcodver)
	_, err := e.Table("me_part_atcodver").Where("finr = ?",t.Finr).Desc("version").Get(vdata)
	if err != nil {
		return err
	}
	t.Version = vdata.Version + 1
	//添加版本信息
	_, err = e.Table("me_part_atcodver").Insert(t)
	if err != nil {
		return err
	}
	//基于fromversion查询数据
	fromdata := make([]MePartAtcodlst, 0)
	err = e.Table("me_part_atcodlst").Where("finr = ? and  version = ?", t.Finr, t.Fromversion).Find(&fromdata)
	if err != nil {
		return err
	}
	//复制信息
	for _, v := range fromdata {
		onedata := MePartAtcodlst{}
		onedata.Finr = v.Finr
		onedata.Version = t.Version
		onedata.Projnr = v.Projnr
		onedata.Partid = v.Partid
		onedata.Attrcode = v.Attrcode
		onedata.Attrvalue = v.Attrvalue
		onedata.Credatuz = t.Credatuz
		onedata.Lastmodif = t.Lastmodif
		onedata.Lastuser = t.Lastuser
		//批量添加
		_, err = e.Table("me_part_atcodlst").Insert(&onedata)
	}

	if err != nil {
		return err
	}
	return nil
}

//查询版本集合
func (t *MePartAtcodver) SelectVal() ([]MePartAtcodver, error) {
	e := db.Eloquent.Master()
	//查询已分配的供应组
	var datalist []MePartAtcodver
	err := e.Table("me_part_atcodver").Where("finr = ? ", t.Finr).Find(&datalist)
	if err != nil {
		return nil, err
	}
	for k, _ := range datalist {
		datalist[k].Clipped()
	}
	return datalist, nil
}

//查询发布正式版本号
func (t *MePartAtcodver) SelectLastVersion() (int, error) {
	e := db.Eloquent.Master()
	//查询已分配的供应组
	var data MePartAtcodver
	_,err := e.Table("me_part_atcodver").Where("finr = ? and released = 1", t.Finr).Desc("version").Get(&data)
	if err != nil {
		return 0, err
	}
	return data.Version, nil
}

//分页
func (t *MePartAtcodver) GetPage(pageSize int, pageIndex int) ([]MePartAtcodver, int, error) {
	data := make([]MePartAtcodver, 0)
	e := db.Eloquent.Master()
	table := e.Table("me_part_atcodver").Where("finr = ? ", t.Finr)
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " me_part_atcodver.* FROM me_part_atcodver " + where + " AND (convert(varchar(10),finr)+convert(varchar(40),attrcode) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),attrcode) FROM me_part_atcodver " + where + " ORDER BY credatuz DESC)) ORDER BY credatuz DESC").Find(&data)
	pcount := new(MePartAtcodver)
	count, err := table.Count(pcount)
	if err != nil {
		return nil, 0, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, int(count), nil
}
//更新正式版
func (t *MePartAtcodver) Update() bool {
	e := db.Eloquent.Master()
	me := new(MePartAtcodver)
	me.Released = 1
	me.Lastuser = t.Lastuser
	me.Lastmodif = t.Lastmodif
	_, err := e.ID(core.PK{t.Finr, t.Version}).Update(me)
	if err != nil {
		return false
	}
	mes := new(MePartAtcodver)
	mes.Released = 0
	mes.Lastuser = t.Lastuser
	mes.Lastmodif = t.Lastmodif
	_, err = e.Cols("released").Where("finr = ? and version != ?",t.Finr, t.Version).Update(mes)
	if err != nil {
		return false
	}
	return true
}
//查
func (t *MePartAtcodver) SelectOne() (MePartAtcodver, error) {
	e := db.Eloquent.Master()
	var data MePartAtcodver
	_, err := e.ID(core.PK{t.Finr, t.Version}).Get(&data)
	data.Clipped()
	if err != nil {
		return data, err
	}
	return data, nil
}

//发布版本
func (t *MePartAtcodver) AddNew() (int,error) {
	e := db.Eloquent.Master()
	vdata := new(MePartAtcodver)
	_, err := e.Table("me_part_atcodver").Where("finr = ?",t.Finr).Desc("version").Get(vdata)
	if err != nil {
		return 0,err
	}
	t.Version = vdata.Version + 1
	//添加版本信息
	_, err = e.Table("me_part_atcodver").Insert(t)
	if err != nil {
		return 0,err
	}
	return t.Version,nil
}