package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"xorm.io/core"
	"errors"
)

// 零件族主数据
type MePartfamily struct {
	Finr         int           `xorm:"pk comment('工厂号') int(0)" json:"me_partfamily-finr"`
	Partfamilyid string        `xorm:"pk comment('零件族Id') VARCHAR(20)" json:"me_partfamily-partfamilyid"`
	Projnr       string        `xorm:"comment('项目号') VARCHAR(10)" json:"me_partfamily-projnr"`
	Descr        string        `xorm:"comment('描述') VARCHAR(40)" json:"me_partfamily-descr"`
	PfCust1      string        `xorm:"comment('零件Id') VARCHAR(40)" json:"me_partfamily-pf_cust1"`
	PfCust2      string        `xorm:"comment('零件Id') VARCHAR(40)" json:"me_partfamily-pf_cust2"`
	PfCust3      string        `xorm:"comment('零件Id') VARCHAR(40)" json:"me_partfamily-pf_cust3"`
	PfCust4      string        `xorm:"comment('零件Id') VARCHAR(40)" json:"me_partfamily-pf_cust4"`
	Lastmodif    string        `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"me_partfamily-lastmodif"`
	Lastuser     string        `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"me_partfamily-lastuser"`
	Credatuz     string        `xorm:"comment('创建时间') VARCHAR(14)" json:"me_partfamily-credatuz"`
	Valst        []MeAttribute `json:"valst" xorm:"-"`
	UnValst      []MeAttribute `json:"unvalst" xorm:"-"`
}

func (t *MePartfamily) TableName() string {
	return "me_partfamily"
}

// 清除string字段的右侧空格
func (t *MePartfamily) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *MePartfamily) Add() error {
	e := db.Eloquent.Master()
	countrole := new(MePartfamily)
	affw, err := e.Table("me_partfamily").ID(core.PK{t.Finr, t.Partfamilyid}).Count(countrole)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("me_partfamily").Insert(t)
	if err != nil {
		return err
	}
	//批量插入
	alldata := make([]MePartfamilyAtcodlst, 0)
	for _, v := range t.Valst {
		onedata := MePartfamilyAtcodlst{}
		onedata.Finr = t.Finr
		onedata.Partfamilyid = t.Partfamilyid
		onedata.Attrvalue2 = v.Attrvalue2
		onedata.Attrcode = v.Attrcode
		onedata.Mandatory = v.Mandatory
		onedata.Verifyrule = v.Verifyrule
		onedata.Credatuz = t.Credatuz
		onedata.Lastuser = t.Lastuser
		onedata.Lastmodif = t.Lastmodif
		alldata = append(alldata, onedata)
	}
	_, err = e.Table("me_partfamily_atcodlst").Insert(&alldata)
	if err != nil {
		return err
	}
	return nil
}

//删
func (t *MePartfamily) Del() bool {
	e := db.Eloquent.Master()
	_, err := e.Table("me_partfamily").Where("finr = ? and partfamilyid = ?", t.Finr, t.Partfamilyid).Delete(&MePartfamily{})
	if err != nil {
		return false
	}
	countrole := new(MePartfamilyAtcodlst)
	affw, _ := e.Table("me_partfamily_atcodlst").Where("finr = ? and partfamilyid = ?", t.Finr, t.Partfamilyid).Count(countrole)
	if affw > 0 {
		_, err = e.Table("me_partfamily_atcodlst").Where("finr = ? and partfamilyid = ?", t.Finr, t.Partfamilyid).Delete(&MePartfamilyAtcodlst{})
		if err != nil {
			return false
		}
	}

	return true
}

//改
func (t *MePartfamily) Update() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Partfamilyid}).Update(t)
	if err != nil {
		return false
	}
	//批量删除
	_, err = e.Table("me_partfamily_atcodlst").Where("finr = ? and partfamilyid = ?", t.Finr, t.Partfamilyid).Delete(&MePartfamilyAtcodlst{})
	if err != nil {
		return false
	}
	//批量插入
	alldata := make([]MePartfamilyAtcodlst, 0)
	for _, v := range t.Valst {
		onedata := MePartfamilyAtcodlst{}
		onedata.Finr = t.Finr
		onedata.Partfamilyid = t.Partfamilyid
		onedata.Attrvalue1 = v.Attrvalue1
		onedata.Attrvalue2 = v.Attrvalue2
		onedata.Attrcode = v.Attrcode
		onedata.Mandatory = v.Mandatory
		onedata.Verifyrule = v.Verifyrule
		onedata.Credatuz = t.Credatuz
		onedata.Lastuser = t.Lastuser
		onedata.Lastmodif = t.Lastmodif
		alldata = append(alldata, onedata)
	}
	_, err = e.Table("me_partfamily_atcodlst").Insert(&alldata)
	if err != nil {
		return false
	}
	return true
}

type MePartfamilyAttribute struct {
	MeAttribute          `xorm:"extends"`
	MePartfamilyAtcodlst `xorm:"extends"`
}

//查
func (t *MePartfamily) SelectOne() (MePartfamily, error) {
	e := db.Eloquent.Master()
	var data MePartfamily
	_, err := e.ID(core.PK{t.Finr, t.Partfamilyid}).Get(&data)
	if err != nil {
		return data, err
	}
	data.Clipped()
	var datalist []MeAttribute
	var alllist []MePartfamilyAttribute
	e.Table("me_attribute").Join("LEFT", "me_partfamily_atcodlst", "me_partfamily_atcodlst.attrcode = me_attribute.attrcode").Where("me_attribute.finr = ? and me_partfamily_atcodlst.partfamilyid = ?", t.Finr, t.Partfamilyid).Find(&alllist)
	var attrcodes []int
	for _, v := range alllist {
		v.MeAttribute.Clipped()
		v.MePartfamilyAtcodlst.Clipped()
		one := MeAttribute{}
		one = v.MeAttribute
		one.Mandatory = v.MePartfamilyAtcodlst.Mandatory
		one.Attrvalue1 = v.MePartfamilyAtcodlst.Attrvalue1
		one.Attrvalue2 = v.MePartfamilyAtcodlst.Attrvalue2
		one.Verifyrule = v.MePartfamilyAtcodlst.Verifyrule
		datalist = append(datalist, one)
		attrcodes = append(attrcodes, v.MeAttribute.Attrcode)
	}

	data.Valst = datalist
	var me MeAttribute
	me.Finr = t.Finr
	UnValst, _ := me.SelectUnArr(attrcodes)
	for k, _ := range UnValst {
		UnValst[k].Clipped()
	}
	data.UnValst = UnValst
	return data, nil
}

//分页
func (t *MePartfamily) GetPage(pageSize int, pageIndex int) ([]MePartfamily, int, error) {
	data := make([]MePartfamily, 0)
	e := db.Eloquent.Master()
	table := e.Table("me_partfamily").Where("finr = ? ", t.Finr)
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if t.Partfamilyid != "" {
		table = table.And("partfamilyid = ?", t.Partfamilyid)
		where += " and partfamilyid = " + "'" + utils.ValueToString(t.Partfamilyid, "") + "'"
	}
	if t.Projnr != "" {
		table = table.And("projnr = ?", t.Projnr)
		where += " and projnr = " + "'" + utils.ValueToString(t.Projnr, "") + "'"
	}
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " me_partfamily.* FROM me_partfamily " + where + " AND (convert(varchar(10),finr)+convert(varchar(40),partfamilyid) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),partfamilyid) FROM me_partfamily " + where + " ORDER BY credatuz DESC)) ORDER BY credatuz DESC").Find(&data)

	pcount := new(MePartfamily)
	count, err := table.Count(pcount)
	if err != nil {
		return nil, 0, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, int(count), nil
}

//批量查询
func (t *MePartfamily) SelectArr() (arr []string, err error) {
	e := db.Eloquent.Master()
	var data []MePartfamily
	err = e.Where("finr = ? ", t.Finr).Cols("partfamilyid").Find(&data)
	for _, v := range data {
		v.Clipped()
		arr = append(arr, v.Partfamilyid)
	}
	if err != nil {
		return arr, err
	}
	return arr, nil
}
