package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"xorm.io/core"
	"errors"
)

//模块主数据
type MePartgroup struct {
	Finr         int                `xorm:"pk comment('工厂号') int(0)" json:"me_partgroup-finr"`
	Partgroupid  string             `xorm:"pk comment('零件族Id') VARCHAR(20)" json:"me_partgroup-partgroupid"`
	Projnr       string             `xorm:"pk comment('项目Id') VARCHAR(40)" json:"me_partgroup-projnr"`
	Descr        string             `xorm:"comment('描述') VARCHAR(40)" json:"me_partgroup-descr"`
	PartgrpCust1 string             `xorm:"comment('预留1') VARCHAR(40)" json:"me_partgroup-partgrp_cust1"`
	PartgrpCust2 string             `xorm:"comment('预留2') VARCHAR(40)" json:"me_partgroup-partgrp_cust2"`
	PartgrpCust3 string             `xorm:"comment('预留3') VARCHAR(40)" json:"me_partgroup-partgrp_cust3"`
	PartgrpCust4 string             `xorm:"comment('预留4') VARCHAR(40)" json:"me_partgroup-partgrp_cust4"`
	Lastmodif    string             `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"me_partgroup-lastmodif"`
	Lastuser     string             `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"me_partgroup-lastuser"`
	Credatuz     string             `xorm:"comment('创建时间') VARCHAR(14)" json:"me_partgroup-credatuz"`
	PartValst    []MePart           `json:"valst" xorm:"-"`
	UnPartValst  []MePart           `json:"unvalst" xorm:"-"`
	AttrValst    []MeAttributegroup `json:"attrvalst" xorm:"-"`
	UnAttrValst  []MeAttributegroup `json:"unattrvalst" xorm:"-"`
}

func (t *MePartgroup) TableName() string {
	return "me_partgroup"
}

// 清除string字段的右侧空格
func (t *MePartgroup) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *MePartgroup) Add() error {
	e := db.Eloquent.Master()
	MePartgroup := new(Snrtab)
	MePartgroup.Finr = t.Finr
	id, err := MePartgroup.GetNextSnr("MePartgroup")
	if err != nil {
		return errors.New("流水号生成失败！")
	}
	t.Partgroupid = id
	_, err = e.Table("me_partgroup").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//删
func (t *MePartgroup) Del() bool {
	e := db.Eloquent.Master()
	//删除模块
	_, err := e.ID(core.PK{t.Finr, t.Partgroupid, t.Projnr}).Delete(&MePartgroup{})
	if err != nil {
		return false
	}
	//删除分配零件
	e.Table("me_partgroup_partlst").Where("finr = ? and partgroupid = ? and projnr = ?", t.Finr, t.Partgroupid, t.Projnr).Delete(&MePartgroupPartlst{})
	//删除关联分配属性组
	e.Table("me_partgroup_attrgrplst").Where("finr = ? and partgroupid = ? and projnr = ?", t.Finr, t.Partgroupid, t.Projnr).Delete(&MePartgroupAttrgrplst{})
	return true
}

//改
func (t *MePartgroup) Update() bool {
	e := db.Eloquent.Master()
	columns := []string{"descr", "partgrp_cust1", "partgrp_cust2", "partgrp_cust3", "partgrp_cust4", "lastmodif", "lastuser"}
	_, err := e.ID(core.PK{t.Finr, t.Partgroupid, t.Projnr}).Cols(columns...).Update(t)
	if err != nil {
		return false
	}
	return true
}

//查
func (t *MePartgroup) SelectOne() (MePartgroup, error) {
	e := db.Eloquent.Master()
	var data MePartgroup
	_, err := e.ID(core.PK{t.Finr, t.Partgroupid, t.Projnr}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

//分页
func (t *MePartgroup) GetPage(pageSize int, pageIndex int) ([]MePartgroup, int, error) {
	data := make([]MePartgroup, 0)
	e := db.Eloquent.Master()
	table := e.Table("me_partgroup").Where("finr = ? ", t.Finr)
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if t.Partgroupid != "" {
		table = table.And("partgroupid = ?", t.Partgroupid)
		where += " and partgroupid = " + "'" + utils.ValueToString(t.Partgroupid, "") + "'"
	}
	if t.Projnr != "" {
		table = table.And("projnr = ?", t.Projnr)
		where += " and projnr = " + "'" + utils.ValueToString(t.Projnr, "") + "'"
	}
	if t.Descr != "" {
		descr := "%" + t.Descr + "%"
		table = table.And("descr like ?", descr)
		where += " and descr like " + "'" + utils.ValueToString(descr, "") + "'"
	}
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " me_partgroup.* FROM me_partgroup " + where + " AND (convert(varchar(10),finr)+convert(varchar(40),partgroupid)+convert(varchar(40),projnr) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),partgroupid)+convert(varchar(40),projnr) FROM me_partgroup " + where + " ORDER BY credatuz DESC)) ORDER BY credatuz DESC").Find(&data)

	pcount := new(MePartgroup)
	count, err := table.Count(pcount)
	if err != nil {
		return nil, 0, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, int(count), nil
}

type PartGroup struct {
	MePartgroup    `xorm:"extends"`
	MePartAtcodlst `xorm:"extends"`
	MePart         `xorm:"extends"`
}

//查询所有的模块名
func (t *MePartgroup) SelectData() ([]MePartgroup, error) {
	e := db.Eloquent.Master()
	data := make([]MePartgroup, 0)

	//查询所有模块
	query := e.Table("me_partgroup").Where("finr = ?", t.Finr)
	if t.Projnr != "" {
		query = query.And("projnr = ?", t.Projnr)
	}
	err := query.Find(&data)
	if err != nil {
		return data, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, nil
}
