package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"errors"
	"github.com/tealeg/xlsx"
	"xorm.io/core"
)

//零件分配表 表头
type MePartRule struct {
	Finr             int             `xorm:"pk comment('工厂号') Int" json:"me_partrule-finr"`
	Partid           string          `xorm:"pk comment('零件id') VARCHAR(20)" json:"me_partrule-partid"`
	Projnr           string          `xorm:"pk comment('项目号') VARCHAR(10)" json:"me_partrule-projnr"`
	Ruleid           string          `xorm:"pk comment('规则id') VARCHAR(20)" json:"me_partrule-ruleid"`
	Descr            string          `xorm:"comment('描述') VARCHAR(40)" json:"me_partrule-descr"`
	Partruletype     string          `xorm:"comment('规则类型') VARCHAR(10)" json:"me_partrule-partrule_type"`
	PartruleIprecond int             `xorm:"comment('规则条件（零件数量）') INT" json:"me_partrule-partrule_iprecond"`
	PartruleCprecond string          `xorm:"comment('规则条件（零件族）') VARCHAR(20)" json:"me_partrule-partrule_cprecond"`
	Lastmodif        string          `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"me_partrule-lastmodif"`
	Lastuser         string          `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"me_partrule-lastuser"`
	Credatuz         string          `xorm:"comment('创建时间') VARCHAR(14)" json:"me_partrule-credatuz"`
	Valst            []MeSupplyGroup `json:"valst" xorm:"-"`
	UnValst          []MeSupplyGroup `json:"unvalst" xorm:"-"`
}

func (t *MePartRule) TableName() string {
	return "me_partrule"
}

// 清除string字段的右侧空格
func (t *MePartRule) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *MePartRule) Add() error {
	e := db.Eloquent.Master()
	count := new(MePartRule)
	affw, err := e.Table("me_partrule").ID(core.PK{t.Finr, t.Partid, t.Projnr, t.Ruleid}).Count(count)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("me_partrule").Insert(t)
	if err != nil {
		return err
	}
	alldata := make([]MePartRuleQtyassign, 0)
	//批量插入
	for k, v := range t.Valst {
		data := MePartRuleQtyassign{}
		data.Finr = t.Finr
		data.Partid = t.Partid
		data.Supplygroupid = v.Supplygroupid
		data.Pos = k + 1
		data.Projnr = t.Projnr
		data.Ruleid = t.Ruleid
		data.Assignqty = v.Assignqty
		data.Credatuz = t.Credatuz
		data.Lastuser = t.Lastuser
		data.Lastmodif = t.Lastmodif
		alldata = append(alldata, data)
	}
	_, err = e.Table("me_partrule_qtyassign").Insert(&alldata)
	if err != nil {
		return err
	}
	return nil
}

//删
func (t *MePartRule) Del() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Partid, t.Projnr, t.Ruleid}).Delete(&MePartRule{})
	if err != nil {
		return false
	}
	countrole := new(MePartRuleQtyassign)
	affw, _ := e.Table("me_partrule_qtyassign").Where("finr = ? and partid = ? and projnr = ? and ruleid = ?", t.Finr, t.Partid, t.Projnr, t.Ruleid).Count(countrole)
	if affw > 0 {
		_, err = e.Where("finr = ? and partid = ? and projnr = ? and ruleid = ?", t.Finr, t.Partid, t.Projnr, t.Ruleid).Delete(&MePartRuleQtyassign{})
		if err != nil {
			return false
		}
	}
	return true
}

//改
func (t *MePartRule) Update() bool {
	e := db.Eloquent.Master()
	columns := []string{"descr", "partrule_type", "partrule_iprecond", "partrule_cprecond", "lastmodif", "lastuser"}
	_, err := e.ID(core.PK{t.Finr, t.Partid, t.Projnr, t.Ruleid}).Cols(columns...).Update(t)
	if err != nil {
		return false
	}
	//批量删除
	_, err = e.Table("me_partrule_qtyassign").Where("finr = ? and partid = ? and projnr = ? and ruleid = ?", t.Finr, t.Partid, t.Projnr, t.Ruleid).Delete(&MeAttributeValst{})
	if err != nil {
		return false
	}
	alldata := make([]MePartRuleQtyassign, 0)
	//批量插入
	for k, v := range t.Valst {
		data := MePartRuleQtyassign{}
		data.Finr = t.Finr
		data.Partid = t.Partid
		data.Supplygroupid = v.Supplygroupid
		data.Pos = k + 1
		data.Projnr = t.Projnr
		data.Ruleid = t.Ruleid
		data.Assignqty = v.Assignqty
		data.Credatuz = t.Credatuz
		data.Lastuser = t.Lastuser
		data.Lastmodif = t.Lastmodif
		alldata = append(alldata, data)
	}
	_, err = e.Table("me_partrule_qtyassign").Insert(&alldata)
	if err != nil {
		return false
	}
	return true
}

type MePartGroupType struct {
	MeSupplyGroup       `xorm:"extends"`
	MePartRuleQtyassign `xorm:"extends"`
}

//查
func (t *MePartRule) SelectOne() (MePartRule, error) {
	e := db.Eloquent.Master()
	var data MePartRule
	_, err := e.Table("me_partrule").ID(core.PK{t.Finr, t.Partid, t.Projnr, t.Ruleid}).Get(&data)
	if err != nil {
		return data, err
	}
	data.Clipped()
	var datalist []MePartGroupType
	var datalists []MeSupplyGroup
	err = e.Table("me_supplygroup").Join("LEFT", "me_partrule_qtyassign", "me_partrule_qtyassign.finr = me_supplygroup.finr and me_partrule_qtyassign.supplygroupid = me_supplygroup.supplygroupid").Where("me_supplygroup.finr = ? and me_partrule_qtyassign.partid = ? and me_partrule_qtyassign.projnr = ? and me_partrule_qtyassign.ruleid = ?", t.Finr, t.Partid, t.Projnr, t.Ruleid).Find(&datalist)
	if err != nil {
		return data, err
	}
	var res []string
	for _, v := range datalist {
		v.MeSupplyGroup.Clipped()
		res = append(res, v.MeSupplyGroup.Supplygroupid)
		v.MeSupplyGroup.Assignqty = v.MePartRuleQtyassign.Assignqty
		datalists = append(datalists, v.MeSupplyGroup)
	}
	data.Valst = datalists

	var dataUnValst []MeSupplyGroup
	e.Table("me_supplygroup").NotIn("supplygroupid", res).Where("finr = ?", t.Finr).Find(&dataUnValst)

	for k, _ := range dataUnValst {
		dataUnValst[k].Clipped()
	}
	data.UnValst = dataUnValst
	return data, nil
}

//分页
func (t *MePartRule) GetPage(pageSize int, pageIndex int) ([]MePartRule, int, error) {
	data := make([]MePartRule, 0)
	e := db.Eloquent.Master()
	table := e.Table("me_partrule").Where("finr = ? ", t.Finr)
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if !utils.ValueIsEmpty(t.Partid) {
		table = table.And("partid = ?", t.Partid)
		where += " and partid = " + "'" + utils.ValueToString(t.Partid, "") + "'"
	}
	if t.Projnr != "" {
		table = table.And("projnr = ?", t.Projnr)
		where += " and projnr = " + "'" + utils.ValueToString(t.Projnr, "") + "'"
	}
	if !utils.ValueIsEmpty(t.Descr) {
		descr := "%" + t.Descr + "%"
		table = table.And("descr like ?", descr)
		where += " and attrgroup like " + "'" + descr + "'"
	}
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " me_partrule.* FROM me_partrule " + where + " AND (convert(varchar(10),finr)+convert(varchar(40),partid)+convert(varchar(40),projnr)+convert(varchar(40),ruleid) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),partid)+convert(varchar(40),projnr)+convert(varchar(40),ruleid) FROM me_partrule " + where + " ORDER BY credatuz DESC)) ORDER BY credatuz DESC").Find(&data)
	pcount := new(MePartRule)
	count, err := table.Count(pcount)
	if err != nil {
		return data, 0, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, int(count), nil
}

//查询未分配的供应组
func (t *MePartRule) SelectVal() ([]MeSupplyGroup, error) {
	e := db.Eloquent.Master()
	//查询已分配的供应组
	var data []MeSupplyGroup
	err := e.Table("me_supplygroup").Where("finr = ?", t.Finr).Find(&data)
	if err != nil {
		return nil, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, nil
}

//导出报表
func (t *MePartRule) ReadData() (string, string, error) {
	datalist := make([]MePartRule, 0)
	e := db.Eloquent.Master()
	table := e.Table("me_partrule").Where("finr = ? ", t.Finr)
	if !utils.ValueIsEmpty(t.Partid) {
		table = table.And("partid = ?", t.Partid)
	}
	if t.Projnr != "" {
		table = table.And("projnr = ?", t.Projnr)
	}
	if !utils.ValueIsEmpty(t.Descr) {
		descr := "%" + t.Descr + "%"
		table = table.And("descr like ?", descr)
	}
	err := table.Find(&datalist)
	if err != nil {
		return "", "", err
	}
	for k, _ := range datalist {
		datalist[k].Clipped()
	}

	var file *xlsx.File
	var sheet *xlsx.Sheet
	var row *xlsx.Row
	var cell *xlsx.Cell

	file = xlsx.NewFile()
	sheet, err = file.AddSheet("Sheet1")
	if err != nil {
		return "", "", err
	}

	tableHead := true

	for _, val := range datalist {
		if tableHead {
			//添加表头
			row = sheet.AddRow()
			//项目号
			cell = row.AddCell()
			cell.Value = "项目号"
			//零件ID
			cell = row.AddCell()
			cell.Value = "零件ID"
			//描述
			cell = row.AddCell()
			cell.Value = "描述"
			//规则id
			cell = row.AddCell()
			cell.Value = "规则id"
			//规则类型
			cell = row.AddCell()
			cell.Value = "规则类型"
			//规则（零件数量）
			cell = row.AddCell()
			cell.Value = "规则（零件数量）"
			//规则（零件族）
			cell = row.AddCell()
			cell.Value = "规则（零件族）"
			//修改人
			cell = row.AddCell()
			cell.Value = "修改人"
			//更改时间
			cell = row.AddCell()
			cell.Value = "更改时间"
			//创建时间
			cell = row.AddCell()
			cell.Value = "创建时间"
			tableHead = false
		}
		row = sheet.AddRow()
		//项目号
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Projnr, "")
		//零件ID
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Partid, "")
		//描述
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Descr, "")
		//规则id
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Ruleid, "")
		//规则类型
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Partruletype, "")
		//规则（零件数量）
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.PartruleCprecond, "")
		//规则（零件族）
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.PartruleIprecond, "")
		//修改人
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Lastuser, "")
		//更改时间
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Lastmodif, "")
		//创建时间
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Credatuz, "")
	}
	name := utils.MakeOrderSn("零件规则")
	filename := name + ".xlsx" //文件名

	exceldir,_ := utils.GetCurrentPath("web/public/uploadxlsx") // 目录+文件名
	excelfile := exceldir + "/" + filename
	err = file.Save(excelfile)
	if err != nil {
		return "", "", err
	}
	return excelfile, filename, nil
}