package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"fmt"
	"github.com/soniah/evaler"
	"strings"
	"xorm.io/core"
)

type PlnForecastArticle struct {
	Finr             int                   `xorm:"pk comment('工厂号') Int" json:"pln_forecast_article-finr"`
	Artnr            string                `xorm:"pk comment('产品ID') VARCHAR(20)" json:"pln_forecast_article-artnr"`
	Fartnr           string                `xorm:"comment('父产品ID') VARCHAR(20)" json:"pln_forecast_article-fartnr"`
	Custplantnr      string                `xorm:"comment('工厂号') VARCHAR(20)" json:"pln_forecast_article-custplantnr"`
	Level            string                `xorm:"comment('等级') VARCHAR(2)" json:"pln_forecast_article-level"`
	Arttype          string                `xorm:"comment('类型') VARCHAR(20)" json:"pln_forecast_article-arttype"`
	DemandCalFormula string                `xorm:"comment('项目ID') VARCHAR(20)" json:"pln_forecast_article-demand_cal_formula"`
	Descr            string                `xorm:"comment('描述1') VARCHAR(40)" json:"pln_forecast_article-descr"`
	Lastmodif        string                `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"pln_forecast_article-lastmodif"`
	Lastuser         string                `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"pln_forecast_article-lastuser"`
	Credatuz         string                `xorm:"comment('创建时间') VARCHAR(14)" json:"pln_forecast_article-credatuz"`
	Info             []map[string]int      `xorm:"-"json:"info"`
	SortInfo         []SortInfo            `xorm:"-"json:"sort_info"`
	Linfo            []map[string]TakeData `xorm:"-"json:"linfo"`
	Children         []PlnForecastArticle  `json:"children" xorm:"-"`
}

type SortInfo struct {
	Date string
	Qty  int
}

func (t *PlnForecastArticle) TableName() string {
	return "pln_forecast_article"
}

// 清除string字段的右侧空格
func (t *PlnForecastArticle) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *PlnForecastArticle) Add() error {
	e := db.Eloquent.Master()
	snr := new(Snrtab)
	snr.Finr = t.Finr
	anr,err := snr.GetNextSnr("Anr")
	if err !=nil{
		return err
	}
	t.Artnr = anr
	_, err = e.Table("pln_forecast_article").Insert(t)
	if err != nil {
		return err
	}
	return nil
}

//删
func (t *PlnForecastArticle) Del() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Artnr}).Delete(&PlnForecastArticle{})
	if err != nil {
		return false
	}
	if t.Level == "1" {
		_, err := e.Where("finr = ? and fartnr = ?", t.Finr, t.Artnr).Delete(&PlnForecastArticle{})
		if err != nil {
			return false
		}
	}
	return true
}

//改
func (t *PlnForecastArticle) Update() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Artnr}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//查
func (t *PlnForecastArticle) SelectOne() (PlnForecastArticle, error) {
	e := db.Eloquent.Master()
	var data PlnForecastArticle
	_, err := e.ID(core.PK{t.Finr, t.Artnr}).Get(&data)
	if err != nil {
		return data, err
	}
	data.Clipped()
	return data, nil
}

//分页
func (t *PlnForecastArticle) GetPage(pageSize int, pageIndex int) ([]PlnForecastArticle, int, error) {
	data := make([]PlnForecastArticle, 0)
	suns := make([]PlnForecastArticle, 0)
	tem := map[string][]PlnForecastArticle{}
	e := db.Eloquent.Master()
	table := e.Table("pln_forecast_article").Where("finr = ? and level = 1", t.Finr)
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'" + " and level = 1 "
	if !utils.ValueIsEmpty(t.Descr) {
		descr := "%"+t.Descr + "%"
		table = table.And("descr like ?",descr)
		where += " and descr like " + "'" + descr + "'"
	}
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " pln_forecast_article.* FROM pln_forecast_article " + where + " AND (convert(varchar(10),finr)+convert(varchar(40),artnr) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),artnr) FROM pln_forecast_article " + where + " ORDER BY arttype DESC)) ORDER BY arttype DESC").Find(&data)
	pcount := new(PlnForecastArticle)
	count, err := table.Count(pcount)
	if err != nil {
		return data, 0, err
	}
	var fathers []string
	for k, v := range data {
		data[k].Clipped()
		fathers = append(fathers, v.Artnr)
	}
	err = e.Table("pln_forecast_article").In("fartnr", fathers).Desc("arttype").Find(&suns)
	if err != nil {
		return data, 0, err
	}
	for _, v := range suns {
		v.Clipped()
		key := v.Fartnr
		tem[key] = append(tem[key], v)
	}
	for k, v := range data {
		key := v.Artnr
		val, ok := tem[key]
		if ok {
			data[k].Children = val
		}
	}
	return data, int(count), nil
}

//查询产品列表
func (t *PlnForecastArticle) SelectAll() ([]PlnForecastArticle, error) {
	e := db.Eloquent.Master()
	data := make([]PlnForecastArticle, 0)
	err := e.Table("pln_forecast_article").Where("finr = ?", t.Finr).Find(&data)
	if err != nil {
		return data, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, nil
}

//验证计算公式
func (t *PlnForecastArticle) VerifyFormula() (string, string, error) {
	e := db.Eloquent.Master()
	//查询出运算公式
	data := make([]PlnForecastFormula, 0)
	err := e.Table("pln_forecast_formula").Where("finr = ?", t.Finr).Find(&data)
	if err != nil {
		return "", "", err
	}
	formulaMap := map[string]string{}
	for _, v := range data {
		v.Clipped()
		key := v.Code
		val := "(" + v.DemandCalFormula + ")"
		formulaMap[key] = val
	}

	partids := make([]PlnForecastCriticalItem, 0)

	//第一步：查询核心零件
	err = e.Table("pln_forecast_critical_item").Where("finr = ?", t.Finr).Find(&partids)
	if err != nil {
		return "", "", err
	}
	partsMap := map[string]string{}
	for _, v := range partids {
		v.Clipped()
		key := v.Abrev
		val := "100"
		partsMap[key] = val
	}

	DemandCalFormula := t.DemandCalFormula

	fmt.Printf("公式替换前:%v", DemandCalFormula)

	for oldstr, newstr := range formulaMap {
		//替换所有的
		DemandCalFormula = strings.Replace(DemandCalFormula, oldstr, utils.ValueToString(newstr, ""), -1)
	}

	//第二步：所有的值用100替换,遍历partsMap,替换公式
	for oldstr, newstr := range partsMap {
		//替换所有的
		DemandCalFormula = strings.Replace(DemandCalFormula, oldstr, utils.ValueToString(newstr, ""), -1)
	}

	fmt.Printf("公式替换后:%v", DemandCalFormula)
	result, err := evaler.Eval(DemandCalFormula)
	if err != nil {
		return "", "", err
	}
	num, err := evaler.BigratToInt(result)
	if err != nil {
		return "", "", err
	}
	//第三步：返回原来的计算公式和新得计算公式
	resultstr := DemandCalFormula + " = " + utils.ValueToString(num, "")
	return t.DemandCalFormula, resultstr, nil
}

//查询父类产品列表
func (t *PlnForecastArticle) SelectFathersAll() ([]PlnForecastArticle, error) {
	e := db.Eloquent.Master()
	data := make([]PlnForecastArticle, 0)
	suns := make([]PlnForecastArticle, 0)
	tem := map[string][]PlnForecastArticle{}
	err := e.Table("pln_forecast_article").Where("finr = ? and level = 1", t.Finr).Desc("custplantnr", "arttype").Find(&data)
	if err != nil {
		return data, err
	}
	var fathers []string
	for k, v := range data {
		data[k].Clipped()
		fathers = append(fathers, v.Artnr)
	}
	err = e.Table("pln_forecast_article").In("fartnr", fathers).Desc("custplantnr", "arttype").Find(&suns)
	if err != nil {
		return data, err
	}
	for _, v := range suns {
		v.Clipped()
		key := v.Fartnr
		tem[key] = append(tem[key], v)
	}
	for k, v := range data {
		key := v.Artnr
		val, ok := tem[key]
		if ok {
			data[k].Children = val
		}
	}
	return data, nil
}
