package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"strings"
	"time"
	"xorm.io/core"
)

type PlnForecastArticleTotal struct {
	Finr        int    `xorm:"pk comment('工厂号') Int" json:"pln_forecast_article_total-finr"`
	Artnr       string `xorm:"pk comment('产品ID') VARCHAR(64)" json:"pln_forecast_article_total-artnr"`
	Demanddate  string `xorm:"pk comment('添加日期') VARCHAR(20)" json:"pln_forecast_article_total-demanddate"`
	Demandyear  int    `xorm:"comment('添加年份') Int" json:"pln_forecast_article_total-demandyear"`
	Demandmonth int    `xorm:"comment('添加月份') Int" json:"pln_forecast_article_total-demandmonth"`
	Demandweek  int    `xorm:"comment('周') Int" json:"pln_forecast_article_total-demandweek"`
	Custplantnr string `xorm:"comment('工厂号') VARCHAR(20)" json:"pln_forecast_article_total-custplantnr"`
	Demandqty   int    `xorm:"comment('数量') Int" json:"pln_forecast_article_total-demandqty"`
	Lastmodif   string `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"pln_forecast_article_total-lastmodif"`
	Lastuser    string `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"pln_forecast_article_total-lastuser"`
	Credatuz    string `xorm:"comment('创建时间') VARCHAR(14)" json:"pln_forecast_article_total-credatuz"`
}

func (t *PlnForecastArticleTotal) TableName() string {
	return "pln_forecast_article_total"
}

// 清除string字段的右侧空格
func (t *PlnForecastArticleTotal) Clipped() {
	utils.TrimStruct(t, *t)
}

//记录统计，并且更新
func (t *PlnForecastArticleTotal) Add() error {
	e := db.Eloquent.Master()
	countrole := new(PlnForecastArticleTotal)
	affw, err := e.Table("pln_forecast_article_total").ID(core.PK{t.Finr, t.Artnr, t.Demanddate}).Count(countrole)
	if err != nil {
		return err
	}
	if affw > 0 {
		//更新
		_, err = e.ID(core.PK{t.Finr, t.Artnr, t.Demanddate}).Update(t)
	} else {
		//添加
		_, err = e.Table("pln_forecast_article_total").Insert(t)
	}
	if err != nil {
		return err
	}
	return nil
}

//查询历史记录（日报）
func (t *PlnForecastArticleTotal) SelectAllDay(timedate string) ([]PlnForecastArticleTotal, error) {
	e := db.Eloquent.Master()
	data := make([]PlnForecastArticleTotal, 0)
	query := e.Table("pln_forecast_article_total").Where("finr = ?", t.Finr)

	if !utils.ValueIsEmpty(t.Artnr) {
		query = query.And("artnr = ?",t.Artnr)
	}
	if !utils.ValueIsEmpty(timedate) {
		datatime := strings.Split(timedate, "_")
		firstmonth := utils.ValueToString(datatime[0], "") + "'"
		lastmonth := utils.ValueToString(datatime[1], "") + "'"
		query = query.And("demanddate >= ?",firstmonth)
		query = query.And("demanddate <= ?",lastmonth)
	} else {
		firstmonth, lastmonth := utils.TimeReturnMonth(time.Now())
		query = query.And("demanddate >= ?",firstmonth)
		query = query.And("demanddate <= ?",lastmonth)
	}
	err := query.Find(&data)
	if err != nil {
		return data, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, nil
}
//查询历史记录（月报）
func (t *PlnForecastArticleTotal) SelectAllMonth(timedate string) (data []PlnForecastArticleTotal, err error) {
	e := db.Eloquent.Master()
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if !utils.ValueIsEmpty(t.Artnr) {
		where += " and artnr = " + "'" + utils.ValueToString(t.Artnr, "") + "'"
	}
	if !utils.ValueIsEmpty(timedate) {
		datatime := strings.Split(timedate, "_")
		where += " and demanddate >= " + "'" + utils.ValueToString(datatime[0], "") + "'"
		where += " and demanddate <= " + "'" + utils.ValueToString(datatime[1], "") + "'"
	}
	err = e.SQL("select artnr,SUM(demandqty) as demandqty,demandyear,demandmonth from pln_forecast_article_total " + where + "  GROUP BY artnr,demandyear,demandmonth").OrderBy("artnr,demandyear,demandmonth").Find(&data)
	if err != nil {
		return data,  err
	}
	return data,  err
}