package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"xorm.io/core"
	"errors"
)

type PlnForecastCalendar struct {
	Finr        int                         `xorm:"pk comment('工厂号') Int" json:"pln_forecast_calendar-finr"`
	Calendarid  string                      `xorm:"pk comment('工作日历id')  VARCHAR(20)" json:"pln_forecast_calendar-calendarid"`
	Custplantnr string                      `xorm:"comment('工厂号') VARCHAR(20)" json:"pln_forecast_calendar-custplantnr"`
	Mydefault   int                         `xorm:"comment('默认') Int" json:"pln_forecast_calendar-mydefault"`
	Descr       string                      `xorm:"comment('描述')  VARCHAR(80)" json:"pln_forecast_calendar-descr"`
	Lastmodif   string                      `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"pln_forecast_calendar-lastmodif"`
	Lastuser    string                      `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"pln_forecast_calendar-lastuser"`
	Credatuz    string                      `xorm:"comment('创建时间') VARCHAR(14)" json:"pln_forecast_calendar-credatuz"`
	Valst       []PlnForecastCalendarDaylst `json:"valst" xorm:"-"`
}

func (t *PlnForecastCalendar) TableName() string {
	return "pln_forecast_calendar"
}

// 清除string字段的右侧空格
func (t *PlnForecastCalendar) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *PlnForecastCalendar) Add() error {
	e := db.Eloquent.Master()
	session := e.NewSession()
	defer session.Close()
	// add Begin() before any action
	err := session.Begin()
	if err != nil {
		return err
	}
	countrole := new(PlnForecastCalendar)
	affw, err := e.Table("pln_forecast_calendar").ID(core.PK{t.Finr, t.Calendarid}).Count(countrole)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = session.Table("pln_forecast_calendar").Insert(t)
	if err != nil {
		session.Rollback()
		return err
	}
	for _, v := range t.Valst {
		one := PlnForecastCalendarDaylst{}
		one.Finr = t.Finr
		one.Effactor = v.Effactor
		one.Calendarid = t.Calendarid
		one.Caldate = v.Caldate
		one.Credatuz = t.Credatuz
		one.Lastuser = t.Lastuser
		one.Lastmodif = t.Lastmodif
		_, err = session.Table("pln_forecast_calendar_daylst").Insert(&one)
		if err != nil {
			session.Rollback()
			return err
		}

	}
	err = session.Commit()
	if err != nil {
		return err
	}
	return nil
}

//删
func (t *PlnForecastCalendar) Del() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Calendarid}).Delete(&PlnForecastCalendar{})
	if err != nil {
		return false
	}
	countrole := new(PlnForecastCalendarDaylst)
	affw, _ := e.Table("pln_forecast_calendar_daylst").Where("finr = ? and calendarid = ?", t.Finr, t.Calendarid).Count(countrole)
	if affw > 0 {
		_, err = e.Where("finr = ? and calendarid = ?", t.Finr, t.Calendarid).Delete(&PlnForecastCalendarDaylst{})
		if err != nil {
			return false
		}
	}
	return true
}

//改
func (t *PlnForecastCalendar) Update() error {
	e := db.Eloquent.Master()
	session := e.NewSession()
	defer session.Close()
	// add Begin() before any action
	err := session.Begin()
	if err != nil {
		return err
	}
	_, err = session.ID(core.PK{t.Finr, t.Calendarid}).Cols("custplantnr", "mydefault", "descr", "lastmodif", "lastuser", "credatuz").Update(t)
	if err != nil {
		session.Rollback()
		return err
	}
	if t.Mydefault > 0 {
		//如果当前工厂日历设置为默认，其他更新
		_, err := session.Table("pln_forecast_calendar").Where("finr = ? and calendarid != ? and custplantnr = ?", t.Finr, t.Calendarid, t.Custplantnr).Cols("mydefault").Update(map[string]interface{}{
			"mydefault": 0,
		})
		if err != nil {
			session.Rollback()
			return err
		}
	}
	//批量删除
	_, err = session.Table("pln_forecast_calendar_daylst").Where("finr = ? and calendarid = ?", t.Finr, t.Calendarid).Delete(&PlnForecastCalendarDaylst{})
	if err != nil {
		session.Rollback()
		return err
	}
	//批量插入
	for _, v := range t.Valst {
		one := PlnForecastCalendarDaylst{}
		one.Finr = t.Finr
		one.Effactor = v.Effactor
		one.Calendarid = t.Calendarid
		one.Caldate = v.Caldate
		one.Credatuz = t.Credatuz
		one.Lastuser = t.Lastuser
		one.Lastmodif = t.Lastmodif
		_, err = session.Table("pln_forecast_calendar_daylst").Insert(&one)
		if err != nil {
			session.Rollback()
			return err
		}
	}
	err = session.Commit()
	if err != nil {
		return err
	}
	return nil
}

//查
func (t *PlnForecastCalendar) SelectOne() (PlnForecastCalendar, error) {
	e := db.Eloquent.Master()
	var data PlnForecastCalendar
	_, err := e.ID(core.PK{t.Finr, t.Calendarid}).Get(&data)
	if err != nil {
		return data, err
	}
	data.Clipped()
	datalist := make([]PlnForecastCalendarDaylst, 0)
	err = e.Table("pln_forecast_calendar_daylst").Where("finr = ? and calendarid = ?", t.Finr, t.Calendarid).Find(&datalist)
	if err != nil {
		return data, err
	}
	for k, _ := range datalist {
		datalist[k].Clipped()
	}
	data.Valst = datalist
	return data, nil
}

//分页
func (t *PlnForecastCalendar) GetPage(pageSize int, pageIndex int) ([]PlnForecastCalendar, int, error) {
	data := make([]PlnForecastCalendar, 0)
	e := db.Eloquent.Master()
	table := e.Table("pln_forecast_calendar").Where("finr = ? ", t.Finr)
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if !utils.ValueIsEmpty(t.Calendarid) {
		table = table.And("calendarid = ?", t.Calendarid)
		where += " and calendarid = " + "'" + t.Calendarid + "'"
	}
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " pln_forecast_calendar.* FROM pln_forecast_calendar " + where + " AND (convert(varchar(10),finr)+convert(varchar(40),calendarid) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),calendarid) FROM pln_forecast_calendar " + where + " ORDER BY credatuz DESC)) ORDER BY credatuz DESC").Find(&data)
	pcount := new(PlnForecastCalendar)
	count, err := table.Count(pcount)
	if err != nil {
		return data, 0, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, int(count), nil
}

//查
func (t *PlnForecastCalendar) SelectAll() ([]PlnForecastCalendar, error) {
	e := db.Eloquent.Master()

	data := make([]PlnForecastCalendar, 0)
	err := e.Table("pln_forecast_calendar").Where("finr = ? and mydefault = ?", t.Finr, 1).Desc("custplantnr").Find(&data)
	if err != nil {
		return data, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, nil
}
