package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"xorm.io/core"
	"errors"
)

type PlnForecastCriticalItem struct {
	Finr      int    `xorm:"pk comment('工厂号') Int" json:"pln_forecast_critical_item-finr"`
	Partid    string `xorm:"pk comment('零件ID') VARCHAR(20)" json:"pln_forecast_critical_item-partid"`
	Abrev     string `xorm:"comment('简写') VARCHAR(14)" json:"pln_forecast_critical_item-abrev"`
	Lastmodif string `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"pln_forecast_critical_item-lastmodif"`
	Lastuser  string `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"pln_forecast_critical_item-lastuser"`
	Credatuz  string `xorm:"comment('创建时间') VARCHAR(14)" json:"pln_forecast_critical_item-credatuz"`
	Descr     string `xorm:"-" json:"pln_forecast_critical_item-descr"`
}

func (t *PlnForecastCriticalItem) TableName() string {
	return "pln_forecast_critical_item"
}

// 清除string字段的右侧空格
func (t *PlnForecastCriticalItem) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *PlnForecastCriticalItem) Add() error {
	e := db.Eloquent.Master()
	countrole := new(PlnForecastCriticalItem)
	affw, err := e.Table("pln_forecast_critical_item").ID(core.PK{t.Finr, t.Partid}).Count(countrole)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("pln_forecast_critical_item").Insert(t)
	if err != nil {
		return err
	}
	return nil
}

//删
func (t *PlnForecastCriticalItem) Del() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Partid}).Delete(&PlnForecastCriticalItem{})
	if err != nil {
		return false
	}
	return true
}

//改
func (t *PlnForecastCriticalItem) Update() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Partid}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//查
func (t *PlnForecastCriticalItem) SelectOne() (PlnForecastCriticalItem, error) {
	e := db.Eloquent.Master()
	var data PlnForecastCriticalItem
	_, err := e.ID(core.PK{t.Finr, t.Partid}).Get(&data)
	if err != nil {
		return data, err
	}
	data.Clipped()
	return data, nil
}
type PlnForecastCriticalItemPart struct {
	PlnForecastCriticalItem `xorm:"extends"`
	MePart `xorm:"extends"`
}
//分页
func (t *PlnForecastCriticalItem) GetPage(pageSize int, pageIndex int) ([]PlnForecastCriticalItem, int, error) {
	datalist := make([]PlnForecastCriticalItem, 0)
	data := make([]PlnForecastCriticalItemPart, 0)
	e := db.Eloquent.Master()
	table := e.Table("pln_forecast_critical_item").Where("pln_forecast_critical_item.finr = ? ", t.Finr)
	where := " LEFT JOIN me_part ON me_part.partid = pln_forecast_critical_item.partid and me_part.finr = pln_forecast_critical_item.finr where pln_forecast_critical_item.finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if !utils.ValueIsEmpty(t.Partid) {
		table = table.And("partid = ?", t.Partid)
		where += " and pln_forecast_critical_item.partid = " + "'" + t.Partid + "'"
	}
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " * FROM pln_forecast_critical_item " + where + " AND (convert(varchar(10),pln_forecast_critical_item.finr)+convert(varchar(40),pln_forecast_critical_item.partid) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),pln_forecast_critical_item.finr)+convert(varchar(40),pln_forecast_critical_item.partid) FROM pln_forecast_critical_item " + where + " ORDER BY pln_forecast_critical_item.credatuz DESC)) ORDER BY pln_forecast_critical_item.credatuz DESC").Find(&data)
	pcount := new(PlnForecastCriticalItem)
	count, err := table.Count(pcount)
	if err != nil {
		return datalist, 0, err
	}
	for _, v := range data {
		v.PlnForecastCriticalItem.Clipped()
		v.MePart.Clipped()
		one := PlnForecastCriticalItem{}
		one.Finr = v.PlnForecastCriticalItem.Finr
		one.Partid = v.PlnForecastCriticalItem.Partid
		one.Abrev = v.PlnForecastCriticalItem.Abrev
		one.Lastmodif = v.PlnForecastCriticalItem.Lastmodif
		one.Lastuser = v.PlnForecastCriticalItem.Lastuser
		one.Credatuz = v.PlnForecastCriticalItem.Credatuz
		one.Descr = v.MePart.Descr2
		datalist = append(datalist,one)
	}

	return datalist, int(count), nil
}

//查询核心零件列表
func (t *PlnForecastCriticalItem) SelectAll() ([]PlnForecastCriticalItem, error) {
	e := db.Eloquent.Master()
	datalist := make([]PlnForecastCriticalItem, 0)
	data := make([]PlnForecastCriticalItemPart, 0)
	err := e.Table("pln_forecast_critical_item").Join("LEFT","me_part","me_part.partid = pln_forecast_critical_item.partid and me_part.finr = pln_forecast_critical_item.finr").Where("pln_forecast_critical_item.finr = ?", t.Finr).Find(&data)
	if err != nil {
		return datalist, err
	}
	for _, v := range data {
		v.PlnForecastCriticalItem.Clipped()
		v.MePart.Clipped()
		one := PlnForecastCriticalItem{}
		one.Finr = v.PlnForecastCriticalItem.Finr
		one.Partid = v.PlnForecastCriticalItem.Partid
		one.Abrev = v.PlnForecastCriticalItem.Abrev
		one.Lastmodif = v.PlnForecastCriticalItem.Lastmodif
		one.Lastuser = v.PlnForecastCriticalItem.Lastuser
		one.Credatuz = v.PlnForecastCriticalItem.Credatuz
		one.Descr = v.MePart.Descr2
		datalist = append(datalist,one)
	}
	return datalist, nil
}
