package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"xorm.io/core"
	"errors"
)

type PlnForecastFormula struct {
	Finr             int    `xorm:"pk comment('工厂号') Int" json:"pln_forecast_formula-finr"`
	Code             string `xorm:"pk comment('代号') VARCHAR(64)" json:"pln_forecast_formula-code"`
	Descr            string `xorm:"comment('描述') VARCHAR(40)" json:"pln_forecast_formula-descr"`
	DemandCalFormula string `xorm:"comment('计算公式')  VARCHAR(20)" json:"pln_forecast_formula-demand_cal_formula"`
	Lastmodif        string `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"pln_forecast_formula-lastmodif"`
	Lastuser         string `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"pln_forecast_formula-lastuser"`
	Credatuz         string `xorm:"comment('创建时间') VARCHAR(14)" json:"pln_forecast_formula-credatuz"`
}

func (t *PlnForecastFormula) TableName() string {
	return "pln_forecast_formula"
}

// 清除string字段的右侧空格
func (t *PlnForecastFormula) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *PlnForecastFormula) Add() error {
	e := db.Eloquent.Master()
	count := new(PlnForecastFormula)
	affw, err := e.Table("pln_forecast_formula").ID(core.PK{t.Finr, t.Code}).Count(count)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("pln_forecast_formula").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//删
func (t *PlnForecastFormula) Del() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Code}).Delete(&PlnForecastFormula{})
	if err != nil {
		return false
	}
	return true
}

//改
func (t *PlnForecastFormula) Update() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Code}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//查
func (t *PlnForecastFormula) SelectOne() (PlnForecastFormula, error) {
	e := db.Eloquent.Master()
	var data PlnForecastFormula
	_, err := e.ID(core.PK{t.Finr, t.Code}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

//分页
func (t *PlnForecastFormula) GetPage(pageSize int, pageIndex int) ([]PlnForecastFormula, int, error) {
	data := make([]PlnForecastFormula, 0)
	e := db.Eloquent.Master()
	table := e.Table("pln_forecast_formula").Where("finr = ? ", t.Finr)
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " pln_forecast_formula.* FROM pln_forecast_formula " + where + " AND (convert(varchar(10),finr)+convert(varchar(40),code) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),code) FROM pln_forecast_formula " + where + " ORDER BY code DESC)) ORDER BY code DESC").Find(&data)
	pcount := new(PlnForecastFormula)
	count, err := table.Count(pcount)
	if err != nil {
		return data, 0, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, int(count), nil
}

//查询当前PlnForecastFormulas集合
func (t *PlnForecastFormula) SelectArr() ([]PlnForecastFormula, error) {
	e := db.Eloquent.Master()
	data := make([]PlnForecastFormula,0)
	err := e.Where("finr = ?", t.Finr).Find(&data)
	if err != nil {
		return data, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, nil
}
