package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"fmt"
	"github.com/tealeg/xlsx"
	"sort"
	"strings"
	"time"
)

type PlnForecastItemDailydemand struct {
	Finr        int    `xorm:"pk comment('工厂号') Int" json:"pln_forecast_item_dailydemand-finr"`
	Artnr       string `xorm:"pk comment('产品ID') VARCHAR(20)" json:"pln_forecast_item_dailydemand-artnr"`
	Forecastid  string `xorm:"pk comment('流水号ID') VARCHAR(20)" json:"pln_forecast_item_dailydemand-forecastid"`
	Demanddate  string `xorm:"pk comment('项目ID') VARCHAR(20)" json:"pln_forecast_item_dailydemand-demanddate"`
	Custplantnr string `xorm:"comment('工厂号') VARCHAR(20)" json:"pln_forecast_item_dailydemand-custplantnr"`
	Demandqty   int    `xorm:"comment('描述1') Int" json:"pln_forecast_item_dailydemand-demandqty"`
	Demandyear  int    `xorm:"comment('年') Int" json:"pln_forecast_item_dailydemand-demandyear"`
	Demandmonth int    `xorm:"comment('月') Int" json:"pln_forecast_item_dailydemand-demandmonth"`
	Demandweek  int    `xorm:"comment('周') Int" json:"pln_forecast_item_dailydemand-demandweek"`
	Lastmodif   string `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"pln_forecast_item_dailydemand-lastmodif"`
	Lastuser    string `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"pln_forecast_item_dailydemand-lastuser"`
	Credatuz    string `xorm:"comment('创建时间') VARCHAR(14)" json:"pln_forecast_item_dailydemand-credatuz"`
	Descr       string `json:"pln_forecast_item_dailydemand-descr" xorm:"-"`
}

func (t *PlnForecastItemDailydemand) TableName() string {
	return "pln_forecast_item_dailydemand"
}

// 清除string字段的右侧空格
func (t *PlnForecastItemDailydemand) Clipped() {
	utils.TrimStruct(t, *t)
}

//分页(月报)
func (t *PlnForecastItemDailydemand) GetPage(pageSize int, pageIndex int, timedate string,articles string) ([]PlnForecastArticle, int, error) {
	artdata := make([]PlnForecastArticle, 0)
	data := make([]PlnForecastItemDailydemand, 0)
	engine := db.Eloquent.Master()
	var arrs []string

	table := engine.Table("pln_forecast_article").Where("finr = ? ", t.Finr)
	wherestr := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if !utils.ValueIsEmpty(t.Artnr) {
		where += " and artnr = " + "'" + utils.ValueToString(t.Artnr, "") + "'"
	}
	if !utils.ValueIsEmpty(articles){
		artnrs := strings.Split(articles,",")
		artnrsStr := utils.IDsToStrings(artnrs)
		where += " and artnr in (" + artnrsStr + ")"
		wherestr += " and artnr in (" + artnrsStr + ")"
		table = table.In("artnr",artnrs)
	}
	if t.Demandyear > 0 {
		where += " and demandyear = " + "'" + utils.ValueToString(t.Demandyear, "") + "'"
	} else {
		t.Demandyear = utils.ValueToInt(utils.TimeFormat(time.Now(), "yyyy"), 0)
	}

	if !utils.ValueIsEmpty(t.Forecastid) {
		where += " and forecastid = " + "'" + utils.ValueToString(t.Forecastid, "") + "'"
	}
	if !utils.ValueIsEmpty(timedate) {
		datatime := strings.Split(timedate, "_")
		where += " and demanddate >= " + "'" + utils.ValueToString(datatime[0], "") + "'"
		where += " and demanddate <= " + "'" + utils.ValueToString(datatime[1], "") + "'"
		arrs = utils.SelectMonth(datatime[0], datatime[1])
	} else {
		arrs = []string{"01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"}
		for k, v := range arrs {
			val := utils.ValueToString(t.Demandyear, "") + "/" + utils.ValueToString(v, "")
			arrs[k] = val
		}
	}
	/******第一步：查询预测版本的所有产品信息，以 产品ID(artnr) +"-" +日期（demanddate）为Key，产品数量为值，构造数组********/
	err := engine.SQL("select artnr,forecastid,SUM(demandqty) as demandqty,demandyear,demandmonth from pln_forecast_item_dailydemand " + where + "  GROUP BY artnr,demandyear,demandmonth,forecastid").OrderBy("artnr,demandyear,demandmonth,forecastid").Find(&data)
	if err != nil {
		return artdata, 0, err
	}
	forecastData := make(map[string]int)
	for _, v := range data {
		v.Clipped()
		monthKey := ""
		if len(utils.ValueToString(v.Demandmonth, "")) == 1 {
			monthKey = utils.ValueToString(v.Demandyear, "") + "/0" + utils.ValueToString(v.Demandmonth, "")
		} else {
			monthKey = utils.ValueToString(v.Demandyear, "") + "/" + utils.ValueToString(v.Demandmonth, "")
		}
		key := v.Artnr + "-" + monthKey
		forecastData[key] = v.Demandqty
	}

	/******第二步：查询历史记录，以 产品ID(artnr) +"-" +月份（demandmonth）为Key，产品数量为值，构造数组********/
	dayArts := new(PlnForecastArticleTotal)
	dayArts.Finr = t.Finr
	dayArts.Artnr = t.Artnr
	totalMonths, _ := dayArts.SelectAllMonth(timedate)
	//解析成K=>V的形式
	totalKeys := make(map[string]int)
	for _, v := range totalMonths {
		v.Clipped()
		Demandmonth := ""
		if len(utils.ValueToString(v.Demandmonth, "")) == 1 {
			Demandmonth = utils.ValueToString(v.Demandyear, "") + "/0" + utils.ValueToString(v.Demandmonth, "")
		} else {
			Demandmonth = utils.ValueToString(v.Demandyear, "") + "/" + utils.ValueToString(v.Demandmonth, "")
		}
		key := v.Artnr + "-" + Demandmonth
		val := v.Demandqty
		totalKeys[key] = val
	}
	/******第三步：分页查询产品列表**********/
	Offset := (pageIndex - 1) * pageSize
	err = engine.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " pln_forecast_article.* FROM pln_forecast_article " + wherestr + " AND (convert(varchar(10),finr)+convert(varchar(40),artnr) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),artnr) FROM pln_forecast_article " + wherestr + " ORDER BY arttype DESC)) ORDER BY arttype DESC").Find(&artdata)
	if err != nil {
		fmt.Println(err)
		return nil, 0, err
	}
	pcount := new(PlnForecastItemDailydemand)
	count, err := table.Count(pcount)
	if err != nil {
		return nil, 0, err
	}
	for k, v := range artdata {
		artdata[k].Clipped()
		v.Clipped()
		datearrs := make(map[string]int)
		for _, vv := range arrs {
			key := utils.ValueToString(vv, "")
			datearrs[key] = 0
			temKey := v.Artnr + "-" + key
			//判断预测列表里是否存在，存在就替换
			val, ok := forecastData[temKey]
			if ok {
				datearrs[key] = val
			}else{
				//判断历史记录里是否存在，存在就替换
				value, ok := totalKeys[temKey]
				if ok {
					datearrs[key] = value
				}
			}
		}
		artdata[k].Info = append(artdata[k].Info, datearrs)
	}

	return artdata, int(count), nil
}

//分页(日报)
func (t *PlnForecastItemDailydemand) GetDayPage(pageSize int, pageIndex int, timedate string,articles string) ([]PlnForecastArticle, int, error) {
	artdata := make([]PlnForecastArticle, 0)
	data := make([]PlnForecastItemDailydemand, 0)
	engine := db.Eloquent.Master()
	var arrs []string

	table := engine.Table("pln_forecast_article").Where("finr = ? ", t.Finr)

	wherestr := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"

	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if !utils.ValueIsEmpty(t.Artnr) {
		where += " and artnr = " + "'" + utils.ValueToString(t.Artnr, "") + "'"
		table = table.And("artnr =  ?", t.Artnr)
		wherestr += " and artnr = " + "'" + utils.ValueToString(t.Artnr, "") + "'"
	}
	if t.Demandyear > 0 {
		where += " and demandyear = " + "'" + utils.ValueToString(t.Demandyear, "") + "'"
	} else {
		t.Demandyear = utils.ValueToInt(utils.TimeFormat(time.Now(), "yyyy"), 0)
	}
	if !utils.ValueIsEmpty(articles){
		artnrs := strings.Split(articles,",")
		artnrsStr := utils.IDsToStrings(artnrs)
		where += " and artnr in (" + artnrsStr + ")"
		wherestr += " and artnr in (" + artnrsStr + ")"
		table = table.In("artnr",artnrs)
	}
	if !utils.ValueIsEmpty(t.Forecastid) {
		where += " and forecastid = " + "'" + utils.ValueToString(t.Forecastid, "") + "'"
	}
	//查询出以日期的数组集合
	if !utils.ValueIsEmpty(timedate) {
		datatime := strings.Split(timedate, "_")
		where += " and demanddate >= " + "'" + utils.ValueToString(datatime[0], "") + "'"
		where += " and demanddate <= " + "'" + utils.ValueToString(datatime[1], "") + "'"
		arrs = utils.SelectDate(datatime[0], datatime[1])
	} else {
		firstmonth, lastmonth := utils.TimeReturnMonth(time.Now())
		arrs = utils.SelectDate(firstmonth, lastmonth)
	}
	/******第一步：查询预测版本的所有产品信息，以 产品ID(artnr) +"-" +日期（demanddate）为Key，产品数量为值，构造数组********/
	err := engine.SQL("select artnr,forecastid,SUM(demandqty) as demandqty,demandyear,demandmonth,demanddate from pln_forecast_item_dailydemand " + where + "  GROUP BY artnr,demandyear,demandmonth,demanddate,forecastid").OrderBy("artnr,demandyear,demandmonth,demanddate,forecastid").Find(&data)
	if err != nil {
		return artdata, 0, err
	}
	forecastData := make(map[string]int)
	for _, v := range data {
		v.Clipped()
		key := v.Artnr + "-" + v.Demanddate
		val := v.Demandqty
		forecastData[key] = val
	}
	/******第二步：查询历史记录，以 产品ID(artnr) +"-" +日期（demanddate）为Key，产品数量为值，构造数组********/
	dayArts := new(PlnForecastArticleTotal)
	dayArts.Finr = t.Finr
	dayArts.Artnr = t.Artnr
	totalDays, _ := dayArts.SelectAllDay(timedate)
	//解析成K=>V的形式
	totalKeys := make(map[string]int)
	for _, v := range totalDays {
		key := v.Artnr + "-" + v.Demanddate
		val := v.Demandqty
		totalKeys[key] = val
	}
	/******第三步：分页查询产品列表**********/
	Offset := (pageIndex - 1) * pageSize
	err = engine.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " pln_forecast_article.* FROM pln_forecast_article " + wherestr + " AND (convert(varchar(10),finr)+convert(varchar(40),artnr) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),artnr) FROM pln_forecast_article " + wherestr + " ORDER BY arttype DESC)) ORDER BY arttype DESC").Find(&artdata)
	if err != nil {
		fmt.Println(err)
		return nil, 0, err
	}
	pcount := new(PlnForecastItemDailydemand)
	count, err := table.Count(pcount)
	if err != nil {
		return nil, 0, err
	}
	/******第四步：构造出想要的数组**********/
	for k, v := range artdata {
		artdata[k].Clipped()
		v.Clipped()
		datearrs := make(map[string]int)
		for _, vv := range arrs {
			key := utils.ValueToString(vv, "")
			datearrs[key] = 0
			temKey := v.Artnr + "-" + key
			//判断预测列表里是否存在，存在就替换
			val, ok := forecastData[temKey]
			if ok {
				datearrs[key] = val
			}else{
				//判断历史记录里是否存在，存在就替换
				value, ok := totalKeys[temKey]
				if ok {
					datearrs[key] = value
				}
			}
		}
		artdata[k].Info = append(artdata[k].Info, datearrs)
	}

	return artdata, int(count), nil
}

//树状图统计
func (t *PlnForecastItemDailydemand) Dendrogram(forecastids []string, timedate string) (map[string]interface{}, error) {
	data := make([]PlnForecastItemDailydemand, 0)
	datatem := map[string][]PlnForecastItemDailydemand{}
	datalist := make(map[string]interface{})
	datalast := make(map[string]interface{})
	var arrs []string
	engine := db.Eloquent.Master()

	//查询版本号
	desc := new(PlnForecastLanding)
	desc.Finr = t.Finr

	descrs, _ := desc.SelectArr(forecastids)
	desMap := make(map[string]string)
	for _, v := range descrs {
		v.Clipped()
		key := v.Forecastid
		val := v.Descr
		desMap[key] = val
	}
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if !utils.ValueIsEmpty(t.Artnr) {
		where += " and artnr = " + "'" + utils.ValueToString(t.Artnr, "") + "'"
	}

	if !utils.ValueIsEmpty(timedate) {
		datatime := strings.Split(timedate, "_")
		where += " and demanddate >= " + "'" + utils.ValueToString(datatime[0], "") + "'"
		where += " and demanddate <= " + "'" + utils.ValueToString(datatime[1], "") + "'"
		arrs = utils.SelectMonth(datatime[0], datatime[1])
	} else {
		t.Demandyear = utils.ValueToInt(utils.TimeFormat(time.Now(), "yyyy"), 0)
		arrs = []string{"01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"}
		for k, v := range arrs {
			val := utils.ValueToString(t.Demandyear, "") + "/" + utils.ValueToString(v, "")
			arrs[k] = val
		}
	}
	if !utils.ValueIsEmpty(forecastids) {
		ids := utils.IDsToStrings(forecastids)
		where += " and forecastid in (" + ids + ")"
	}
	err := engine.SQL("select artnr,forecastid,SUM(demandqty) as demandqty,demandyear,demandmonth from pln_forecast_item_dailydemand " + where + "  GROUP BY artnr,demandyear,demandmonth,forecastid").OrderBy("artnr,demandyear,demandmonth,forecastid").Find(&data)
	if err != nil {
		return nil, err
	}

	for _, v := range data {
		v.Clipped()
		key := v.Forecastid
		datatem[key] = append(datatem[key], v)
	}
	for kk, vv := range datatem {
		key := kk
		datearrs := make(map[string]interface{})
		for _, v := range arrs {
			key := utils.ValueToString(v, "")
			datearrs[key] = 0
		}
		for _, vvv := range vv {
			valkey := ""
			if len(utils.ValueToString(vvv.Demandmonth, "")) == 1 {
				valkey = utils.ValueToString(vvv.Demandyear, "") + "/0" + utils.ValueToString(vvv.Demandmonth, "")
			} else {
				valkey = utils.ValueToString(vvv.Demandyear, "") + "/" + utils.ValueToString(vvv.Demandmonth, "")
			}

			_, ok := datearrs[valkey]
			if ok {
				datearrs[valkey] = vvv.Demandqty
			}
		}
		datalist[key] = datearrs
	}
	for _, v := range forecastids {
		_, ok := datalist[v]
		if !ok {
			datearrs := make(map[string]interface{})
			for _, v := range arrs {
				key := utils.ValueToString(v, "")
				datearrs[key] = 0
			}
			datalist[v] = datearrs
		}
	}
	for k, v := range datalist {
		val, ok := desMap[k]
		if ok {
			key := val
			datalast[key] = v
		}

	}
	return datalast, err
}

type TakeData struct {
	Qty      int
	Takerate string
}

//统计
func (t *PlnForecastItemDailydemand) GetTotal(articles string) ([]PlnForecastArticle, error) {
	artdata := make([]PlnForecastArticle, 0)
	dataArr := make([]PlnForecastArticle, 0)
	data := make([]PlnForecastItemDailydemand, 0)
	timedate := ""
	engine := db.Eloquent.Master()
	var arrs []string

	table := engine.Table("pln_forecast_article").Where("finr = ? ", t.Finr)
	where := " where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if !utils.ValueIsEmpty(t.Artnr) {
		where += " and artnr = " + "'" + utils.ValueToString(t.Artnr, "") + "'"
		table = table.And("artnr = ?",t.Artnr)
	}
	if !utils.ValueIsEmpty(articles){
		artnrs := strings.Split(articles,",")
		artnrsStr := utils.IDsToStrings(artnrs)
		where += " and artnr in (" + artnrsStr + ")"
		table = table.In("artnr",artnrs)
	}
	if t.Demandyear > 0 {
		where += " and demandyear = " + "'" + utils.ValueToString(t.Demandyear, "") + "'"
	} else {
		t.Demandyear = utils.ValueToInt(utils.TimeFormat(time.Now(), "yyyy"), 0)
	}

	if !utils.ValueIsEmpty(t.Forecastid) {
		where += " and forecastid = " + "'" + utils.ValueToString(t.Forecastid, "") + "'"
	}
	if !utils.ValueIsEmpty(timedate) {
		datatime := strings.Split(timedate, "_")
		where += " and demanddate >= " + "'" + utils.ValueToString(datatime[0], "") + "'"
		where += " and demanddate <= " + "'" + utils.ValueToString(datatime[1], "") + "'"
		arrs = utils.SelectMonth(datatime[0], datatime[1])
	} else {
		arrs = []string{"01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"}
		for k, v := range arrs {
			val := utils.ValueToString(t.Demandyear, "") + "/" + utils.ValueToString(v, "")
			arrs[k] = val
		}
	}
	/******第一步：查询预测版本的所有产品信息，以 产品ID(artnr) +"-" +日期（demanddate）为Key，产品数量为值，构造数组********/
	err := engine.SQL("select artnr,forecastid,SUM(demandqty) as demandqty,demandyear,demandmonth from pln_forecast_item_dailydemand " + where + "  GROUP BY artnr,demandyear,demandmonth,forecastid").OrderBy("artnr,demandyear,demandmonth,forecastid").Find(&data)
	if err != nil {
		return artdata, err
	}
	forecastData := make(map[string]int)
	for _, v := range data {
		v.Clipped()
		monthKey := ""
		if len(utils.ValueToString(v.Demandmonth, "")) == 1 {
			monthKey = utils.ValueToString(v.Demandyear, "") + "/0" + utils.ValueToString(v.Demandmonth, "")
		} else {
			monthKey = utils.ValueToString(v.Demandyear, "") + "/" + utils.ValueToString(v.Demandmonth, "")
		}
		key := v.Artnr + "-" + monthKey
		forecastData[key] = v.Demandqty
	}

	/******第二步：查询历史记录，以 产品ID(artnr) +"-" +月份（demandmonth）为Key，产品数量为值，构造数组********/
	dayArts := new(PlnForecastArticleTotal)
	dayArts.Finr = t.Finr
	dayArts.Artnr = t.Artnr
	totalMonths, _ := dayArts.SelectAllMonth(timedate)
	//解析成K=>V的形式
	totalKeys := make(map[string]int)
	for _, v := range totalMonths {
		v.Clipped()
		Demandmonth := ""
		if len(utils.ValueToString(v.Demandmonth, "")) == 1 {
			Demandmonth = utils.ValueToString(v.Demandyear, "") + "/0" + utils.ValueToString(v.Demandmonth, "")
		} else {
			Demandmonth = utils.ValueToString(v.Demandyear, "") + "/" + utils.ValueToString(v.Demandmonth, "")
		}
		key := v.Artnr + "-" + Demandmonth
		val := v.Demandqty
		totalKeys[key] = val
	}
	/******第三步：分页查询产品列表**********/
	//查询所有的产品
	err = table.Desc("arttype").Find(&artdata)
	if err != nil {
		return nil, err
	}
	for k, v := range artdata {
		artdata[k].Clipped()
		v.Clipped()
		datearrs := make(map[string]int)
		for _, vv := range arrs {
			key := utils.ValueToString(vv, "")
			datearrs[key] = 0
			temKey := v.Artnr + "-" + key
			//判断预测列表里是否存在，存在就替换
			val, ok := forecastData[temKey]
			if ok {
				datearrs[key] = val
			}else{
				//判断历史记录里是否存在，存在就替换
				value, ok := totalKeys[temKey]
				if ok {
					datearrs[key] = value
				}
			}
		}
		artdata[k].Info = append(artdata[k].Info, datearrs)
	}
	for _, v := range artdata {
		v.Clipped()
		if utils.ValueToInt(v.Level, 0) != 1 {
			continue
		}
		val := DiGuiTakeRate(&artdata, v)
		dataArr = append(dataArr, val)
	}
	return dataArr, nil
}

//递归算法
/***
*逻辑:递归算法
*第一个数据为父类产品，第二个数据为所有产品；
*通过循环筛选出父类的子类产品，
***********************/
func DiGuiTakeRate(datalist *[]PlnForecastArticle, item PlnForecastArticle) PlnForecastArticle {
	list := *datalist
	info := map[string]interface{}{}
	var fQty int

	for _, v := range item.Info {
		for kk, vv := range v {
			info[kk] = vv
			fQty += utils.ValueToInt(vv, 0)
		}

	}
	min := make([]PlnForecastArticle, 0)
	for j := 0; j < len(list); j++ {
		if item.Artnr != list[j].Fartnr {
			continue
		}
		mi := PlnForecastArticle{}
		mi.Finr = list[j].Finr
		mi.Artnr = list[j].Artnr
		mi.Descr = list[j].Descr
		mi.Linfo = make([]map[string]TakeData, 0)
		for _, v := range list[j].Info {
			var qty int
			for kk, vv := range v {
				key := kk
				datatake := TakeData{}
				datatake.Qty = utils.ValueToInt(vv, 0)
				val, ok := info[kk]
				if ok {
					if utils.ValueToInt(val, 0) > 0 {
						floatval := utils.ValueToFloat(vv, 0.0) / utils.ValueToFloat(val, 0.0)
						datatake.Takerate = utils.Bar(floatval, 0)
					} else {
						floatval := 0.0
						datatake.Takerate = utils.Bar(floatval, 0)
					}

				} else {
					floatval := 0.0
					datatake.Takerate = utils.Bar(floatval, 0)
				}
				linfo := make(map[string]TakeData)
				linfo[key] = datatake
				mi.Linfo = append(mi.Linfo, linfo)
				qty += utils.ValueToInt(vv, 0)
			}
			datatake := TakeData{}
			datatake.Qty = qty
			if fQty > 0 {
				floatval := utils.ValueToFloat(qty, 0.0) / utils.ValueToFloat(fQty, 0.0)
				datatake.Takerate = utils.Bar(floatval, 0)
			} else {
				floatval := 0.0
				datatake.Takerate = utils.Bar(floatval, 0)
			}

			linfo := make(map[string]TakeData)
			tkey := "Total"
			linfo[tkey] = datatake
			mi.Linfo = append(mi.Linfo, linfo)
		}
		//计算
		min = append(min, mi)
	}
	item.Children = min
	return item
}

/**
*方法：打开报表,多维数组变成一维数组
*结构体：TakeRate
***/
type TakeRate struct {
	Year      string `json:"year"`
	ParentArt string `json:"parent_art"`
	Art       string `json:"art"`
	Rate      string `json:"rate"`
	Jan       string `json:"jan"`
	Feb       string `json:"feb"`
	Mar       string `json:"mar"`
	Apr       string `json:"apr"`
	May       string `json:"may"`
	Jun       string `json:"jun"`
	Jul       string `json:"jul"`
	Aug       string `json:"aug"`
	Sep       string `json:"sep"`
	Oct       string `json:"oct"`
	Nov       string `json:"nov"`
	Dec       string `json:"dec"`
	Total     string `json:"total"`
}

func (t *PlnForecastItemDailydemand) TotalToDealWith(articles string) ([]TakeRate, error) {
	data := make([]TakeRate, 0)
	m := new(PlnForecastItemDailydemand)
	m.Forecastid = t.Forecastid
	m.Finr = t.Finr
	if t.Demandyear > 0 {
		m.Demandyear = t.Demandyear
	} else {
		m.Demandyear = utils.ValueToInt(utils.TimeFormat(time.Now(), "yyyy"), 0)
	}
	datalist, err := m.GetTotal(articles)
	if err != nil {
		return data, err
	}
	for _, v := range datalist {
		if !utils.ValueIsEmpty(v.Children) {
			//先添加子产品
			for _, vv := range v.Children {
				//生成两条数据，一条是数量，一条是rate
				dataqty := TakeRate{}
				dataqty.Year = utils.ValueToString(m.Demandyear, "")
				dataqty.ParentArt = v.Descr
				dataqty.Art = vv.Descr
				dataqty.Rate = "Qty"

				datarate := TakeRate{}
				datarate.Year = utils.ValueToString(m.Demandyear, "")
				datarate.ParentArt = v.Descr
				datarate.Art = vv.Descr
				datarate.Rate = "Takerate"
				for _, vvv := range vv.Linfo {
					for kkkk, vvvv := range vvv {
						if strings.Contains(kkkk, "/01") {
							dataqty.Jan = utils.ValueToString(vvvv.Qty, "")
							datarate.Jan = utils.ValueToString(vvvv.Takerate, "")
						} else if strings.Contains(kkkk, "/02") {
							dataqty.Feb = utils.ValueToString(vvvv.Qty, "")
							datarate.Feb = utils.ValueToString(vvvv.Takerate, "")
						} else if strings.Contains(kkkk, "/03") {
							dataqty.Mar = utils.ValueToString(vvvv.Qty, "")
							datarate.Mar = utils.ValueToString(vvvv.Takerate, "")
						} else if strings.Contains(kkkk, "/04") {
							dataqty.Apr = utils.ValueToString(vvvv.Qty, "")
							datarate.Apr = utils.ValueToString(vvvv.Takerate, "")
						} else if strings.Contains(kkkk, "/05") {
							dataqty.May = utils.ValueToString(vvvv.Qty, "")
							datarate.May = utils.ValueToString(vvvv.Takerate, "")
						} else if strings.Contains(kkkk, "/06") {
							dataqty.Jun = utils.ValueToString(vvvv.Qty, "")
							datarate.Jun = utils.ValueToString(vvvv.Takerate, "")
						} else if strings.Contains(kkkk, "/07") {
							dataqty.Jul = utils.ValueToString(vvvv.Qty, "")
							datarate.Jul = utils.ValueToString(vvvv.Takerate, "")
						} else if strings.Contains(kkkk, "/08") {
							dataqty.Aug = utils.ValueToString(vvvv.Qty, "")
							datarate.Aug = utils.ValueToString(vvvv.Takerate, "")
						} else if strings.Contains(kkkk, "/09") {
							dataqty.Sep = utils.ValueToString(vvvv.Qty, "")
							datarate.Sep = utils.ValueToString(vvvv.Takerate, "")
						} else if strings.Contains(kkkk, "/10") {
							dataqty.Oct = utils.ValueToString(vvvv.Qty, "")
							datarate.Oct = utils.ValueToString(vvvv.Takerate, "")
						} else if strings.Contains(kkkk, "/11") {
							dataqty.Nov = utils.ValueToString(vvvv.Qty, "")
							datarate.Nov = utils.ValueToString(vvvv.Takerate, "")
						} else if strings.Contains(kkkk, "/12") {
							dataqty.Dec = utils.ValueToString(vvvv.Qty, "")
							datarate.Dec = utils.ValueToString(vvvv.Takerate, "")
						} else if strings.Contains(kkkk, "Total") {
							dataqty.Total = utils.ValueToString(vvvv.Qty, "")
							datarate.Total = utils.ValueToString(vvvv.Takerate, "")
						}
					}

				}
				data = append(data, dataqty)
				data = append(data, datarate)
			}

		}
		//添加父类产品
		datainfo := TakeRate{}
		datainfo.Year = utils.ValueToString(m.Demandyear, "")
		datainfo.ParentArt = v.Descr
		datainfo.Art = "Total Demand"
		datainfo.Rate = "Total"
		infototal := 0
		for _, vv := range v.Info {
			for kkkk, vvvv := range vv {
				if strings.Contains(kkkk, "/01") {
					datainfo.Jan = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToInt(vvvv, 0)
				} else if strings.Contains(kkkk, "/02") {
					datainfo.Feb = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToInt(vvvv, 0)
				} else if strings.Contains(kkkk, "/03") {
					datainfo.Mar = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToInt(vvvv, 0)
				} else if strings.Contains(kkkk, "/04") {
					datainfo.Apr = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToInt(vvvv, 0)
				} else if strings.Contains(kkkk, "/05") {
					datainfo.May = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToInt(vvvv, 0)
				} else if strings.Contains(kkkk, "/06") {
					datainfo.Jun = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToInt(vvvv, 0)
				} else if strings.Contains(kkkk, "/07") {
					datainfo.Jul = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToInt(vvvv, 0)
				} else if strings.Contains(kkkk, "/08") {
					datainfo.Aug = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToInt(vvvv, 0)
				} else if strings.Contains(kkkk, "/09") {
					datainfo.Sep = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToInt(vvvv, 0)
				} else if strings.Contains(kkkk, "/10") {
					datainfo.Oct = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToInt(vvvv, 0)
				} else if strings.Contains(kkkk, "/11") {
					datainfo.Nov = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToInt(vvvv, 0)
				} else if strings.Contains(kkkk, "/12") {
					datainfo.Dec = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToInt(vvvv, 0)
				}
			}

		}
		datainfo.Total = utils.ValueToString(infototal, "")
		data = append(data, datainfo)
	}

	return data, nil
}

//月报统计
func (t *PlnForecastItemDailydemand) GetMonthTotal(timedate string,articles string) ([]PlnForecastArticle, error) {
	artdata := make([]PlnForecastArticle, 0)
	data := make([]PlnForecastItemDailydemand, 0)
	engine := db.Eloquent.Master()
	var arrs []string

	table := engine.Table("pln_forecast_article").Where("finr = ? ", t.Finr)
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if !utils.ValueIsEmpty(t.Artnr) {
		where += " and artnr = " + "'" + utils.ValueToString(t.Artnr, "") + "'"
	}
	if t.Demandyear > 0 {
		where += " and demandyear = " + "'" + utils.ValueToString(t.Demandyear, "") + "'"
	} else {
		t.Demandyear = utils.ValueToInt(utils.TimeFormat(time.Now(), "yyyy"), 0)
	}
	if !utils.ValueIsEmpty(articles){
		artnrs := strings.Split(articles,",")
		artnrsStr := utils.IDsToStrings(artnrs)
		where += " and artnr in (" + artnrsStr + ")"
		table = table.In("artnr",artnrs)
	}
	if !utils.ValueIsEmpty(t.Forecastid) {
		where += " and forecastid = " + "'" + utils.ValueToString(t.Forecastid, "") + "'"
	}
	if !utils.ValueIsEmpty(timedate) {
		datatime := strings.Split(timedate, "_")
		where += " and demanddate >= " + "'" + utils.ValueToString(datatime[0], "") + "'"
		where += " and demanddate <= " + "'" + utils.ValueToString(datatime[1], "") + "'"
		arrs = utils.SelectMonth(datatime[0], datatime[1])
	} else {
		arrs = []string{"01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"}
		for k, v := range arrs {
			val := utils.ValueToString(t.Demandyear, "") + "/" + utils.ValueToString(v, "")
			arrs[k] = val
		}
	}
	/******第一步：查询预测版本的所有产品信息，以 产品ID(artnr) +"-" +日期（demanddate）为Key，产品数量为值，构造数组********/
	err := engine.SQL("select artnr,forecastid,SUM(demandqty) as demandqty,demandyear,demandmonth from pln_forecast_item_dailydemand " + where + "  GROUP BY artnr,demandyear,demandmonth,forecastid").OrderBy("artnr,demandyear,demandmonth,forecastid").Find(&data)
	if err != nil {
		return artdata, err
	}
	forecastData := make(map[string]int)
	for _, v := range data {
		v.Clipped()
		monthKey := ""
		if len(utils.ValueToString(v.Demandmonth, "")) == 1 {
			monthKey = utils.ValueToString(v.Demandyear, "") + "/0" + utils.ValueToString(v.Demandmonth, "")
		} else {
			monthKey = utils.ValueToString(v.Demandyear, "") + "/" + utils.ValueToString(v.Demandmonth, "")
		}
		key := v.Artnr + "-" + monthKey
		forecastData[key] = v.Demandqty
	}

	/******第二步：查询历史记录，以 产品ID(artnr) +"-" +月份（demandmonth）为Key，产品数量为值，构造数组********/
	dayArts := new(PlnForecastArticleTotal)
	dayArts.Finr = t.Finr
	dayArts.Artnr = t.Artnr
	totalMonths, _ := dayArts.SelectAllMonth(timedate)
	//解析成K=>V的形式
	totalKeys := make(map[string]int)
	for _, v := range totalMonths {
		v.Clipped()
		Demandmonth := ""
		if len(utils.ValueToString(v.Demandmonth, "")) == 1 {
			Demandmonth = utils.ValueToString(v.Demandyear, "") + "/0" + utils.ValueToString(v.Demandmonth, "")
		} else {
			Demandmonth = utils.ValueToString(v.Demandyear, "") + "/" + utils.ValueToString(v.Demandmonth, "")
		}
		key := v.Artnr + "-" + Demandmonth
		val := v.Demandqty
		totalKeys[key] = val
	}
	/******第三步：分页查询产品列表**********/
	//查询所有的产品
	err = table.Desc("custplantnr","arttype").Find(&artdata)
	if err != nil {
		return nil, err
	}
	for k, v := range artdata {
		artdata[k].Clipped()
		v.Clipped()
		datearrs := make(map[string]int)
		for _, vv := range arrs {
			key := utils.ValueToString(vv, "")
			datearrs[key] = 0
			temKey := v.Artnr + "-" + key
			//判断预测列表里是否存在，存在就替换
			val, ok := forecastData[temKey]
			if ok {
				datearrs[key] = val
			}else{
				//判断历史记录里是否存在，存在就替换
				value, ok := totalKeys[temKey]
				if ok {
					datearrs[key] = value
				}
			}
		}
		artdata[k].Info = append(artdata[k].Info, datearrs)
	}
	return artdata, nil
}

//导出月报
func (t *PlnForecastItemDailydemand) ReadExcelMonthdemand(timedate string,articles string) (string, string, error) {
	m := new(PlnForecastItemDailydemand)
	m.Finr = t.Finr
	m.Forecastid = t.Forecastid
	if t.Demandyear > 0 {
		m.Demandyear = t.Demandyear
	} else {
		m.Demandyear = utils.ValueToInt(utils.TimeFormat(time.Now(), "yyyy"), 0)
	}
	datalist, err := m.GetMonthTotal(timedate,articles)
	if err != nil {
		return "", "", err
	}
	var file *xlsx.File
	var sheet *xlsx.Sheet
	var row *xlsx.Row
	var cell *xlsx.Cell

	file = xlsx.NewFile()
	sheet, err = file.AddSheet("Sheet1")
	if err != nil {
		return "", "", err
	}

	tableHead := true

	//构建一个有序的map

	var sortKeys []string
	for _, val := range datalist {
		if tableHead {
			//添加表头
			row = sheet.AddRow()
			//产品标识
			cell = row.AddCell()
			cell.Value = "产品标识"
			//描述
			cell = row.AddCell()
			cell.Value = "描述"
			for _, v := range val.Info {
				for kk, _ := range v {
					sortKeys = append(sortKeys, kk)
				}
				//对key值排序
				sort.Strings(sortKeys)
				for _, head := range sortKeys {
					//日期表头
					cell = row.AddCell()
					cell.Value = head
				}
			}

			tableHead = false
		}
		row = sheet.AddRow()
		//产品标识
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Artnr, "")
		//描述
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Descr, "")
		//规格商品名称
		for _, v := range val.Info {

			for _, head := range sortKeys {
				//按照sortKeys,顺序取值
				cell = row.AddCell()
				cell.Value = utils.ValueToString(v[head], "")
			}
		}
	}
	name := utils.MakeOrderSn("月报")
	filename := name + ".xlsx" //文件名

	exceldir,_ := utils.GetCurrentPath("web/public/uploadxlsx") // 目录+文件名
	excelfile := exceldir + "/" + filename
	err = file.Save(excelfile)
	if err != nil {
		return "", "", err
	}
	return excelfile, filename, nil
}

//日报统计
func (t *PlnForecastItemDailydemand) GetDayTotal(timedate string,articles string) ([]PlnForecastArticle, error) {
	artdata := make([]PlnForecastArticle, 0)
	data := make([]PlnForecastItemDailydemand, 0)
	engine := db.Eloquent.Master()
	var arrs []string

	table := engine.Table("pln_forecast_article").Where("finr = ? ", t.Finr)
	wherestr := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"

	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if !utils.ValueIsEmpty(t.Artnr) {
		where += " and artnr = " + "'" + utils.ValueToString(t.Artnr, "") + "'"
		table = table.And("artnr =  ?", t.Artnr)
		wherestr += " and artnr = " + "'" + utils.ValueToString(t.Artnr, "") + "'"
	}
	if t.Demandyear > 0 {
		where += " and demandyear = " + "'" + utils.ValueToString(t.Demandyear, "") + "'"
	} else {
		t.Demandyear = utils.ValueToInt(utils.TimeFormat(time.Now(), "yyyy"), 0)
	}
	if !utils.ValueIsEmpty(articles){
		artnrs := strings.Split(articles,",")
		artnrsStr := utils.IDsToStrings(artnrs)
		where += " and artnr in (" + artnrsStr + ")"
		table = table.In("artnr",artnrs)
	}
	if !utils.ValueIsEmpty(t.Forecastid) {
		where += " and forecastid = " + "'" + utils.ValueToString(t.Forecastid, "") + "'"
	}
	//查询出以日期的数组集合
	if !utils.ValueIsEmpty(timedate) {
		datatime := strings.Split(timedate, "_")
		where += " and demanddate >= " + "'" + utils.ValueToString(datatime[0], "") + "'"
		where += " and demanddate <= " + "'" + utils.ValueToString(datatime[1], "") + "'"
		arrs = utils.SelectDate(datatime[0], datatime[1])
	} else {
		firstmonth, lastmonth := utils.TimeReturnMonth(time.Now())
		arrs = utils.SelectDate(firstmonth, lastmonth)
	}
	/******第一步：查询预测版本的所有产品信息，以 产品ID(artnr) +"-" +日期（demanddate）为Key，产品数量为值，构造数组********/
	err := engine.SQL("select artnr,forecastid,SUM(demandqty) as demandqty,demandyear,demandmonth,demanddate from pln_forecast_item_dailydemand " + where + "  GROUP BY artnr,demandyear,demandmonth,demanddate,forecastid").OrderBy("artnr,demandyear,demandmonth,demanddate,forecastid").Find(&data)
	if err != nil {
		return artdata, err
	}
	forecastData := make(map[string]int)
	for _, v := range data {
		v.Clipped()
		key := v.Artnr + "-" + v.Demanddate
		val := v.Demandqty
		forecastData[key] = val
	}
	/******第二步：查询历史记录，以 产品ID(artnr) +"-" +日期（demanddate）为Key，产品数量为值，构造数组********/
	dayArts := new(PlnForecastArticleTotal)
	dayArts.Finr = t.Finr
	dayArts.Artnr = t.Artnr
	totalDays, _ := dayArts.SelectAllDay(timedate)
	//解析成K=>V的形式
	totalKeys := make(map[string]int)
	for _, v := range totalDays {
		key := v.Artnr + "-" + v.Demanddate
		val := v.Demandqty
		totalKeys[key] = val
	}
	/******第三步：分页查询产品列表**********/
	//查询所有的产品
	err = table.Desc("arttype").Find(&artdata)
	if err != nil {
		return nil, err
	}
	/******第四步：构造出想要的数组**********/
	for k, v := range artdata {
		artdata[k].Clipped()
		v.Clipped()
		datearrs := make(map[string]int)
		for _, vv := range arrs {
			key := utils.ValueToString(vv, "")
			datearrs[key] = 0
			temKey := v.Artnr + "-" + key
			//判断预测列表里是否存在，存在就替换
			val, ok := forecastData[temKey]
			if ok {
				datearrs[key] = val
			}else{
				//判断历史记录里是否存在，存在就替换
				value, ok := totalKeys[temKey]
				if ok {
					datearrs[key] = value
				}
			}
		}
		artdata[k].Info = append(artdata[k].Info, datearrs)
	}

	return artdata, nil
}

//导出日报
func (t *PlnForecastItemDailydemand) ReadExcelDaydemand(timedate string,articles string) (string, string, error) {
	m := new(PlnForecastItemDailydemand)
	m.Finr = t.Finr
	m.Forecastid = t.Forecastid
	datalist, err := m.GetDayTotal(timedate,articles)
	fmt.Printf("datalist", datalist)
	if err != nil {
		return "", "", err
	}
	var file *xlsx.File
	var sheet *xlsx.Sheet
	var row *xlsx.Row
	var cell *xlsx.Cell

	file = xlsx.NewFile()
	sheet, err = file.AddSheet("Sheet1")
	if err != nil {
		return "", "", err
	}

	tableHead := true

	//构建一个有序的map
	var sortKeys []string
	for _, val := range datalist {
		if tableHead {
			//添加表头
			row = sheet.AddRow()
			//产品标识
			cell = row.AddCell()
			cell.Value = "产品标识"
			//描述
			cell = row.AddCell()
			cell.Value = "描述"
			for _, v := range val.Info {
				for kk, _ := range v {
					sortKeys = append(sortKeys, kk)
				}
				//对key值排序
				sort.Strings(sortKeys)
				for _, head := range sortKeys {
					//日期表头
					cell = row.AddCell()
					cell.Value = head
				}
			}

			tableHead = false
		}
		row = sheet.AddRow()
		//产品标识
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Artnr, "")
		//描述
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Descr, "")
		//规格商品名称
		for _, v := range val.Info {

			for _, head := range sortKeys {
				//按照sortKeys,顺序取值
				cell = row.AddCell()
				cell.Value = utils.ValueToString(v[head], "")
			}
		}
	}
	name := utils.MakeOrderSn("日报")
	filename := name + ".xlsx" //文件名

	exceldir,_ := utils.GetCurrentPath("web/public/uploadxlsx") // 目录+文件名
	excelfile := exceldir + "/" + filename
	err = file.Save(excelfile)
	if err != nil {
		return "", "", err
	}
	return excelfile, filename, nil
}

//导出预测报表
func (t *PlnForecastItemDailydemand) ReadExcelDemand(articles string) (string, string, error) {
	m := new(PlnForecastItemDailydemand)
	m.Finr = t.Finr
	m.Forecastid = t.Forecastid
	if t.Demandyear > 0 {
		m.Demandyear = t.Demandyear
	} else {
		m.Demandyear = utils.ValueToInt(utils.TimeFormat(time.Now(), "yyyy"), 0)
	}
	datalist, err := m.TotalToDealWith(articles)
	if err != nil {
		return "", "", err
	}
	var file *xlsx.File
	var sheet *xlsx.Sheet
	var row *xlsx.Row
	var cell *xlsx.Cell

	file = xlsx.NewFile()
	sheet, err = file.AddSheet("Sheet1")
	if err != nil {
		return "", "", err
	}

	tableHead := true

	for _, val := range datalist {
		if tableHead {

			//设置表格标题
			row := sheet.AddRow()
			titleRow := row.AddCell()
			titleRow.HMerge = 15 //向右合并列数，不包括自身列
			titleRow.Value = "预测报表"

			//添加表头
			row = sheet.AddRow()
			//Monthly Total
			cell = row.AddCell()
			cell.HMerge = 2 //向右合并列数，不包括自身列
			cell.Value = "Monthly Total"

			//添加两行空白格，用于合并
			cell = row.AddCell()
			cell = row.AddCell()
			//一月
			cell = row.AddCell()
			cell.Value = val.Year + "/" + "Jan"

			//二月
			cell = row.AddCell()
			cell.Value = val.Year + "/" + "Feb"

			//三月
			cell = row.AddCell()
			cell.Value = val.Year + "/" + "Mar"

			//四月
			cell = row.AddCell()
			cell.Value = val.Year + "/" + "Apr"

			//五月
			cell = row.AddCell()
			cell.Value = val.Year + "/" + "May"

			//六月
			cell = row.AddCell()
			cell.Value = val.Year + "/" + "Jun"

			//七月
			cell = row.AddCell()
			cell.Value = val.Year + "/" + "Jul"

			//八月
			cell = row.AddCell()
			cell.Value = val.Year + "/" + "Aug"

			//九月
			cell = row.AddCell()
			cell.Value = val.Year + "/" + "Sep"

			//十月
			cell = row.AddCell()
			cell.Value = val.Year + "/" + "Oct"

			//十一月
			cell = row.AddCell()
			cell.Value = val.Year + "/" + "Nov"

			//十二月
			cell = row.AddCell()
			cell.Value = val.Year + "/" + "Dec"

			//统计
			cell = row.AddCell()
			cell.Value = "Total"

			tableHead = false
		}
		row = sheet.AddRow()
		//父产品() beforeFart
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.ParentArt, "")

		//子产品
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Art, "")

		//类别
		cell = row.AddCell()
		cell.Value = utils.ValueToString(val.Rate, "")

		//一月
		cell = row.AddCell()
		cell.Value = val.Jan

		//二月
		cell = row.AddCell()
		cell.Value = val.Feb

		//三月
		cell = row.AddCell()
		cell.Value = val.Mar

		//四月
		cell = row.AddCell()
		cell.Value = val.Apr

		//五月
		cell = row.AddCell()
		cell.Value = val.May

		//六月
		cell = row.AddCell()
		cell.Value = val.Jun

		//七月
		cell = row.AddCell()
		cell.Value = val.Jul

		//八月
		cell = row.AddCell()
		cell.Value = val.Aug

		//九月
		cell = row.AddCell()
		cell.Value = val.Sep

		//十月
		cell = row.AddCell()
		cell.Value = val.Oct

		//十一月
		cell = row.AddCell()
		cell.Value = val.Nov

		//十二月
		cell = row.AddCell()
		cell.Value = val.Dec

		//统计
		cell = row.AddCell()
		cell.Value = val.Total
	}
	name := utils.MakeOrderSn("预测统计")
	filename := name + ".xlsx" //文件名

	exceldir,_ := utils.GetCurrentPath("web/public/uploadxlsx") // 目录+文件名
	excelfile := exceldir + "/" + filename
	err = file.Save(excelfile)
	if err != nil {
		return "", "", err
	}
	return excelfile, filename, nil
}

/**
*方法：打开报表,多维数组变成一维数组（产能计算）
*结构体：TakeTotal
***/
type TakeTotal struct {
	Year        string `json:"year"`
	Custplantnr string `json:"custplantnr"`
	Art         string `json:"art"`
	Jan         string `json:"jan"`
	Feb         string `json:"feb"`
	Mar         string `json:"mar"`
	Apr         string `json:"apr"`
	May         string `json:"may"`
	Jun         string `json:"jun"`
	Jul         string `json:"jul"`
	Aug         string `json:"aug"`
	Sep         string `json:"sep"`
	Oct         string `json:"oct"`
	Nov         string `json:"nov"`
	Dec         string `json:"dec"`
	Total       string `json:"total"`
}

type MonthFirstToLast struct {
	FirstMonth string
	LastMonth  string
}

//产能预测报表
func (t *PlnForecastItemDailydemand) TotalToCustplantnr(timedate string,articles string) (map[string][]TakeTotal, error) {
	data := make(map[string][]TakeTotal)
	cTotal := make([]TakeTotal, 0) //日历统计
	tTotal := make([]TakeTotal, 0) //统计
	aTotal := make([]TakeTotal, 0) //平均统计

	months := make(map[string]MonthFirstToLast)
	m := new(PlnForecastItemDailydemand)
	m.Forecastid = t.Forecastid
	m.Finr = t.Finr
	if t.Demandyear > 0 {
		m.Demandyear = t.Demandyear
	} else {
		m.Demandyear = utils.ValueToInt(utils.TimeFormat(time.Now(), "yyyy"), 0)
	}
	//月统计数据
	datalist, err := m.GetMonthTotal(timedate,articles)
	if err != nil {
		return nil, err
	}

	//工作日历月统计
	arrs := []string{"01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"}
	for _, v := range arrs {
		val := MonthFirstToLast{}
		key := utils.ValueToString(m.Demandyear, "") + "/" + utils.ValueToString(v, "")
		temdate := utils.ValueToString(m.Demandyear, "") + utils.ValueToString(v, "") + "01"
		timeNow, _ := utils.TimeParseyyyyMMdd(temdate)
		first, last := utils.TimeReturnMonth(timeNow)
		val.FirstMonth = first
		val.LastMonth = last
		months[key] = val
	}
	fmt.Printf("months is :%v", months)

	//查询出所有的工厂id
	c := new(PlnForecastCalendar)
	c.Finr = t.Finr
	cdata, err := c.SelectAll()
	if err != nil {
		return nil, err
	}

	fmt.Printf("cdata is :%v", cdata)
	monthData := make(map[string]string)
	for _, vv := range cdata {
		dataone := TakeTotal{}
		dataone.Year = utils.ValueToString(m.Demandyear, "")
		dataone.Custplantnr = vv.Custplantnr
		//分组统计每个月份的有效时长
		infototal := 0.0
		for k, v := range months {
			//构建 工厂号 + 日期
			key := vv.Custplantnr + k
			//分组查询
			f := new(PlnForecastCalendarDaylst)
			f.Finr = t.Finr
			f.Calendarid = vv.Calendarid
			total, _ := f.SumEfffactor(v.FirstMonth, v.LastMonth)
			val := fmt.Sprintf("%0.2f", total)
			monthData[key] = val

			if strings.Contains(k, "/01") {
				dataone.Jan = utils.ValueToString(val, "")
				infototal += total
			} else if strings.Contains(k, "/02") {
				dataone.Feb = utils.ValueToString(val, "")
				infototal += total
			} else if strings.Contains(k, "/03") {
				dataone.Mar = utils.ValueToString(val, "")
				infototal += total
			} else if strings.Contains(k, "/04") {
				dataone.Apr = utils.ValueToString(val, "")
				infototal += total
			} else if strings.Contains(k, "/05") {
				dataone.May = utils.ValueToString(val, "")
				infototal += total
			} else if strings.Contains(k, "/06") {
				dataone.Jun = utils.ValueToString(val, "")
				infototal += total
			} else if strings.Contains(k, "/07") {
				dataone.Jul = utils.ValueToString(val, "")
				infototal += total
			} else if strings.Contains(k, "/08") {
				dataone.Aug = utils.ValueToString(val, "")
				infototal += total
			} else if strings.Contains(k, "/09") {
				dataone.Sep = utils.ValueToString(val, "")
				infototal += total
			} else if strings.Contains(k, "/10") {
				dataone.Oct = utils.ValueToString(val, "")
				infototal += total
			} else if strings.Contains(k, "/11") {
				dataone.Nov = utils.ValueToString(val, "")
				infototal += total
			} else if strings.Contains(k, "/12") {
				dataone.Dec = utils.ValueToString(val, "")
				infototal += total
			}
			dataone.Total = fmt.Sprintf("%0.2f", infototal)
		}
		cTotal = append(cTotal, dataone)
	}
	fmt.Printf("monthData is :%v", monthData)
	//计算统计  和 平均数
	for _, v := range datalist {
		datainfo := TakeTotal{}
		datainfo.Year = utils.ValueToString(m.Demandyear, "")
		datainfo.Custplantnr = v.Custplantnr
		datainfo.Art = v.Descr

		ainfo := TakeTotal{}
		ainfo.Year = utils.ValueToString(m.Demandyear, "")
		ainfo.Custplantnr = v.Custplantnr
		ainfo.Art = v.Descr

		infototal := 0.0
		atotal := 0.0
		for _, vv := range v.Info {
			for kkkk, vvvv := range vv {
				if strings.Contains(kkkk, "/01") {
					datainfo.Jan = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToFloat(vvvv, 0.0)

					//构建key值
					key := v.Custplantnr + kkkk
					val, ok := monthData[key]
					if ok {
						if utils.ValueToFloat(val, 0.0) > 0.0 {
							valinfo := utils.ValueToFloat(vvvv, 0.0) / utils.ValueToFloat(val, 0.0)
							ainfo.Jan = fmt.Sprintf("%0.0f", valinfo)
							atotal += utils.ValueToFloat(valinfo, 0.0)
						} else {
							ainfo.Jan = "0"
						}

					} else {
						ainfo.Jan = "0"
					}

				} else if strings.Contains(kkkk, "/02") {
					datainfo.Feb = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToFloat(vvvv, 0.0)

					//构建key值
					key := v.Custplantnr + kkkk
					val, ok := monthData[key]
					if ok {
						if utils.ValueToFloat(val, 0.0) > 0.0 {
							valinfo := utils.ValueToFloat(vvvv, 0.0) / utils.ValueToFloat(val, 0.0)
							ainfo.Feb = fmt.Sprintf("%0.0f", valinfo)
							atotal += utils.ValueToFloat(valinfo, 0.0)
						} else {
							ainfo.Feb = "0"
						}

					} else {
						ainfo.Feb = "0"
					}

				} else if strings.Contains(kkkk, "/03") {
					datainfo.Mar = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToFloat(vvvv, 0.0)

					//构建key值
					key := v.Custplantnr + kkkk
					val, ok := monthData[key]
					if ok {
						if utils.ValueToFloat(val, 0.0) > 0.0 {
							valinfo := utils.ValueToFloat(vvvv, 0.0) / utils.ValueToFloat(val, 0.0)
							ainfo.Mar = fmt.Sprintf("%0.0f", valinfo)
							atotal += utils.ValueToFloat(valinfo, 0.0)
						} else {
							ainfo.Mar = "0"
						}

					} else {
						ainfo.Mar = "0"
					}
				} else if strings.Contains(kkkk, "/04") {
					datainfo.Apr = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToFloat(vvvv, 0.0)

					//构建key值
					key := v.Custplantnr + kkkk
					val, ok := monthData[key]
					if ok {
						if utils.ValueToFloat(val, 0.0) > 0.0 {
							valinfo := utils.ValueToFloat(vvvv, 0.0) / utils.ValueToFloat(val, 0.0)
							ainfo.Apr = fmt.Sprintf("%0.0f", valinfo)
							atotal += utils.ValueToFloat(valinfo, 0.0)
						} else {
							ainfo.Apr = "0"
						}

					} else {
						ainfo.Apr = "0"
					}
				} else if strings.Contains(kkkk, "/05") {
					datainfo.May = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToFloat(vvvv, 0.0)

					//构建key值
					key := v.Custplantnr + kkkk
					val, ok := monthData[key]
					if ok {
						if utils.ValueToFloat(val, 0.0) > 0.0 {
							valinfo := utils.ValueToFloat(vvvv, 0.0) / utils.ValueToFloat(val, 0.0)
							ainfo.May = fmt.Sprintf("%0.0f", valinfo)
							atotal += utils.ValueToFloat(valinfo, 0.0)
						} else {
							ainfo.May = "0"
						}

					} else {
						ainfo.May = "0"
					}
				} else if strings.Contains(kkkk, "/06") {
					datainfo.Jun = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToFloat(vvvv, 0.0)

					//构建key值
					key := v.Custplantnr + kkkk
					val, ok := monthData[key]
					if ok {
						if utils.ValueToFloat(val, 0.0) > 0.0 {
							valinfo := utils.ValueToFloat(vvvv, 0.0) / utils.ValueToFloat(val, 0.0)
							ainfo.Jun = fmt.Sprintf("%0.0f", valinfo)
							atotal += utils.ValueToFloat(valinfo, 0.0)
						} else {
							ainfo.Jun = "0"
						}

					} else {
						ainfo.Jun = "0"
					}
				} else if strings.Contains(kkkk, "/07") {
					datainfo.Jul = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToFloat(vvvv, 0.0)

					//构建key值
					key := v.Custplantnr + kkkk
					val, ok := monthData[key]
					if ok {
						if utils.ValueToFloat(val, 0.0) > 0.0 {
							valinfo := utils.ValueToFloat(vvvv, 0.0) / utils.ValueToFloat(val, 0.0)
							ainfo.Jul = fmt.Sprintf("%0.0f", valinfo)
							atotal += utils.ValueToFloat(valinfo, 0.0)
						} else {
							ainfo.Jul = "0"
						}

					} else {
						ainfo.Jul = "0"
					}
				} else if strings.Contains(kkkk, "/08") {
					datainfo.Aug = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToFloat(vvvv, 0.0)

					//构建key值
					key := v.Custplantnr + kkkk
					val, ok := monthData[key]
					if ok {
						if utils.ValueToFloat(val, 0.0) > 0.0 {
							valinfo := utils.ValueToFloat(vvvv, 0.0) / utils.ValueToFloat(val, 0.0)
							ainfo.Aug = fmt.Sprintf("%0.0f", valinfo)
							atotal += utils.ValueToFloat(valinfo, 0.0)
						} else {
							ainfo.Aug = "0"
						}

					} else {
						ainfo.Aug = "0"
					}
				} else if strings.Contains(kkkk, "/09") {
					datainfo.Sep = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToFloat(vvvv, 0.0)

					//构建key值
					key := v.Custplantnr + kkkk
					val, ok := monthData[key]
					if ok {
						if utils.ValueToFloat(val, 0.0) > 0.0 {
							valinfo := utils.ValueToFloat(vvvv, 0.0) / utils.ValueToFloat(val, 0.0)
							ainfo.Sep = fmt.Sprintf("%0.0f", valinfo)
							atotal += utils.ValueToFloat(valinfo, 0.0)
						} else {
							ainfo.Sep = "0"
						}

					} else {
						ainfo.Sep = "0"
					}
				} else if strings.Contains(kkkk, "/10") {
					datainfo.Oct = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToFloat(vvvv, 0.0)

					//构建key值
					key := v.Custplantnr + kkkk
					val, ok := monthData[key]
					if ok {
						if utils.ValueToFloat(val, 0.0) > 0.0 {
							valinfo := utils.ValueToFloat(vvvv, 0.0) / utils.ValueToFloat(val, 0.0)
							ainfo.Oct = fmt.Sprintf("%0.0f", valinfo)
							atotal += utils.ValueToFloat(valinfo, 0.0)
						} else {
							ainfo.Oct = "0"
						}

					} else {
						ainfo.Oct = "0"
					}
				} else if strings.Contains(kkkk, "/11") {
					datainfo.Nov = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToFloat(vvvv, 0.0)

					//构建key值
					key := v.Custplantnr + kkkk
					val, ok := monthData[key]
					if ok {
						if utils.ValueToFloat(val, 0.0) > 0.0 {
							valinfo := utils.ValueToFloat(vvvv, 0.0) / utils.ValueToFloat(val, 0.0)
							ainfo.Nov = fmt.Sprintf("%0.0f", valinfo)
							atotal += utils.ValueToFloat(valinfo, 0.0)
						} else {
							ainfo.Nov = "0"
						}

					} else {
						ainfo.Nov = "0"
					}
				} else if strings.Contains(kkkk, "/12") {
					datainfo.Dec = utils.ValueToString(vvvv, "")
					infototal += utils.ValueToFloat(vvvv, 0.0)

					//构建key值
					key := v.Custplantnr + kkkk
					val, ok := monthData[key]
					if ok {
						if utils.ValueToFloat(val, 0.0) > 0.0 {
							valinfo := utils.ValueToFloat(vvvv, 0.0) / utils.ValueToFloat(val, 0.0)
							ainfo.Dec = fmt.Sprintf("%0.0f", valinfo)
							atotal += utils.ValueToFloat(valinfo, 0.0)
						} else {
							ainfo.Dec = "0"
						}

					} else {
						ainfo.Dec = "0"
					}
				}
			}
			itotal := fmt.Sprintf("%0.0f", infototal)
			datainfo.Total = utils.ValueToString(itotal, "")
			tTotal = append(tTotal, datainfo)
			vtotal := fmt.Sprintf("%0.0f", atotal)
			ainfo.Total = utils.ValueToString(vtotal, "")
			aTotal = append(aTotal, ainfo)
		}
	}

	data["cTotal"] = cTotal
	data["tTotal"] = tTotal
	data["aTotal"] = aTotal
	return data, nil
}

//导出产品预测报表
func (t *PlnForecastItemDailydemand) ReadYearExcelDemand(timedate string,articles string) (string, string, error) {
	m := new(PlnForecastItemDailydemand)
	m.Finr = t.Finr
	m.Forecastid = t.Forecastid
	if t.Demandyear > 0 {
		m.Demandyear = t.Demandyear
	} else {
		m.Demandyear = utils.ValueToInt(utils.TimeFormat(time.Now(), "yyyy"), 0)
	}
	datalist, err := m.TotalToCustplantnr(timedate,articles)
	if err != nil {
		return "", "", err
	}
	var file *xlsx.File
	var sheet *xlsx.Sheet
	var row *xlsx.Row
	var cell *xlsx.Cell

	file = xlsx.NewFile()
	sheet, err = file.AddSheet("Sheet1")
	if err != nil {
		return "", "", err
	}

	tTotal, ok := datalist["tTotal"]
	if ok {
		tableHead := true

		for _, val := range tTotal {

			if tableHead {

				//设置表格标题
				row := sheet.AddRow()
				titleRow := row.AddCell()
				titleRow.HMerge = 14 //向右合并列数，不包括自身列
				titleRow.Value = "预测年报表"

				//添加表头
				row = sheet.AddRow()
				//Monthly Total
				cell = row.AddCell()
				cell.HMerge = 1 //向右合并列数，不包括自身列
				cell.Value = "Monthly Total"

				//添加两行空白格，用于合并
				cell = row.AddCell()

				//一月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Jan"

				//二月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Feb"

				//三月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Mar"

				//四月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Apr"

				//五月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "May"

				//六月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Jun"

				//七月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Jul"

				//八月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Aug"

				//九月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Sep"

				//十月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Oct"

				//十一月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Nov"

				//十二月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Dec"

				//统计
				cell = row.AddCell()
				cell.Value = "Total"

				tableHead = false
			}
			row = sheet.AddRow()
			//工厂号
			cell = row.AddCell()
			cell.Value = utils.ValueToString(val.Custplantnr, "")

			//子产品
			cell = row.AddCell()
			cell.Value = utils.ValueToString(val.Art, "")

			//一月
			cell = row.AddCell()
			cell.Value = val.Jan

			//二月
			cell = row.AddCell()
			cell.Value = val.Feb

			//三月
			cell = row.AddCell()
			cell.Value = val.Mar

			//四月
			cell = row.AddCell()
			cell.Value = val.Apr

			//五月
			cell = row.AddCell()
			cell.Value = val.May

			//六月
			cell = row.AddCell()
			cell.Value = val.Jun

			//七月
			cell = row.AddCell()
			cell.Value = val.Jul

			//八月
			cell = row.AddCell()
			cell.Value = val.Aug

			//九月
			cell = row.AddCell()
			cell.Value = val.Sep

			//十月
			cell = row.AddCell()
			cell.Value = val.Oct

			//十一月
			cell = row.AddCell()
			cell.Value = val.Nov

			//十二月
			cell = row.AddCell()
			cell.Value = val.Dec

			//统计
			cell = row.AddCell()
			cell.Value = val.Total
		}
	}

	cTotal, ok := datalist["cTotal"]
	if ok {
		tableHead := true

		for _, val := range cTotal {

			if tableHead {

				//设置表格标题
				row := sheet.AddRow()
				titleRow := row.AddCell()
				titleRow.HMerge = 14 //向右合并列数，不包括自身列
				titleRow.Value = "预测年报表"

				//添加表头
				row = sheet.AddRow()
				//Monthly Total
				cell = row.AddCell()
				cell.HMerge = 1 //向右合并列数，不包括自身列
				cell.Value = "Monthly Total"

				//添加两行空白格，用于合并
				cell = row.AddCell()
				//一月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Jan"

				//二月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Feb"

				//三月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Mar"

				//四月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Apr"

				//五月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "May"

				//六月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Jun"

				//七月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Jul"

				//八月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Aug"

				//九月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Sep"

				//十月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Oct"

				//十一月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Nov"

				//十二月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Dec"

				//统计
				cell = row.AddCell()
				cell.Value = "Total"

				tableHead = false
			}
			row = sheet.AddRow()
			//工厂号
			cell = row.AddCell()
			cell.Value = utils.ValueToString(val.Custplantnr, "")

			//子产品
			cell = row.AddCell()
			cell.Value = utils.ValueToString(val.Art, "")

			//一月
			cell = row.AddCell()
			cell.Value = val.Jan

			//二月
			cell = row.AddCell()
			cell.Value = val.Feb

			//三月
			cell = row.AddCell()
			cell.Value = val.Mar

			//四月
			cell = row.AddCell()
			cell.Value = val.Apr

			//五月
			cell = row.AddCell()
			cell.Value = val.May

			//六月
			cell = row.AddCell()
			cell.Value = val.Jun

			//七月
			cell = row.AddCell()
			cell.Value = val.Jul

			//八月
			cell = row.AddCell()
			cell.Value = val.Aug

			//九月
			cell = row.AddCell()
			cell.Value = val.Sep

			//十月
			cell = row.AddCell()
			cell.Value = val.Oct

			//十一月
			cell = row.AddCell()
			cell.Value = val.Nov

			//十二月
			cell = row.AddCell()
			cell.Value = val.Dec

			//统计
			cell = row.AddCell()
			cell.Value = val.Total
		}
	}
	aTotal, ok := datalist["aTotal"]
	if ok {
		tableHead := true

		for _, val := range aTotal {

			if tableHead {

				//设置表格标题
				row := sheet.AddRow()
				titleRow := row.AddCell()
				titleRow.HMerge = 14 //向右合并列数，不包括自身列
				titleRow.Value = "预测年报表"

				//添加表头
				row = sheet.AddRow()
				//Monthly Total
				cell = row.AddCell()
				cell.HMerge = 1 //向右合并列数，不包括自身列
				cell.Value = "Monthly Total"

				//添加两行空白格，用于合并
				cell = row.AddCell()

				//一月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Jan"

				//二月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Feb"

				//三月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Mar"

				//四月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Apr"

				//五月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "May"

				//六月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Jun"

				//七月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Jul"

				//八月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Aug"

				//九月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Sep"

				//十月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Oct"

				//十一月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Nov"

				//十二月
				cell = row.AddCell()
				cell.Value = val.Year + "/" + "Dec"

				//统计
				cell = row.AddCell()
				cell.Value = "Total"

				tableHead = false
			}
			row = sheet.AddRow()
			//工厂号
			cell = row.AddCell()
			cell.Value = utils.ValueToString(val.Custplantnr, "")

			//子产品
			cell = row.AddCell()
			cell.Value = utils.ValueToString(val.Art, "")

			//一月
			cell = row.AddCell()
			cell.Value = val.Jan

			//二月
			cell = row.AddCell()
			cell.Value = val.Feb

			//三月
			cell = row.AddCell()
			cell.Value = val.Mar

			//四月
			cell = row.AddCell()
			cell.Value = val.Apr

			//五月
			cell = row.AddCell()
			cell.Value = val.May

			//六月
			cell = row.AddCell()
			cell.Value = val.Jun

			//七月
			cell = row.AddCell()
			cell.Value = val.Jul

			//八月
			cell = row.AddCell()
			cell.Value = val.Aug

			//九月
			cell = row.AddCell()
			cell.Value = val.Sep

			//十月
			cell = row.AddCell()
			cell.Value = val.Oct

			//十一月
			cell = row.AddCell()
			cell.Value = val.Nov

			//十二月
			cell = row.AddCell()
			cell.Value = val.Dec

			//统计
			cell = row.AddCell()
			cell.Value = val.Total
		}
	}
	name := utils.MakeOrderSn("预测统计")
	filename := name + ".xlsx" //文件名

	exceldir,_ := utils.GetCurrentPath("web/public/uploadxlsx") // 目录+文件名
	excelfile := exceldir + "/" + filename
	err = file.Save(excelfile)
	if err != nil {
		return "", "", err
	}
	return excelfile, filename, nil
}

//查询出解析版本的数据
func (t *PlnForecastItemDailydemand) ReadDemand() ([]PlnForecastItemDailydemand, error) {
	engine := db.Eloquent.Master()
	data := make([]PlnForecastItemDailydemand, 0)
	err := engine.Where("finr = ? and forecastid = ?", t.Finr, t.Forecastid).Find(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}
