package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"LAPP_SJA_ME/web/middleware/glog"
	"errors"
	"fmt"
	"math"
	"strings"
	"time"

	"github.com/soniah/evaler"
)

type PlnForecastLanding struct {
	Finr           int    `xorm:"pk comment('工厂号') Int" json:"pln_forecast_landing-finr"`
	Forecastid     string `xorm:"pk comment('版本号') VARCHAR(20)" json:"pln_forecast_landing-forecastid"`
	Serialnumber   string `xorm:"pk comment('流水号ID') VARCHAR(20)" json:"pln_forecast_landing-serialnumber"`
	Descr          string `xorm:"comment('描述1') VARCHAR(40)" json:"pln_forecast_landing-descr"`
	MergeStartdate string `xorm:"comment('开始日期')  VARCHAR(20)" json:"pln_forecast_landing-merge_startdate"`
	Adddate        string `xorm:"comment('开始日期')  VARCHAR(20)" json:"pln_forecast_landing-adddate"`
	Lastmodif      string `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"pln_forecast_landing-lastmodif"`
	Lastuser       string `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"pln_forecast_landing-lastuser"`
	Credatuz       string `xorm:"comment('创建时间') VARCHAR(14)" json:"pln_forecast_landing-credatuz"`
	Custplantnr    string `xorm:"-" json:"pln_forecast_landing-custplantnr"`
}

func (t *PlnForecastLanding) TableName() string {
	return "pln_forecast_landing"
}

// 清除string字段的右侧空格
func (t *PlnForecastLanding) Clipped() {
	utils.TrimStruct(t, *t)
}

type HeadandLanding struct {
	PlnForecastLanding `xorm:"extends"`
	PlnForecastHead    `xorm:"extends"`
}
type PlnForecastLandingInfo struct {
	Finr                int    `json:"pln_forecast_landing-finr"`
	Status              string `json:"pln_forecast_landing-status"`
	Descr               string `json:"pln_forecast_landing-descr"`
	Forecastid          string `json:"pln_forecast_landing-forecastid"`
	Serialnumber        string `json:"pln_forecast_landing-serialnumber"`
	MergeStartdate      string `json:"pln_forecast_landing-merge_startdate"`
	Edifile             string `json:"pln_forecast_landing-edifile"`
	Suppliercode        string `json:"pln_forecast_landing-suppliercode"`
	Suppliername        string `json:"pln_forecast_landing-suppliername"`
	SupplierAddresscode string `json:"pln_forecast_landing-supplier_addresscode"`
	Countrycode         string `json:"pln_forecast_landing-countrycode"`
	Consigneecode       string `json:"pln_forecast_landing-consigneecode"`
	DeliveryAddresscode string `json:"pln_forecast_landing-delivery_addresscode"`
	Custplantnr         string `json:"pln_forecast_landing-custplantnr"`
	Lastmodif           string `json:"pln_forecast_landing-lastmodif"`
	Lastuser            string `json:"pln_forecast_landing-lastuser"`
	Credatuz            string `json:"pln_forecast_landing-credatuz"`
}

//分页
func (t *PlnForecastLanding) GetPage(pageSize int, pageIndex int) ([]PlnForecastLandingInfo, int, error) {
	datalist := make([]PlnForecastLandingInfo, 0)
	data := make([]HeadandLanding, 0)
	es := db.Eloquent.Slaves()
	e := es[0]
	table := e.Table("pln_forecast_landing").Join("LEFT", "pln_forecast_head", "pln_forecast_head.forecastid = pln_forecast_landing.forecastid and pln_forecast_head.serialnumber = pln_forecast_landing.serialnumber and pln_forecast_head.finr = pln_forecast_landing.finr").Where("pln_forecast_landing.finr = ? ", t.Finr)
	where := "where pln_forecast_landing.finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if !utils.ValueIsEmpty(t.Forecastid) {
		table = table.And("pln_forecast_landing.forecastid = ?", t.Forecastid)
		where += " and pln_forecast_landing.forecastid = " + "'" + t.Forecastid + "'"
	}
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " pln_forecast_landing.*,pln_forecast_head.* FROM pln_forecast_landing LEFT JOIN pln_forecast_head ON pln_forecast_head.forecastid = pln_forecast_landing.forecastid and pln_forecast_head.serialnumber = pln_forecast_landing.serialnumber and pln_forecast_head.finr = pln_forecast_landing.finr  " + where + " AND (convert(varchar(10),pln_forecast_landing.finr)+convert(varchar(20),pln_forecast_landing.forecastid)+convert(varchar(20),pln_forecast_landing.serialnumber) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),pln_forecast_landing.finr)+convert(varchar(20),pln_forecast_landing.forecastid)+convert(varchar(20),pln_forecast_landing.serialnumber)  FROM pln_forecast_landing LEFT JOIN pln_forecast_head ON pln_forecast_head.forecastid = pln_forecast_landing.forecastid and  pln_forecast_head.serialnumber = pln_forecast_landing.serialnumber and pln_forecast_head.finr = pln_forecast_landing.finr " + where + " ORDER BY pln_forecast_landing.credatuz DESC)) ORDER BY pln_forecast_landing.credatuz DESC").Find(&data)
	pcount := new(PlnForecastLanding)
	count, err := table.Count(pcount)
	if err != nil {
		return datalist, 0, err
	}
	for _, v := range data {
		v.PlnForecastLanding.Clipped()
		v.PlnForecastHead.Clipped()
		dataone := PlnForecastLandingInfo{}
		dataone.Finr = v.PlnForecastLanding.Finr
		dataone.Serialnumber = v.PlnForecastLanding.Serialnumber
		dataone.Descr = v.PlnForecastLanding.Descr
		dataone.Edifile = v.PlnForecastHead.Edifile
		dataone.Forecastid = v.PlnForecastLanding.Forecastid
		dataone.MergeStartdate = v.PlnForecastLanding.MergeStartdate
		dataone.Custplantnr = v.PlnForecastHead.Custplantnr
		dataone.Status = v.PlnForecastHead.Status
		dataone.Credatuz = v.PlnForecastHead.Credatuz
		dataone.Lastuser = v.PlnForecastHead.Lastuser
		dataone.Lastmodif = v.PlnForecastHead.Lastmodif
		dataone.Suppliercode = v.PlnForecastHead.Suppliercode
		dataone.Suppliername = v.PlnForecastHead.Suppliername
		dataone.SupplierAddresscode = v.PlnForecastHead.SupplierAddresscode
		dataone.Countrycode = v.PlnForecastHead.Countrycode
		dataone.Consigneecode = v.PlnForecastHead.Consigneecode
		dataone.DeliveryAddresscode = v.PlnForecastHead.DeliveryAddresscode
		datalist = append(datalist, dataone)
	}
	return datalist, int(count), nil
}

//解析：根据预测id，查询对应的零件需求，然后比对核心零件（多个核心零件，以最少的为准），生成对应的产品，日以上的计划需要时间工作日历分配到日计划
func (t *PlnForecastLanding) ParseInfo() error {
	es := db.Eloquent.Slaves()
	e := es[0]
	engine := db.Eloquent.Master()

	//初始化事务
	session := engine.NewSession()
	defer session.Close()
	//开启事务
	err := session.Begin()

	count := new(PlnForecastHead)
	//检查当前版本是否已经解析
	total, err := e.Table("pln_forecast_head").Where("finr = ? and forecastid = ? and custplantnr = ? and status = ?", t.Finr, t.Forecastid, t.Custplantnr, "1").Count(count)
	if err != nil {
		return err
	}
	if total >= 1 {
		return errors.New("请删除当前版本已解析的文件！")
	}
	//锁定解析，状态变成已经解析
	_, err = e.Table("pln_forecast_head").Where("finr = ? and forecastid = ? and custplantnr = ? and serialnumber = ?", t.Finr, t.Forecastid, t.Custplantnr, t.Serialnumber).Update(map[string]interface{}{"status": "1"})
	if err != nil {
		return err
	}

	//查询出运算公式
	dataTem := make([]PlnForecastFormula, 0)
	err = engine.Table("pln_forecast_formula").Where("finr = ?", t.Finr).Find(&dataTem)
	if err != nil {
		return err
	}
	formulaMap := make(map[string]string)
	for _, v := range dataTem {
		v.Clipped()
		key := v.Code
		val := "(" + v.DemandCalFormula + ")"
		formulaMap[key] = val
	}

	partids := make([]PlnForecastCriticalItem, 0)
	//第一步：查询核心零件
	err = engine.Table("pln_forecast_critical_item").Where("finr = ?", t.Finr).Find(&partids)
	if err != nil {
		return err
	}
	partsMap := make(map[string]string)
	for _, v := range partids {
		v.Clipped()
		key := v.Partid
		val := v.Abrev
		partsMap[key] = val
	}
	data := make([]PlnForecastArticle, 0)
	//第二步：查询所有对应的项目下的产品集合
	err = engine.Table("pln_forecast_article").Where("finr = ?", t.Finr).Find(&data)
	if err != nil {
		return err
	}

	//第三步：查询所有的预测零件需求
	datalist := make([]PlnForecastItemDemandlst, 0)
	err = e.Table("pln_forecast_item_demandlst").Where("finr = ? and forecastid = ? and custplantnr = ? and serialnumber = ?", t.Finr, t.Forecastid, t.Custplantnr, t.Serialnumber).Find(&datalist)
	if err != nil {
		return err
	}

	//第五步：按照天分割出来
	temData := make([]PlnForecastItemDemandlst, 0)
	for _, v := range datalist {
		if v.Startdate == v.Enddate {
			temData = append(temData, v)
		} else {
			sdata := t.SplitForecastItemDemandlst(v)
			for _, vv := range sdata {
				temData = append(temData, vv)
			}
		}

	}
	daysMap := make(map[string][]PlnForecastItemDemandlst)
	////第六步：以天分组，转成kv形式,key为键
	for _, v := range temData {
		//第七步：遍历预测零件需求,以每天分组
		key := v.Startdate
		daysMap[key] = append(daysMap[key], v)
	}
	//第八步：遍历构造的产品集合,并且添加到数据表里
	for timedate, dayData := range daysMap {
		//变成  a => 20  b => 30 这样的数组
		dataNum := make(map[string]int)
		for _, v := range dayData {
			key := v.Partid
			val, ok := partsMap[key]
			if ok {
				dataNum[val] = v.Demandqty
			}
		}
		//遍历所有的产品，得出对应的数量
		for _, vvv := range data {
			if utils.ValueIsEmpty(vvv.DemandCalFormula) {
				continue
			}

			for oldstr, newstr := range formulaMap {
				//替换所有的
				vvv.DemandCalFormula = strings.Replace(vvv.DemandCalFormula, oldstr, utils.ValueToString(newstr, ""), -1)
			}

			//遍历dataNum,替换公式
			for old, newstr := range dataNum {
				//替换所有的
				vvv.DemandCalFormula = strings.Replace(vvv.DemandCalFormula, old, utils.ValueToString(newstr, ""), -1)
			}
			//没有的话，替换成0
			for _, newstr := range partsMap {
				//替换所有的
				vvv.DemandCalFormula = strings.Replace(vvv.DemandCalFormula, utils.ValueToString(newstr, ""), "0", -1)
			}
			result, err := evaler.Eval(vvv.DemandCalFormula)
			if err != nil {
				session.Rollback()
				return err
			}
			num, err := evaler.BigratToInt(result)
			if err != nil {
				session.Rollback()
				return err
			}
			//解析计划 pln_forecast_article_dailydemand
			pln := new(PlnForecastItemDailydemand)
			day := uint16(utils.ValueToInt(timedate[6:8], 0))
			month := uint16(utils.ValueToInt(timedate[4:6], 0))
			year := uint16(utils.ValueToInt(timedate[0:4], 0))
			week := utils.ZellerFunction2Week(year, month, day)
			pln.Finr = t.Finr
			pln.Forecastid = t.Forecastid
			pln.Demandqty = int(num)
			pln.Demanddate = timedate
			pln.Custplantnr = t.Custplantnr
			pln.Artnr = vvv.Artnr
			pln.Demandmonth = utils.ValueToInt(timedate[4:6], 0)
			pln.Demandweek = week
			pln.Demandyear = utils.ValueToInt(timedate[0:4], 0)
			pln.Lastmodif = t.Lastmodif
			pln.Credatuz = t.Credatuz
			pln.Lastuser = t.Lastuser
			if pln.Demandqty == 0 {
				continue
			}
			_, err = session.Table("pln_forecast_item_dailydemand").Insert(pln)
			//事务回滚
			if err != nil {
				session.Rollback()
				return err
			}
		}
	}
	//执行
	err = session.Commit()
	if err != nil {
		return err
	}
	//执行
	//查询统计出的产品数据，并且记录下来
	p := new(PlnForecastItemDailydemand)
	p.Finr = t.Finr
	p.Forecastid = t.Forecastid
	dataArts, err := p.ReadDemand()
	if err != nil {
		glog.InfoExt("产品月份统计", "统计 err is :", err)
	}
	for _, v := range dataArts {
		if v.Demandqty > 0 {
			art := new(PlnForecastArticleTotal)
			art.Finr = t.Finr
			art.Demanddate = v.Demanddate
			art.Demandweek = v.Demandweek
			art.Demandmonth = v.Demandmonth
			art.Custplantnr = v.Custplantnr
			art.Demandyear = v.Demandyear
			art.Demandqty = v.Demandqty
			art.Artnr = v.Artnr
			art.Lastuser = t.Lastuser
			art.Credatuz = t.Credatuz
			art.Lastmodif = t.Lastmodif
			err = art.Add()
			if err != nil {
				glog.InfoExt("产品月份统计", "统计添加 err is :", err)
			}
		}
	}
	return nil

}

//重新解析：根据预测id，查询对应的零件需求，然后比对核心零件（多个核心零件，以最少的为准），生成对应的产品，日以上的计划需要时间工作日历分配到日计划
func (t *PlnForecastLanding) ResetParseInfo() error {

	es := db.Eloquent.Slaves()
	e := es[0]
	engine := db.Eloquent.Master()

	//初始化事务
	session := engine.NewSession()
	defer session.Close()
	//开启事务
	err := session.Begin()

	//前提：删除生成的数据
	_, err = session.Table("pln_forecast_item_dailydemand").Where("finr = ? and custplantnr = ? and forecastid = ?", t.Finr, t.Custplantnr, t.Forecastid).Delete(&PlnForecastItemDailydemand{})
	if err != nil {
		fmt.Printf("err5 :%v", err)
		session.Rollback()
		return err
	}

	//查询出运算公式
	dataTem := make([]PlnForecastFormula, 0)
	err = engine.Table("pln_forecast_formula").Where("finr = ?", t.Finr).Find(&dataTem)
	if err != nil {
		session.Rollback()
		return err
	}
	formulaMap := map[string]string{}
	for _, v := range dataTem {
		v.Clipped()
		key := v.Code
		val := "(" + v.DemandCalFormula + ")"
		formulaMap[key] = val
	}

	partids := make([]PlnForecastCriticalItem, 0)
	//第一步：查询核心零件
	err = engine.Table("pln_forecast_critical_item").Where("finr = ?", t.Finr).Find(&partids)
	if err != nil {
		fmt.Errorf("err is :%v", err)
		session.Rollback()
		return err
	}
	partsMap := map[string]string{}
	for _, v := range partids {
		v.Clipped()
		key := v.Partid
		val := v.Abrev
		partsMap[key] = val
	}
	data := make([]PlnForecastArticle, 0)
	//第二步：查询所有对应的项目下的产品集合
	err = engine.Table("pln_forecast_article").Where("finr = ?", t.Finr).Find(&data)
	if err != nil {
		session.Rollback()
		return err
	}

	//第三步：查询所有的预测零件需求
	datalist := make([]PlnForecastItemDemandlst, 0)
	err = e.Table("pln_forecast_item_demandlst").Where("finr = ? and forecastid = ? and custplantnr = ? and serialnumber = ?", t.Finr, t.Forecastid, t.Custplantnr, t.Serialnumber).Find(&datalist)
	if err != nil {
		session.Rollback()
		return err
	}

	//第五步：按照天分割出来
	temData := make([]PlnForecastItemDemandlst, 0)
	for _, v := range datalist {
		if v.Startdate == v.Enddate {
			temData = append(temData, v)
		} else {
			sdata := t.SplitForecastItemDemandlst(v)
			for _, vv := range sdata {
				temData = append(temData, vv)
			}

		}

	}
	daysMap := make(map[string][]PlnForecastItemDemandlst, 0)
	//第六步：以天分组，转成kv形式,key为键
	for _, v := range temData {
		//第七步：遍历预测零件需求,以每天分组
		key := v.Startdate
		daysMap[key] = append(daysMap[key], v)
	}
	//第八步：遍历构造的产品集合,并且添加到数据表里
	for timedate, dayData := range daysMap {
		//变成  a => 20  b => 30 这样的数组
		dataNum := make(map[string]int)
		for _, v := range dayData {
			key := v.Partid
			val, ok := partsMap[key]
			if ok {
				dataNum[val] = v.Demandqty
			}
		}
		//遍历所有的产品，得出对应的数量
		for _, vvv := range data {
			if utils.ValueIsEmpty(vvv.DemandCalFormula) {
				continue
			}

			for oldstr, newstr := range formulaMap {
				//替换所有的
				vvv.DemandCalFormula = strings.Replace(vvv.DemandCalFormula, oldstr, utils.ValueToString(newstr, ""), -1)
			}

			//遍历dataNum,替换公式
			for old, new := range dataNum {
				//替换所有的
				vvv.DemandCalFormula = strings.Replace(vvv.DemandCalFormula, old, utils.ValueToString(new, ""), -1)
			}
			//没有的话，替换成0
			for _, new := range partsMap {
				//替换所有的
				vvv.DemandCalFormula = strings.Replace(vvv.DemandCalFormula, utils.ValueToString(new, ""), "0", -1)
			}
			result, err := evaler.Eval(vvv.DemandCalFormula)
			if err != nil {
				session.Rollback()
				return err
			}
			num, err := evaler.BigratToInt(result)
			if err != nil {
				session.Rollback()
				return err
			}
			//解析计划 pln_forecast_article_dailydemand
			pln := new(PlnForecastItemDailydemand)
			day := uint16(utils.ValueToInt(timedate[6:8], 0))
			month := uint16(utils.ValueToInt(timedate[4:6], 0))
			year := uint16(utils.ValueToInt(timedate[0:4], 0))
			week := utils.ZellerFunction2Week(year, month, day)
			pln.Finr = t.Finr
			pln.Forecastid = t.Forecastid
			pln.Demandqty = int(num)
			pln.Demanddate = timedate
			pln.Custplantnr = t.Custplantnr
			pln.Artnr = vvv.Artnr
			pln.Demandmonth = utils.ValueToInt(timedate[4:6], 0)
			pln.Demandweek = week
			pln.Demandyear = utils.ValueToInt(timedate[0:4], 0)
			pln.Lastmodif = t.Lastmodif
			pln.Credatuz = t.Credatuz
			pln.Lastuser = t.Lastuser
			if pln.Demandqty == 0 {
				continue
			}
			_, err = session.Table("pln_forecast_item_dailydemand").Insert(pln)
			//事务回滚
			if err != nil {
				session.Rollback()
				return err
			}
		}
	}
	//执行
	err = session.Commit()
	if err != nil {
		return err
	}

	//查询统计出的产品数据，并且记录下来
	p := new(PlnForecastItemDailydemand)
	p.Finr = t.Finr
	p.Forecastid = t.Forecastid
	dataArts, err := p.ReadDemand()
	if err != nil {
		glog.InfoExt("产品月份统计", "统计 err is :", err)
	}
	for _, v := range dataArts {
		if v.Demandqty > 0 {
			art := new(PlnForecastArticleTotal)
			art.Finr = t.Finr
			art.Demanddate = v.Demanddate
			art.Demandweek = v.Demandweek
			art.Demandmonth = v.Demandmonth
			art.Custplantnr = v.Custplantnr
			art.Demandyear = v.Demandyear
			art.Demandqty = v.Demandqty
			art.Artnr = v.Artnr
			art.Lastuser = t.Lastuser
			art.Credatuz = t.Credatuz
			art.Lastmodif = t.Lastmodif
			err = art.Add()
			if err != nil {
				glog.InfoExt("产品月份统计", "统计添加 err is :", err)
			}
		}
	}
	return nil

}

type workCalendar struct {
	PlnForecastCalendar       `xorm:"extends"`
	PlnForecastCalendarDaylst `xorm:"extends"`
}

/**
*功能：日以上的计划，需要分割到天
*算法：除第一天产品个数取整，
*    第一天个数 = 总数 - 其它天数的产品
***/

func (t *PlnForecastLanding) SplitForecastItemDemandlst(items PlnForecastItemDemandlst) []PlnForecastItemDemandlst {
	e := db.Eloquent.Master()
	data := make([]PlnForecastItemDemandlst, 0)
	//查询工作日历
	datalist := make([]workCalendar, 0)
	err := e.Table("pln_forecast_calendar").Alias("a").Join("LEFT", []string{"pln_forecast_calendar_daylst", "p"}, "a.calendarid = p.calendarid and a.finr = p.finr").Where("a.finr = ?  and p.caldate >=? and p.caldate <= ? and a.custplantnr = ? and a.mydefault = 1", t.Finr, items.Startdate, items.Enddate, items.Custplantnr).Asc("p.caldate").Find(&datalist)
	if err != nil {
		return data
	}
	if !utils.ValueIsEmpty(datalist) {
		//计算出分母系数
		total := 0.0
		//记录除了第一天的零件数量，并且向下取整数 math.Floor(x)
		for _, v := range datalist {
			total += utils.ValueToFloat(v.Effactor, 0.0)
		}
		//记录除了第一天的产品数
		sum := 0
		lenth := len(datalist)
		for k, v := range datalist {
			dataone := PlnForecastItemDemandlst{}
			dataone.Finr = t.Finr
			dataone.Partid = items.Partid
			dataone.Forecastid = items.Forecastid
			dataone.Custplantnr = items.Custplantnr
			dataone.Startdate = v.Caldate
			dataone.Enddate = v.Caldate

			if total > 0 {
				if utils.ValueToFloat(v.Effactor, 0.0) == 0.0 {
					dataone.Demandqty = 0
				} else {
					Demandqty := utils.ValueToFloat(items.Demandqty, 0.0) * utils.ValueToFloat(v.Effactor, 0.0) / total
					dataone.Demandqty = utils.ValueToInt(math.Floor(Demandqty), 0)
					sum += dataone.Demandqty
				}
			} else {
				dataone.Demandqty = 0
			}
			data = append(data, dataone)
			if (k + 1) == lenth {
				//进入最后一次循环，判断是否有剩余，有的话，加入到任意一天非0的一天计划
				lastDemandqty := items.Demandqty - sum
				if lastDemandqty>0{
					for kk,vv:=range data{
						if vv.Demandqty >0{
							data[kk].Demandqty = vv.Demandqty + lastDemandqty
							break
						}
					}
				}
			}
		}
	}
	return data
}

//查询流水集合
func (t *PlnForecastLanding) SelectAll() ([]PlnForecastLanding, error) {
	es := db.Eloquent.Slaves()
	e := es[0]

	data := make([]PlnForecastLanding, 0)
	err := e.Table("pln_forecast_landing").Cols("forecastid", "descr").Distinct("forecastid").Where("finr = ? ", t.Finr).Find(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

//查询流水集合
func (t *PlnForecastLanding) SelectArr(plns []string) ([]PlnForecastLanding, error) {
	es := db.Eloquent.Slaves()
	e := es[0]

	data := make([]PlnForecastLanding, 0)
	err := e.Table("pln_forecast_landing").Distinct("forecastid").Where("finr = ? ", t.Finr).In("forecastid", plns).Find(&data)
	if err != nil {
		return data, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, nil
}

//获取基础数据
func (t *PlnForecastLanding) SaveParseResult(filename string, lab *LAB) error {

	Credatuz := utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	Lastmodif := utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	Lastuser := t.Lastuser
	es := db.Eloquent.Slaves()
	engine := es[0]
	//初始化事务
	session := engine.NewSession()
	defer session.Close()
	//开启事务
	err := session.Begin()

	//判断当日是否有添加文件记录，有的话返回流水号
	temd := PlnForecastLanding{}
	_, err = engine.Table("pln_forecast_landing").Where("finr = ? and adddate = ?", t.Finr, t.Adddate).Get(&temd)
	if err != nil {
		return err
	}
	forecastid := ""
	if !utils.ValueIsEmpty(temd.Forecastid) {
		forecastid = temd.Forecastid
	} else {
		//获取版本
		snrtab := new(Snrtab)
		snrtab.Finr = t.Finr
		forecastid, err = snrtab.GetNextSnr("ForecastId")
		if err != nil {
			return err
		}

	}

	//获取流水号
	seri := new(Snrtab)
	seri.Finr = t.Finr
	Serialnumber, err := seri.GetNextSnr("Serialnumber")
	if err != nil {
		return err
	}

	var values []int
	for _, ard := range lab.ArdList {
		for _, del := range ard.DelList {
			startdate := utils.TimeFormat(del.StartDate, "yyyyMMdd")
			val := utils.ValueToInt(startdate, 0)
			values = append(values, val)
		}
	}
	small := utils.BubbleSmall(values)
	MergeStartdate := utils.ValueToString(small, "")

	// pln_forecast_landing
	pln_forecast_landing := new(PlnForecastLanding)
	pln_forecast_landing.Forecastid = forecastid
	pln_forecast_landing.Serialnumber = Serialnumber
	pln_forecast_landing.Finr = t.Finr
	pln_forecast_landing.Descr = utils.WeekByDate(time.Now()) + utils.TimeFormat(time.Now(), "MM.dd")
	pln_forecast_landing.MergeStartdate = MergeStartdate
	pln_forecast_landing.Adddate = t.Adddate
	pln_forecast_landing.Lastmodif = Lastmodif
	pln_forecast_landing.Credatuz = Credatuz
	pln_forecast_landing.Lastuser = Lastuser
	_, err = session.Table("pln_forecast_landing").Insert(pln_forecast_landing)
	//事务回滚
	if err != nil {
		session.Rollback()
		fmt.Printf("添加失败原因2:%v", err)
		return err
	}
	tem := strings.Split(filename, ".")
	//获取消息头部 pln_forecast_head
	pln_forecast_head := new(PlnForecastHead)
	pln_forecast_head.Finr = t.Finr
	pln_forecast_head.Forecastid = forecastid
	pln_forecast_head.Serialnumber = Serialnumber
	pln_forecast_head.Suppliercode = lab.Sdt.SupplierCode
	pln_forecast_head.Suppliername = lab.Sdt.SupplierName
	pln_forecast_head.SupplierAddresscode = lab.Csg.AdditionalDest
	pln_forecast_head.Countrycode = lab.Sdt.CountryCode
	pln_forecast_head.Consigneecode = lab.Bdt.Consignee
	pln_forecast_head.DeliveryAddresscode = lab.Csg.DeliveryAddressCode
	pln_forecast_head.Edifile = filename
	pln_forecast_head.Custplantnr = tem[1]
	pln_forecast_head.Lastmodif = Lastmodif
	pln_forecast_head.Credatuz = Credatuz
	pln_forecast_head.Lastuser = Lastuser
	_, err = session.Table("pln_forecast_head").Insert(pln_forecast_head)
	//事务回滚
	if err != nil {
		session.Rollback()
		fmt.Printf("添加失败原因1:%v", err)
		return err
	}

	for _, ard := range lab.ArdList {
		//pln_forecast_item 零件清单信息
		pln_forecast_item := new(PlnForecastItem)
		pln_forecast_item.Finr = t.Finr
		pln_forecast_item.Forecastid = forecastid
		pln_forecast_item.Serialnumber = Serialnumber
		pln_forecast_item.Partid = ard.PartId
		pln_forecast_item.Descr = ard.Descr
		pln_forecast_item.Uom = ard.Uom
		pln_forecast_item.MaterialPlanner = ard.Sad.MaterialPlanner
		pln_forecast_item.Purchaseorder = ard.PurchaseOrder
		pln_forecast_item.Mrpdate = utils.TimeFormat(ard.Dst.MrpDate, "yyyyMMdd")
		pln_forecast_item.Totalqty = ard.Dst.TotalQty
		pln_forecast_item.DeliveryDocnr = ard.Adi.DocNr
		pln_forecast_item.Custplantnr = tem[1]
		pln_forecast_item.DeliveryDocdate = utils.TimeFormat(ard.Adi.DocDate, "yyyyMMdd")
		pln_forecast_item.Lastmodif = Lastmodif
		pln_forecast_item.Credatuz = Credatuz
		pln_forecast_item.Lastuser = Lastuser
		_, err = session.Table("pln_forecast_item").Insert(pln_forecast_item)
		//事务回滚
		if err != nil {
			session.Rollback()
			fmt.Printf("添加失败原因3:%v", err)
			return err
		}

		//LAB 零件需求信息
		for _, del := range ard.DelList {
			//pln_forecast_item_demandlst
			pln_forecast_item_demandlst := new(PlnForecastItemDemandlst)
			pln_forecast_item_demandlst.Finr = t.Finr
			pln_forecast_item_demandlst.Forecastid = forecastid
			pln_forecast_item_demandlst.Serialnumber = Serialnumber
			pln_forecast_item_demandlst.Custplantnr = tem[1]
			pln_forecast_item_demandlst.Partid = ard.PartId
			pln_forecast_item_demandlst.Demandqty = del.DemandQty
			pln_forecast_item_demandlst.Startdate = utils.TimeFormat(del.StartDate, "yyyyMMdd")
			pln_forecast_item_demandlst.Enddate = utils.TimeFormat(del.EndDate, "yyyyMMdd")
			pln_forecast_item_demandlst.Lastmodif = Lastmodif
			pln_forecast_item_demandlst.Credatuz = Credatuz
			pln_forecast_item_demandlst.Lastuser = Lastuser
			_, err = session.Table("pln_forecast_item_demandlst").Insert(pln_forecast_item_demandlst)
			//事务回滚
			if err != nil {
				session.Rollback()
				fmt.Printf("添加失败原因4:%v", err)
				return err
			}
		}
	}

	//执行
	err = session.Commit()
	if err != nil {
		fmt.Printf("添加失败原因5:%v", err)
		return err
	}
	return nil
}

//删
func (t *PlnForecastLanding) Del() error {
	es := db.Eloquent.Slaves()
	e := es[0]
	engine := db.Eloquent.Master()
	session := engine.NewSession()
	defer session.Close()
	// add Begin() before any action
	err := session.Begin()
	_, err = e.Table("pln_forecast_head").Where("finr = ? and forecastid = ? and  serialnumber = ?", t.Finr, t.Forecastid, t.Serialnumber).Update(map[string]interface{}{"status": "0"})
	if err != nil {
		fmt.Printf("err4 :%v", err)
		session.Rollback()
		return err
	}
	_, err = session.Where("finr = ? and custplantnr = ? and forecastid = ?", t.Finr, t.Custplantnr, t.Forecastid).Delete(&PlnForecastItemDailydemand{})
	if err != nil {
		fmt.Printf("err5 :%v", err)
		session.Rollback()
		return err
	}
	err = session.Commit()
	if err != nil {
		fmt.Printf("err6 :%v", err)
		return err
	}
	return nil
}
