package models

import (
	"LAPP_SJA_ME/conf"
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"errors"
	"xorm.io/core"
)

/******备注：打印服务数据放到从库***************/
type Printheadtab struct {
	Finr          int    `xorm:"not null pk comment('车间编号') INT(0)" json:"printheadtab-finr"`
	Printheadid   string `xorm:"not null pk comment('打印信息头id') VARCHAR(40)" json:"printheadtab-printheadid"`
	Printerid     string `xorm:"not null comment('打印机id') VARCHAR(18)" json:"printheadtab-printerid"`
	Printfiletype string `xorm:"not null comment('打印机文件类型') VARCHAR(10)" json:"printheadtab-printfiletype"`
	Printobjid    string `xorm:"not null comment('printheadtab.printobjid') VARCHAR(40)" json:"printheadtab-printobjid"`
	Printobjtype  string `xorm:"not null comment('printheadtab.printobjtype') VARCHAR(12)" json:"printheadtab-printobjtype"`
	Status        string `xorm:"not null comment('状态') VARCHAR(1)" json:"printheadtab-status"`
	Templatefile  string `xorm:"not null comment('模板文件') VARCHAR(40)" json:"printheadtab-templatefile"`
	Printcopies   int    `xorm:"not null comment('计划打印份数') INT(0)" json:"printheadtab-printcopies"`
	Printedtime   string `xorm:"not null comment('实际打印时间') VARCHAR(14)" json:"printheadtab-printedtime"`
	Printedcopies int    `xorm:"not null comment('已打印份数') INT(0)" json:"printheadtab-printedcopies"`
	Orientation   int    `xorm:"not null comment('方向') INT(0)" json:"printheadtab-orientation"`
	Lastmodif     string `xorm:"not null comment('上一次更改日期') VARCHAR(14)" json:"printheadtab-lastmodif"`
	Lastuser      string `xorm:"not null comment('最近编辑人员') VARCHAR(20)" json:"printheadtab-lastuser"`
	Credatuz      string `xorm:"not null comment('创建时间') VARCHAR(14)" json:"printheadtab-credatuz"`
}

func (t *Printheadtab) TableName() string {
	return "printheadtab"
}

// 清除string字段的右侧空格
func (t *Printheadtab) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *Printheadtab) Add() error {
	e := db.Eloquent.Master()
	countrole := new(Printheadtab)
	affw, err := e.Table("printheadtab").ID(core.PK{t.Finr, t.Printheadid}).Count(countrole)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("printheadtab").Insert(t)
	if err != nil {
		return err
	}
	return nil
}

//删
func (t *Printheadtab) Del() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Printheadid}).Delete(&Printheadtab{})
	if err != nil {
		return false
	}
	return true
}

//改
func (t *Printheadtab) Update() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Printheadid}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//查
func (t *Printheadtab) SelectOne() (Printheadtab, error) {
	e := db.Eloquent.Master()
	var data Printheadtab
	_, err := e.ID(core.PK{t.Finr, t.Printheadid}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

//分页
func (t *Printheadtab) GetPage(pageSize int, pageIndex int) ([]Printheadtab, int, error) {
	data := make([]Printheadtab, 0)
	e := db.Eloquent.Master()
	table := e.Table("printheadtab").Where("finr = ? ", t.Finr)
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if !utils.ValueIsEmpty(t.Printheadid) {
		table = table.And("printheadid = ?", t.Printheadid)
		where += " and printheadid = " + "'" + t.Printheadid + "'"
	}
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " printheadtab.* FROM printheadtab " + where + " AND (convert(varchar(10),finr)+convert(varchar(40),printheadid) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),printheadid) FROM printheadtab " + where + " ORDER BY credatuz DESC)) ORDER BY credatuz DESC").Find(&data)
	pcount := new(Printheadtab)
	count, err := table.Count(pcount)
	if err != nil {
		return data, 0, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, int(count), nil
}

func (t *Printheadtab) GetPrintheadList(conf *conf.EnvConfig) ([]Printheadtab, error) {
	var printheadlist []Printheadtab
	e := db.Eloquent.Master()
	// 获取所有开口的打印任务头
	err := e.Where("finr = ? and status = ?", conf.Finr, "N").Find(&printheadlist)
	if err != nil {
		return nil, err
	}

	return printheadlist, nil
}
//删除七天前的打印记录及其明细
func (t *Printheadtab) DelBefore7DayRecords(daytime string) bool {
	e := db.Eloquent.Master()
	data := make([]Printheadtab, 0)
	//查询status = 'C' ,并且是七天前的数据
	err := e.Table("printheadtab").Where("finr = ? and status = ? and credatuz < ?", t.Finr, "C", daytime).Limit(50).Find(&data)
	if err != nil {
		return false
	}
	//Printheadid数组
	var printheadids []string
	for _, v := range data {
		printheadids = append(printheadids, v.Printheadid)
	}
	_, err = e.Table("printheadtab").In("printheadid", printheadids).Where("finr = ?",t.Finr).Delete(&Printheadtab{})
	if err != nil {
		return false
	}
	_, err = e.Table("printdetailtab").In("printheadid", printheadids).Where("finr = ?",t.Finr).Delete(&Printdetailtab{})
	if err != nil {
		return false
	}
	return true
}
