package models

import (
	"LAPP_SJA_ME/db"
	"LAPP_SJA_ME/utils"
	"errors"
	"xorm.io/core"
)

type Uom struct {
	Finr          int     `xorm:"pk comment('车间编号') Int" json:"uom-finr"`
	Uom           string  `xorm:"pk comment('BOM记录标识') VARCHAR(20)" json:"uom-uom"`
	Uomname       string  `xorm:"comment('物料号') VARCHAR(20)" json:"uom-uomname"`
	Uomtype       string  `xorm:"comment('描述') VARCHAR(40)" json:"uom-uomtype"`
	Is_basic_uom  string  `xorm:"comment('') VARCHAR(40)" json:"uom-is_basic_uom"`
	Basic_uom     string  `xorm:"comment('') VARCHAR(40)" json:"uom-basic_uom"`
	Convert_ratio float64 `xorm:"comment('') Double" json:"uom-convert_ratio"`
	Lastmodif     string  `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"uom-lastmodif"`
	Lastuser      string  `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"uom-lastuser"`
	Credatuz      string  `xorm:"comment('创建时间') VARCHAR(14)" json:"uom-credatuz"`
}

func (t *Uom) TableName() string {
	return "uom"
}

// 清除string字段的右侧空格
func (t *Uom) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *Uom) Add() error {
	e := db.Eloquent.Master()
	count := new(Uom)
	affw, err := e.Table("uom").ID(core.PK{t.Finr, t.Uom}).Count(count)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("uom").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//删
func (t *Uom) Del() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Uom}).Delete(&Uom{})
	if err != nil {
		return false
	}
	return true
}

//改
func (t *Uom) Update() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Finr, t.Uom}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//查
func (t *Uom) SelectOne() (Uom, error) {
	e := db.Eloquent.Master()
	var data Uom
	_, err := e.ID(core.PK{t.Finr, t.Uom}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

//分页
func (t *Uom) GetPage(pageSize int, pageIndex int) ([]Uom, int, error) {
	data := make([]Uom, 0)
	e := db.Eloquent.Master()
	table := e.Table("uom").Where("finr = ? ", t.Finr)
	where := "where finr = " + "'" + utils.ValueToString(t.Finr, "") + "'"
	if t.Uom != "" {
		uom := "%" + t.Uom + "%"
		table = table.And("uom like ?", uom)
		where += " and uom like " + "'" + utils.ValueToString(uom, "") + "'"
	}
	Offset := (pageIndex - 1) * pageSize
	err := e.SQL("SELECT TOP " + utils.ValueToString(pageSize, "") + " uom.* FROM uom " + where + " AND (convert(varchar(10),finr)+convert(varchar(40),uom) NOT IN (SELECT TOP " + utils.ValueToString(Offset, "") + " convert(varchar(10),finr)+convert(varchar(40),uom) FROM uom " + where + " ORDER BY credatuz DESC)) ORDER BY credatuz DESC").Find(&data)
	if err != nil {
		return data, 0, err
	}
	pcount := new(Uom)
	count, err := table.Count(pcount)
	if err != nil {
		return data, 0, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, int(count), nil
}

//查询当前Uoms集合
func (t *Uom) SelectArr() ([]Uom, error) {
	e := db.Eloquent.Master()
	var data []Uom
	err := e.Where("finr = ?", t.Finr).Find(&data)
	if err != nil {
		return data, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, nil
}
