package routes

import (
	"LAPP_SJA_ME/web/controllers"
	"LAPP_SJA_ME/web/middleware"
	"LAPP_SJA_ME/web/middleware/cors"
	"github.com/kataras/iris"
)

// 所有的路由
func Hub(app *iris.Application) {

	/* 定义路由 */
	main := app.Party("/", cors.Mycors()).AllowMethods(iris.MethodOptions)

	//中间件，验证token和权限路径
	main.Use(middleware.ServeHTTP)

	home := main.Party("/")

	home.Get("/sysMenu", controllers.DynamicMenu) // 获取动态菜单

	// 用户API模块
	user := main.Party("/user")
	user.Post("/login", controllers.Login)            //登录
	user.Get("/plants", controllers.PlantsTable)      //车间列表
	user.Get("/getuserinfo", controllers.GetUserInfo) // 返回用户信息
	//个人中心
	user.Get("/profile", controllers.GetSysUserProfile)  //获取当前登录用户
	user.Get("/getimg", controllers.GetImg)              //重定向获取图片
	user.Post("/avatar", controllers.InsetSysUserAvatar) //上传图片
	user.Put("/pwd", controllers.SysUserUpdatePwd)       //更新密码

	//// 权限API模块
	admin := main.Party("/admin")
	{
		// 用户管理
		users := admin.Party("/users")
		users.Get("/userlist", controllers.GetSysUserList)                  // 用户列表
		users.Get("/user/{userId:string}", controllers.GetSysUser)          //获取用户
		users.Get("/urole", controllers.GetSysUserInit)                     //获取用户角色
		users.Post("/adduser", controllers.InsertSysUser)                   //添加用户
		users.Put("/upuser", controllers.UpdateSysUser)                     //更新用户
		users.Delete("/deluser/{userId:string}", controllers.DeleteSysUser) //删除用户
		users.Post("/resetpwd", controllers.ResetSysUserPwd)                //重置密码

		//角色管理
		role := admin.Party("/role")
		role.Get("/rolelist", controllers.GetRoleList)                  // 角色报表
		role.Get("/rolebyid", controllers.GetRole)                      // 查询角色
		role.Post("/addrole", controllers.InsertRole)                   // 创建角色
		role.Put("/uprole", controllers.UpdateRole)                     // 更新角色
		role.Delete("/delrole/{roleId:string}", controllers.DeleteRole) // 删除角色

		//菜单管理
		menu := admin.Party("/menu")
		menu.Get("/menulist", controllers.GetMenuList)                                  //菜单列表
		menu.Get("/menubyid", controllers.GetMenu)                                      //by id查询
		menu.Post("/addmenu", controllers.InsertMenu)                                   //添加菜单
		menu.Put("/upmenu", controllers.UpdateMenu)                                     //更新菜单
		menu.Delete("/delmenu/{id:int}", controllers.DeleteMenu)                        //删除菜单
		menu.Get("/menuids", controllers.GetMenuIDS)                                    //通过ids查询菜单
		menu.Get("/menuTreeselect", controllers.GetMenuTreeelect)                       //获取菜单列表
		menu.Get("/roleMenuTreeselect/{roleId:int}", controllers.GetMenuTreeRoleselect) //获取菜单

		// 日志管理
		logs := admin.Party("/logs")
		logs.Get("/loglist", controllers.Loglist)

		// 翻译列表
		tabnames := admin.Party("/tabnames")
		tabnames.Post("/addinfo", controllers.TabInfo)
		tabnames.Get("/listpage", controllers.TabDataListPage)
		tabnames.Get("/list", controllers.TabDataList)
		tabnames.Get("/download", controllers.Download)
		tabnames.Delete("/del", controllers.DeleteTab)

		// 工艺零件主数据
		meparts := admin.Party("/mepart")
		meparts.Get("/listpage", controllers.GetMePartList)
		meparts.Get("/selectinfo", controllers.GetMePart)
		meparts.Post("/addinfo", controllers.InsertMePart)
		meparts.Delete("/del", controllers.DeleteMePart)
		meparts.Put("/upinfo", controllers.UpdateMePart)
		meparts.Get("/selectarr", controllers.SelectArrMePart)
		meparts.Get("/download", controllers.DownloadMePart) //导出零件

		//  零件族主数据
		me_partfamilys := admin.Party("/me_partfamily")
		me_partfamilys.Get("/listpage", controllers.GetMePartfamilyList)
		me_partfamilys.Get("/selectinfo", controllers.GetMePartfamily)
		me_partfamilys.Post("/addinfo", controllers.InsertMePartfamily)
		me_partfamilys.Delete("/del", controllers.DeleteMePartfamily)
		me_partfamilys.Put("/upinfo", controllers.UpdateMePartfamily)
		me_partfamilys.Get("/selectarr", controllers.SelectArrMePartfamily)

		//   模块主数据
		me_partgroups := admin.Party("/me_partgroup")
		me_partgroups.Get("/listpage", controllers.GetMePartgroupList)
		me_partgroups.Get("/selectinfo", controllers.GetMePartgroup)
		me_partgroups.Get("/selectparts", controllers.SelectInfoPartGroup)
		me_partgroups.Put("/saveparts", controllers.SavePartGroup)
		me_partgroups.Get("/selectattrs", controllers.SelectInfoAttrGroup)
		me_partgroups.Put("/saveattrs", controllers.SaveAttrGroup)
		me_partgroups.Post("/addinfo", controllers.InsertMePartgroup)
		me_partgroups.Delete("/del", controllers.DeleteMePartgroup)
		me_partgroups.Put("/upinfo", controllers.UpdateMePartgroup)

		//  属性主数据
		me_attributes := admin.Party("/me_attribute")
		me_attributes.Get("/listpage", controllers.GetMeAttributeList)
		me_attributes.Get("/selectinfo", controllers.GetMeAttribute)
		me_attributes.Post("/addinfo", controllers.InsertMeAttribute)
		me_attributes.Delete("/del", controllers.DeleteMeAttribute)
		me_attributes.Put("/upinfo", controllers.UpdateMeAttribute)
		me_attributes.Get("/selectarr", controllers.SelectArrMeAttribute)
		me_attributes.Get("/selectdata", controllers.SelectDataMeAttribute)
		me_attributes.Get("/selectarrname", controllers.SelectArrMeAttributeName)
		me_attributes.Get("/download", controllers.DownloadMeAttribute)

		//  属性值主数据
		me_attribute_valsts := admin.Party("/me_attribute_valst")
		me_attribute_valsts.Get("/listpage", controllers.GetMeAttributeValstList)
		me_attribute_valsts.Get("/selectinfo", controllers.GetMeAttributeValst)
		me_attribute_valsts.Post("/addinfo", controllers.InsertMeAttributeValst)
		me_attribute_valsts.Delete("/del", controllers.DeleteMeAttributeValst)
		me_attribute_valsts.Put("/upinfo", controllers.UpdateMeAttributeValst)

		// 生产线
		worklines := admin.Party("/workline")
		worklines.Get("/listpage", controllers.GetWorklineList)
		worklines.Get("/selectinfo", controllers.GetWorkline)
		worklines.Post("/addinfo", controllers.InsertWorkline)
		worklines.Delete("/del", controllers.DeleteWorkline)
		worklines.Put("/upinfo", controllers.UpdateWorkline)
		worklines.Get("/selectarr", controllers.SelectArrWorkline)

		//  供应组数据
		me_supplygroup := admin.Party("/me_supplygroup")
		me_supplygroup.Get("/listpage", controllers.GetMeSupplyGroupList)
		me_supplygroup.Get("/selectinfo", controllers.GetMeSupplyGroup)
		me_supplygroup.Get("/selectarr", controllers.SelectArrMeSupplyGroup)
		me_supplygroup.Post("/addinfo", controllers.InsertMeSupplyGroup)
		me_supplygroup.Delete("/del", controllers.DeleteMeSupplyGroup)
		me_supplygroup.Put("/upinfo", controllers.UpdateMeSupplyGroup)

		//零件属性分配
		me_part_atcodlst := admin.Party("/me_part_atcodlst")
		me_part_atcodlst.Get("/listpage", controllers.GetMePartAtcodlstList)
		me_part_atcodlst.Get("/getmodes", controllers.SelectModelsArr)
		me_part_atcodlst.Get("/getattrcode", controllers.SelectAttrsArr)
		me_part_atcodlst.Get("/getattrval", controllers.SelectAttrVals)
		me_part_atcodlst.Post("/addinfo", controllers.SaveMePartAtcodlst)
		me_part_atcodlst.Delete("/del", controllers.DeleteMePartAtcodlst)
		me_part_atcodlst.Post("/upexcel", controllers.UpExcelMePartAtcodlst)

		//属性组
		me_attrbutegroup := admin.Party("/me_attrbutegroup")
		me_attrbutegroup.Get("/listpage", controllers.GetMeAttributeGroupList)    //分页查询
		me_attrbutegroup.Post("/addinfo", controllers.InsertMeAttributeGroup)     //添加
		me_attrbutegroup.Delete("/del", controllers.DeleteMeAttributeGroup)       //删除
		me_attrbutegroup.Put("/upinfo", controllers.UpdateMeAttributeGroup)       //更新
		me_attrbutegroup.Get("/getinfo", controllers.GetMeAttributeGroup)         //查询单条
		me_attrbutegroup.Get("/selectarr", controllers.SelectArrMeAttributeGroup) //查询未分配
		me_attrbutegroup.Get("/selectkv", controllers.SelectKVMeAttributeGroup)   //查询属性组描述，kv形式返回

		// 标准定义
		stdeftab := admin.Party("/stdeftab")
		stdeftab.Get("/listpage", controllers.GetStdeftabList)
		stdeftab.Get("/selectinfo", controllers.GetStdeftab)
		stdeftab.Post("/addinfo", controllers.InsertStdeftab)
		stdeftab.Delete("/del", controllers.DeleteStdeftab)
		stdeftab.Put("/upinfo", controllers.UpdateStdeftab)
		stdeftab.Get("/selectarr", controllers.SelectStdeftabArr)
		stdeftab.Get("/selectall", controllers.SelectStdeftabAll)

		// 零件分配
		me_partrule := admin.Party("/me_partrule")
		me_partrule.Get("/listpage", controllers.GetMePartruleList)
		me_partrule.Get("/selectinfo", controllers.GetMePartrule)
		me_partrule.Post("/addinfo", controllers.InsertMePartrule)
		me_partrule.Delete("/del", controllers.DeleteMePartrule)
		me_partrule.Put("/upinfo", controllers.UpdateMePartrule)
		me_partrule.Get("/selectarr", controllers.SelectArrMePartrule) //查询未分配
		me_partrule.Get("/download", controllers.DownloadMePartrule)

		// 版本控制
		me_part_atcodver := admin.Party("/me_part_atcodver")
		me_part_atcodver.Post("/addinfo", controllers.InsertMePartAtcodver)
		me_part_atcodver.Get("/selectinfo", controllers.GetMePartAtcodver)
		me_part_atcodver.Put("/upinfo", controllers.UpdateMePartAtcodver)

		//版本操作日志
		me_part_atcodver_trace := admin.Party("/me_part_atcodver_trace")
		me_part_atcodver_trace.Get("/listpage", controllers.GetMePartAtcodverTraceList)

		// 订单流水号
		snrtab := admin.Party("/snrtab")
		snrtab.Get("/listpage", controllers.GetSnrtabList)
		snrtab.Get("/selectinfo", controllers.GetSnrtab)
		snrtab.Post("/addinfo", controllers.InsertSnrtab)
		snrtab.Delete("/del", controllers.DeleteSnrtab)
		snrtab.Put("/upinfo", controllers.UpdateSnrtab)

		// 导入表
		scripttab := admin.Party("/scripttab")
		scripttab.Get("/listpage", controllers.GetScripttabList)
		scripttab.Get("/selectinfo", controllers.GetScripttab)
		scripttab.Post("/addinfo", controllers.InsertScripttab)
		scripttab.Delete("/del", controllers.DeleteScripttab)
		scripttab.Put("/upinfo", controllers.UpdateScripttab)
		scripttab.Post("/upexcel", controllers.UpExcelScripttab)
		scripttab.Post("/uptwoexcel", controllers.UpDoubleExcelScripttab)
		scripttab.Get("/selectarr", controllers.SelectArrScripttab)

		//查询项目id
		me_project := admin.Party("/me_project")
		me_project.Get("/selectall", controllers.SelectMeProjectAll)

		// 产品
		pln_forecast_article := admin.Party("/pln_forecast_article")
		pln_forecast_article.Get("/listpage", controllers.GetPlnForecastArticleList)
		pln_forecast_article.Get("/selectinfo", controllers.GetPlnForecastArticle)
		pln_forecast_article.Post("/addinfo", controllers.InsertPlnForecastArticle)
		pln_forecast_article.Delete("/del", controllers.DeletePlnForecastArticle)
		pln_forecast_article.Get("/selectall", controllers.SelectAllPlnForecastArticle)
		pln_forecast_article.Get("/selectfatherall", controllers.SelectFatherAllPlnForecastArticle)
		pln_forecast_article.Get("/verifyformula", controllers.VerifyFormulaPlnForecastArticle)
		pln_forecast_article.Put("/upinfo", controllers.UpdatePlnForecastArticle)

		// 核心零件
		pln_forecast_critical_item := admin.Party("/pln_forecast_critical_item")
		pln_forecast_critical_item.Get("/listpage", controllers.GetPlnForecastCriticalItemList)
		pln_forecast_critical_item.Get("/selectinfo", controllers.GetPlnForecastCriticalItem)
		pln_forecast_critical_item.Post("/addinfo", controllers.InsertPlnForecastCriticalItem)
		pln_forecast_critical_item.Get("/selectall", controllers.SelectAllPlnForecastCriticalItem)
		pln_forecast_critical_item.Delete("/del", controllers.DeletePlnForecastCriticalItem)
		pln_forecast_critical_item.Put("/upinfo", controllers.UpdatePlnForecastCriticalItem)

		// 工作日历
		pln_forecast_calendar := admin.Party("/pln_forecast_calendar")
		pln_forecast_calendar.Get("/listpage", controllers.GetPlnForecastCalendarList)
		pln_forecast_calendar.Get("/selectinfo", controllers.GetPlnForecastCalendar)
		pln_forecast_calendar.Post("/addinfo", controllers.InsertPlnForecastCalendar)
		pln_forecast_calendar.Delete("/del", controllers.DeletePlnForecastCalendar)
		pln_forecast_calendar.Put("/upinfo", controllers.UpdatePlnForecastCalendar)

		//报表计划列表
		pln_forecast_landing := admin.Party("/pln_forecast_landing")
		pln_forecast_landing.Get("/listpage", controllers.GetPlnForecastLandingList)
		pln_forecast_landing.Post("/upfile", controllers.UploadLab)
		pln_forecast_landing.Get("/parseinfo", controllers.ParseInfoPlnForecast)           //解析
		pln_forecast_landing.Get("/resetparseinfo", controllers.ResetParseInfoPlnForecast) //重新解析
		pln_forecast_landing.Get("/selectall", controllers.SelectPlnForecastAll)
		pln_forecast_landing.Delete("/del", controllers.DelPlnForecast)

		//产品计划列表
		pln_forecast_item_dailydemand := admin.Party("/pln_forecast_item_dailydemand")
		pln_forecast_item_dailydemand.Get("/listpage", controllers.GetPlnForecastItemDailydemandList)         //月统计
		pln_forecast_item_dailydemand.Get("/listdaypage", controllers.GetPlnForecastItemDailydemandByDayList) //日统计
		pln_forecast_item_dailydemand.Get("/dendrogram", controllers.GetDendrogram)
		pln_forecast_item_dailydemand.Get("/total", controllers.GetPlnForecastItemDailydemandTotal) //产品统计报表
		pln_forecast_item_dailydemand.Get("/yeartotal", controllers.TotalToCustplantnr)             //工厂统计报表
		pln_forecast_item_dailydemand.Get("/readdayexcel", controllers.ReadExcelDaydemand)          //导出日报报表
		pln_forecast_item_dailydemand.Get("/readmonthexcel", controllers.ReadExcelMonthdemand)      //导出月报报表
		pln_forecast_item_dailydemand.Get("/readexcel", controllers.ReadExcelDemand)                //导出产品统计报表
		pln_forecast_item_dailydemand.Get("/readyearexcel", controllers.ReadYearExcelDemand)        //导出工厂统计报表

		// 运算
		pln_forecast_formula := admin.Party("/pln_forecast_formula")
		pln_forecast_formula.Get("/listpage", controllers.GetPlnForecastFormulaList)
		pln_forecast_formula.Get("/selectinfo", controllers.GetPlnForecastFormula)
		pln_forecast_formula.Post("/addinfo", controllers.InsertPlnForecastFormula)
		pln_forecast_formula.Put("/upinfo", controllers.UpdatePlnForecastFormula)
		pln_forecast_formula.Delete("/del", controllers.DeletePlnForecastFormula)
		pln_forecast_formula.Get("/selectarr", controllers.SelectPlnForecastFormulaArr)

		// etl表
		etltab := admin.Party("/etltab")
		etltab.Get("/listpage", controllers.GetEtltabList)
		etltab.Get("/selectinfo", controllers.GetEtltab)
		etltab.Post("/addinfo", controllers.InsertEtltab)
		etltab.Delete("/del", controllers.DeleteEtltab)
		etltab.Put("/upinfo", controllers.UpdateEtltab)
		etltab.Get("/selectarr", controllers.SelectArrEtltab)

		//buffer管理
		buffer := admin.Party("/buffer")
		buffer.Get("/bufferlist", controllers.Bufferlist)
		buffer.Get("/testsql", controllers.TestEtltabSql)

		// database管理
		database := admin.Party("/database")
		database.Get("/listpage", controllers.Databaselist)
		database.Get("/selectinfo", controllers.GetDatabase)
		database.Post("/addinfo", controllers.InsertDatabase)
		database.Put("/upinfo", controllers.UpdateDatabase)
		database.Delete("/del", controllers.DelDatabase)

		//日模型
		daymodel := admin.Party("/daymodel")
		daymodel.Get("/listpage", controllers.GetDaymodelList)
		daymodel.Get("/selectinfo", controllers.GetDaymodel)
		daymodel.Post("/addinfo", controllers.InsertDaymodel)
		daymodel.Delete("/del", controllers.DeleteDaymodel)
		daymodel.Put("/upinfo", controllers.UpdateDaymodel)
		daymodel.Get("/selectarr", controllers.SelectArrDaymodel)

		//周模型
		weekmodel := admin.Party("/weekmodel")
		weekmodel.Get("/listpage", controllers.GetWeekmodelList)
		weekmodel.Get("/selectinfo", controllers.GetWeekmodel)
		weekmodel.Get("/selectarr", controllers.SelectArrWeekmodel)
		weekmodel.Post("/addinfo", controllers.InsertWeekmodel)
		weekmodel.Delete("/del", controllers.DeleteWeekmodel)
		weekmodel.Put("/upinfo", controllers.UpdateWeekmodel)

		//工作日历
		workcalendar := admin.Party("/workcalendar")
		workcalendar.Get("/listpage", controllers.GetWorkcalendarList)
		workcalendar.Get("/selectinfo", controllers.GetWorkcalendar)
		workcalendar.Get("/selectarr", controllers.SelectArrWorkcalendar)
		workcalendar.Post("/addinfo", controllers.InsertWorkcalendar)
		workcalendar.Delete("/del", controllers.DeleteWorkcalendar)
		workcalendar.Put("/upinfo", controllers.UpdateWorkcalendar)

		//产线特殊工作日
		workline_specialday := admin.Party("/workline_specialday")
		workline_specialday.Get("/listpage", controllers.GetWorklineSpecialdayList)
		workline_specialday.Get("/selectinfo", controllers.GetWorklineSpecialday)
		workline_specialday.Post("/addinfo", controllers.InsertWorklineSpecialday)
		workline_specialday.Delete("/del", controllers.DeleteWorklineSpecialday)
		workline_specialday.Put("/upinfo", controllers.UpdateWorklineSpecialday)

		//打印机
		printertab := admin.Party("/printertab")
		printertab.Get("/listpage", controllers.GetPrintertabList)
		printertab.Get("/selectinfo", controllers.GetPrintertab)
		printertab.Post("/addinfo", controllers.InsertPrintertab)
		printertab.Delete("/del", controllers.DeletePrintertab)
		printertab.Put("/upinfo", controllers.UpdatePrintertab)

		//打印机任务
		printheadtab := admin.Party("/printheadtab")
		printheadtab.Get("/listpage", controllers.GetPrintheadtabList)
		printheadtab.Get("/selectinfo", controllers.GetPrintheadtab)
		printheadtab.Post("/addinfo", controllers.InsertPrintheadtab)
		printheadtab.Delete("/del", controllers.DeletePrintheadtab)
		printheadtab.Put("/upinfo", controllers.UpdatePrintheadtab)

		//打印明细
		printdetailtab := admin.Party("/printdetailtab")
		printdetailtab.Get("/listpage", controllers.GetPrintdetailtabList)

		// 标准单位定义uom
		uom := admin.Party("/uom")
		uom.Get("/listpage", controllers.GetUomList)
		uom.Get("/selectinfo", controllers.GetUom)
		uom.Post("/addinfo", controllers.InsertUom)
		uom.Delete("/del", controllers.DeleteUom)
		uom.Put("/upinfo", controllers.UpdateUom)
		uom.Get("/selectarr", controllers.SelectUomArr)

		// Bom头信息
		me_bomhead := admin.Party("/me_bomhead")
		me_bomhead.Get("/listpage", controllers.GetMeBomheadList)
		me_bomhead.Get("/selectinfo", controllers.GetMeBomhead)
		me_bomhead.Post("/addinfo", controllers.InsertMeBomhead)
		me_bomhead.Delete("/del", controllers.DeleteMeBomhead)
		me_bomhead.Put("/upinfo", controllers.UpdateMeBomhead)

		// Bom明细
		me_bomlst := admin.Party("/me_bomlst")
		me_bomlst.Get("/treeinfo", controllers.GetMeBomlstList)
		me_bomlst.Post("/addinfo", controllers.InsertMeBomlst)
		me_bomlst.Get("/selectinfo", controllers.GetMeBomlst)
		me_bomlst.Delete("/del", controllers.DeleteMeBomlst)
		me_bomlst.Put("/upinfo", controllers.UpdateMeBomlst)
	}

}
